@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapcontext.c 5893 2007-01-21 17:48:37Z dan $
+ * $Id: mapcontext.c 6378 2007-07-24 04:51:14Z sdlime $
  *
  * Name:     mapcontext.c
  * Project:  MapServer
@@ -17,8 +17,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  * 
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  * 
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -26,265 +26,11 @@
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
- * DEALINGS IN THE SOFTWARE.
- **********************************************************************
- *
- * $Log$
- * Revision 1.78.2.1  2007/01/21 17:48:37  dan
- * Ignore MS_DELETE layers in msWriteMapContext() (bug 2011)
- *
- * Revision 1.78  2006/09/05 13:49:42  julien
- * WMC true/false bool and use format when formatlist not avail. bug 1723,1692
- *
- * Revision 1.77  2006/06/15 14:58:24  julien
- * Add SLD xsd and 1.1.0 reference in the root element
- *
- * Revision 1.76  2006/06/14 18:13:39  julien
- * Support WMC Min/Max scale in write mode. bug 1581
- *
- * Revision 1.75  2006/03/10 15:30:15  julien
- * Set the wms_time metadata when we have the time dimension in context1.1
- *
- * Revision 1.74  2006/03/09 21:08:54  julien
- * Remove XML header tag check
- *
- * Revision 1.73  2006/02/14 03:53:04  julien
- * Change layer server type for 1.1.0 version
- *
- * Revision 1.72  2006/02/14 03:38:47  julien
- * Update to MapContext 1.1.0, add dimensions support in context bug 1581
- *
- * Revision 1.71  2006/02/09 16:57:14  julien
- * Support SLD_BODY in Web Map Context
- *
- * Revision 1.70  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.69  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.68  2004/11/10 20:55:40  assefa
- * Do not output <StyleList> if the metadata wms_stylelist is an empty
- * string (Bug 595).
- *
- * Revision 1.67  2004/11/02 21:01:00  assefa
- * Add a 2nd optional argument to msLoadMapContext function (Bug 1023).
- *
- * Revision 1.66  2004/10/29 22:48:03  assefa
- * Use of metadata ows_schema_location (Bug 1013).
- *
- * Revision 1.65  2004/10/25 17:30:38  julien
- * Print function for OGC URLs components. msOWSPrintURLType() (Bug 944)
- *
- * Revision 1.64  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.63  2004/10/15 20:29:03  assefa
- * Add support for OGC mapcontext through mapserver cgi : Bug 946.
- *
- * Revision 1.62  2004/10/01 21:26:55  frank
- * Use msIO_ API.
- *
- * Revision 1.61  2004/09/30 13:29:56  julien
- * Fix a typo in Format encoding that print all formats in each tags.
- *
- * Revision 1.60  2004/09/23 19:18:10  julien
- * Encode all metadata and parameter printed in an XML document (Bug 802)
- *
- * Revision 1.59  2004/09/20 12:31:08  julien
- * Output parameters (SRS and DataURL) in order required by the spec. Bug 863
- *
- * Revision 1.58  2004/09/06 16:06:43  julien
- * Cleanup code to separate parsing into different functions.
- *
- * Revision 1.57  2004/08/03 23:26:24  dan
- * Cleanup OWS version tests in the code, mapwms.c (bug 799)
- *
- * Revision 1.56  2004/08/03 22:12:34  dan
- * Cleanup OWS version tests in the code, started with mapcontext.c (bug 799)
- *
- * Revision 1.55  2004/06/22 20:55:20  sean
- * Towards resolving issue 737 changed hashTableObj to a structure which contains a hashObj **items.  Changed all hash table access functions to operate on the target table by reference.  msFreeHashTable should not be used on the hashTableObj type members of mapserver structures, use msFreeHashItems instead.
- *
- * Revision 1.54  2004/04/16 19:12:31  assefa
- * Correct bug on windows when opening xml file (open it in binary mode).
- *
- * Revision 1.53  2004/04/14 05:14:54  dan
- * Added ability to pass a default value to msOWSPrintMetadataList() (bug 616)
- *
- * Revision 1.52  2004/04/14 04:54:30  dan
- * Created msOWSLookupMetadata() and added namespaces lookup in all
- * msOWSPrint*Metadata() functions. Also pass namespaces=NULL everywhere
- * that calls those functions for now to avoid breaking something just
- * before the release. (bug 615, 568)
- *
- * Revision 1.51  2003/12/23 20:40:38  julien
- * Replace legendurl, logourl, descriptionurl, dataurl and metadataurl metadata
- * by four new metadata by metadata replaced. The new metadata are called
- * legendurl_width, legendurl_height, legendurl_format, legendurl_href,
- * logourl_width, etc...
- * Old dependancy to the metadata with four value, space separated, are kept.
- *
- * Revision 1.50  2003/12/22 17:00:21  julien
- * Implement DataURL, MetadataURL and DescriptionURL (Bug 523)
- *
- * Revision 1.49  2003/07/31 16:10:34  dan
- * Enable map context stuff only if USE_OGR is set (cpl_minixml dependency)
- *
- * Revision 1.48  2003/07/15 15:22:23  assefa
- * Modify schema location url.
- *
- * Revision 1.47  2003/07/15 14:07:07  assefa
- * replace View_context by ViewContext for 1.0. support.
- *
- * Revision 1.46  2003/07/11 15:43:20  dan
- * Try to pick a supported format when current format is not supported
- *
- * Revision 1.45  2003/06/26 12:43:14  assefa
- * typo : replace printf by fprintf.
- *
- * Revision 1.44  2003/06/26 02:49:47  assefa
- * Add support for version 1.0.0
- *
- * Revision 1.43  2003/04/09 07:13:49  dan
- * Added GetContext (custom) request in WMS interface.
- * Added missing gml: namespace in 0.1.7 context output.
- *
- * Revision 1.42  2003/03/07 21:22:50  julien
- * Fix a typo in ContactFacsimileTelephone
- *
- * Revision 1.41  2003/02/21 19:19:49  julien
- * Do not put 'init=' before the srs to proj when it begin by 'AUTO:'
- *
- * Revision 1.40  2003/02/04 14:33:18  julien
- * Fix the closing tag of View_Context
- *
- * Revision 1.39  2003/01/30 22:46:32  julien
- * mv context_* metadata to wms_context_*
- *
- * Revision 1.38  2003/01/30 22:43:29  julien
- * Remove logourl heritage and customize encoding
- *
- * Revision 1.37  2003/01/30 21:17:03  julien
- * Implement the version 0.1.7
- *
- * Revision 1.36  2002/12/20 20:21:13  julien
- * wms_style__title is now set correctly
- *
- * Revision 1.35  2002/12/19 19:26:11  julien
- * Don't set the projection for each layer
- *
- * Revision 1.34  2002/12/11 18:10:39  julien
- * Remove possible WARNING inside other tags
- *
- * Revision 1.33  2002/11/29 19:28:17  julien
- * Replace ' ' by ',' in stylelist and formatlist srs
- *
- * Revision 1.32  2002/11/27 21:00:44  julien
- * Fatal error if server version is missing in load function
- *
- * Revision 1.31  2002/11/27 20:14:19  julien
- * Use the map srs if no layer srs is specify
- *
- * Revision 1.30  2002/11/26 01:37:55  dan
- * Fixed compile warnings
- *
- * Revision 1.29  2002/11/25 21:48:07  dan
- * Set map units after setting new projections in msLoadMapContext()
- *
- * Revision 1.28  2002/11/25 14:48:10  julien
- * One SRS tag with multiple SRS elements space separated
- *
- * Revision 1.27  2002/11/22 21:50:33  julien
- * Fix SRS and LegendURL for 0.1.2
- *
- * Revision 1.26  2002/11/22 17:42:52  julien
- * Support DataURL and LogoURL for 0.1.2 version
- *
- * Revision 1.25  2002/11/22 15:12:51  julien
- * Fix the seg fault temporaly fixed on 2002/11/21 17:07:34
- *
- * Revision 1.24  2002/11/21 20:39:40  julien
- * Support the wms_srs metadata and support multiple SRS tag
- *
- * Revision 1.23  2002/11/21 17:07:34  julien
- * temporaly fix a seg. fault with pszSLD2
- *
- * Revision 1.22  2002/11/21 15:54:46  julien
- * Valid empty Format and style, some chage to 0.1.2 version
- *
- * Revision 1.21  2002/11/20 23:57:55  julien
- * Remove duplicated code of style and format and create a wms_name
- *
- * Revision 1.20  2002/11/20 22:17:09  julien
- * Replace fatal error by msDebug
- *
- * Revision 1.19  2002/11/20 21:25:35  dan
- * Duh! Forgot to set the proper path for the contexts/0.1.4/context.xsd
- *
- * Revision 1.18  2002/11/20 21:22:32  dan
- * Added msOWSGetSchemasLocation() for use by both WFS and WMS Map Context
- *
- * Revision 1.17  2002/11/20 19:08:55  julien
- * Support onlineResource of version 0.1.2
- *
- * Revision 1.16  2002/11/20 17:17:21  julien
- * Support version 0.1.2 of MapContext
- * Remove warning from tags
- * Encode and decode all url
- *
- * Revision 1.15  2002/11/15 18:53:10  assefa
- * Correct test on fread cuasing problems on Windows.
- *
- * Revision 1.14  2002/11/14 18:46:14  julien
- * Include the ifdef to compile without the USE_WMS_LYR tag
- *
- * Revision 1.13  2002/11/13 16:54:23  julien
- * Change the search of the header to be flexible.
- *
- * Revision 1.12  2002/11/07 21:50:19  julien
- * Set the layer type to RASTER
- *
- * Revision 1.11  2002/11/07 21:16:45  julien
- * Fix warning in ContactInfo
- *
- * Revision 1.10  2002/11/07 15:53:14  julien
- * Put duplicated code in a single function
- * Fix many typo error
- *
- * Revision 1.9  2002/11/05 21:56:13  julien
- * Remove fatal write mistake in msSaveMapContext
- *
- * Revision 1.8  2002/11/04 20:39:17  julien
- * Change a validation to prevent a crash
- *
- * Revision 1.6  2002/11/04 20:16:09  julien
- * Change a validation to prevent a crash
- *
- * Revision 1.5  2002/10/31 21:25:55  julien
- * transform EPSG:*** to init=epsg:***
- *
- * Revision 1.4  2002/10/31 20:00:07  julien
- * transform EPSG:*** to init=epsg:***
- *
- * Revision 1.3  2002/10/31 18:57:35  sacha
- * Fill layerorder and layer.index in msLoadMapContext
- *
- * Revision 1.2  2002/10/28 22:31:09  dan
- * Added map arg to initLayer() call
- *
- * Revision 1.1  2002/10/28 20:31:20  dan
- * New support for WMS Map Context (from Julien)
- *
- * Revision 1.1  2002/10/22 20:03:57  julien
- * Add the mapcontext support
- *
- **********************************************************************/
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapcontext.c 5893 2007-01-21 17:48:37Z dan $")
+MS_CVSID("$Id: mapcontext.c 6378 2007-07-24 04:51:14Z sdlime $")
 
 #if defined(USE_WMS_LYR) && defined(USE_OGR)
 
@@ -711,7 +457,7 @@ int msLoadMapContextLayerFormat(CPLXMLNode *psFormat, layerObj *layer)
       pszValue = msLookupHashTable(&(layer->metadata), 
                                    "wms_formatlist");
 
-      papszList = split(pszValue, ',', &numformats);
+      papszList = msStringSplit(pszValue, ',', &numformats);
       for(i=0; i < numformats; i++)
       {
           if (
@@ -1145,12 +891,12 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
   layerObj *layer;
 
   /* Init new layer */
-  layer = &(map->layers[map->numlayers]);
+  layer = (GET_LAYER(map, map->numlayers));
   initLayer(layer, map);
   layer->map = (mapObj *)map;
   layer->type = MS_LAYER_RASTER;
   /* save the index */
-  map->layers[map->numlayers].index = map->numlayers;
+  GET_LAYER(map, map->numlayers)->index = map->numlayers;
   map->layerorder[map->numlayers] = map->numlayers;
   map->numlayers++;
   
@@ -1234,13 +980,13 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
   pszValue = (char*)CPLGetXMLValue(psLayer, "sld:MinScaleDenominator", NULL);
   if(pszValue != NULL)
   {
-      layer->minscale = atof(pszValue);
+      layer->minscaledenom = atof(pszValue);
   }
 
   pszValue = (char*)CPLGetXMLValue(psLayer, "sld:MaxScaleDenominator", NULL);
   if(pszValue != NULL)
   {
-      layer->maxscale = atof(pszValue);
+      layer->maxscaledenom = atof(pszValue);
   }
 
   /*  */
@@ -1760,6 +1506,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                             " id=\"%s\"", NULL);
 
   msIO_fprintf( stream, " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
+  msIO_fprintf( stream, " xmlns:ogc=\"http://www.opengis.net/ogc\"");
 
   if( nVersion >= OWS_0_1_7 && nVersion < OWS_1_0_0 )
   {
@@ -1847,7 +1594,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
 
           pszValue = msLookupHashTable(&(map->web.metadata), 
                                        "wms_keywordlist");
-          papszKeywords = split(pszValue, ',', &nKeywords);
+          papszKeywords = msStringSplit(pszValue, ',', &nKeywords);
           if(nKeywords > 0 && papszKeywords)
           {
               msIO_fprintf( stream, "    <KeywordList>\n");
@@ -1919,48 +1666,48 @@ int msWriteMapContext(mapObj *map, FILE *stream)
   /* Loop on all layer   */
   for(i=0; i<map->numlayers; i++)
   {
-      if(map->layers[i].status != MS_DELETE && map->layers[i].connectiontype == MS_WMS)
+      if(GET_LAYER(map, i)->status != MS_DELETE && GET_LAYER(map, i)->connectiontype == MS_WMS)
       {
-          if(map->layers[i].status == MS_OFF)
+          if(GET_LAYER(map, i)->status == MS_OFF)
               nValue = 1;
           else
               nValue = 0;
           msIO_fprintf(stream, "    <Layer queryable=\"%d\" hidden=\"%d\">\n", 
-                  msIsLayerQueryable(&(map->layers[i])), nValue);
+                  msIsLayerQueryable(GET_LAYER(map, i)), nValue);
 
           /*  */
           /* Server definition */
           /*  */
-          if(nVersion <= OWS_1_0_0 )
-              msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+          if(nVersion < OWS_1_0_0 )
+              msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                                        NULL, "wms_server_version", OWS_WARN,
                              "      <Server service=\"WMS\" version=\"%s\" ",
                                        "1.0.0");
           else
-              msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+              msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                                        NULL, "wms_server_version", OWS_WARN,
                           "      <Server service=\"OGC:WMS\" version=\"%s\" ",
                                        "1.0.0");
-          if(map->layers[i].name)
-              msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+          if(GET_LAYER(map, i)->name)
+              msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                                        NULL, "wms_title", OWS_NOERR, 
-                                       "title=\"%s\">\n", map->layers[i].name);
+                                       "title=\"%s\">\n", GET_LAYER(map, i)->name);
           else
           {
-              msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+              msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                                        NULL, "wms_title", OWS_NOERR, 
                                        "title=\"%s\">\n", "");
           }
 
           /* Get base url of the online resource to be the default value */
-          if(map->layers[i].connection)
-              pszValue = strdup( map->layers[i].connection );
+          if(GET_LAYER(map, i)->connection)
+              pszValue = strdup( GET_LAYER(map, i)->connection );
           else
               pszValue = strdup( "" );
           pszChar = strchr(pszValue, '?');
           if( pszChar )
               pszValue[pszChar - pszValue] = '\0';
-          if(msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+          if(msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                                       NULL, "wms_onlineresource", OWS_WARN, 
          "        <OnlineResource xlink:type=\"simple\" xlink:href=\"%s\"/>\n",
                                       pszValue) == OWS_WARN)
@@ -1973,15 +1720,15 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           /*  */
           /* Layer information */
           /*  */
-          msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+          msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                              NULL, "wms_name", OWS_WARN, 
                              "      <Name>%s</Name>\n", 
-                             map->layers[i].name);
-          msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+                             GET_LAYER(map, i)->name);
+          msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                              NULL, "wms_title", OWS_WARN, 
                              "      <Title>%s</Title>\n", 
-                             map->layers[i].name);
-          msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+                             GET_LAYER(map, i)->name);
+          msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                              NULL, "wms_abstract", OWS_NOERR, 
                              "      <Abstract>%s</Abstract>\n", 
                              NULL);
@@ -1989,7 +1736,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           /* DataURL */
           if(nVersion <= OWS_0_1_4)
           {
-              msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+              msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                                  NULL, "wms_dataurl", OWS_NOERR, 
                                  "      <DataURL>%s</DataURL>\n", 
                                  NULL);
@@ -2002,7 +1749,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
               /* Note: in version 0.1.7 the width and height are not included  */
               /* in the Context file, but they are included in the metadata for */
               /* for consistency with the URLType. */
-              msOWSPrintURLType(stream, &(map->layers[i].metadata), "MO", 
+              msOWSPrintURLType(stream, &(GET_LAYER(map, i)->metadata), "MO", 
                                 "dataurl", OWS_NOERR, NULL, "DataURL", NULL, 
                                 " width=\"%s\"", " height=\"%s\"",
                                 " format=\"%s\"", 
@@ -2016,7 +1763,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           /* The MetadataURL have a width, height, format and an URL */
           /* The metadata will be structured like this:  */
           /* "width height format url" */
-          msOWSPrintURLType(stream, &(map->layers[i].metadata), "MO", 
+          msOWSPrintURLType(stream, &(GET_LAYER(map, i)->metadata), "MO", 
                             "metadataurl", OWS_NOERR, NULL, "MetadataURL",NULL,
                             " width=\"%s\"", " height=\"%s\""," format=\"%s\"",
                             "        <OnlineResource xlink:type=\"simple\""
@@ -2025,18 +1772,18 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                             MS_TRUE, NULL, NULL, NULL, NULL, NULL, "      ");
 
           /* MinScale && MaxScale */
-          if(nVersion >= OWS_1_1_0 && map->layers[i].minscale > 0)
+          if(nVersion >= OWS_1_1_0 && GET_LAYER(map, i)->minscaledenom > 0)
               msIO_fprintf(stream, 
                "      <sld:MinScaleDenominator>%g</sld:MinScaleDenominator>\n",
-                           map->layers[i].minscale);
-          if(nVersion >= OWS_1_1_0 && map->layers[i].maxscale > 0)
+                           GET_LAYER(map, i)->minscaledenom);
+          if(nVersion >= OWS_1_1_0 && GET_LAYER(map, i)->maxscaledenom > 0)
               msIO_fprintf(stream, 
                "      <sld:MaxScaleDenominator>%g</sld:MaxScaleDenominator>\n",
-                           map->layers[i].maxscale);
+                           GET_LAYER(map, i)->maxscaledenom);
 
           /* Layer SRS */
-          pszValue = (char*)msOWSGetEPSGProj(&(map->layers[i].projection), 
-                                             &(map->layers[i].metadata),
+          pszValue = (char*)msOWSGetEPSGProj(&(GET_LAYER(map, i)->projection), 
+                                             &(GET_LAYER(map, i)->metadata),
                                              "MO", MS_FALSE);
           if(pszValue && (strcasecmp(pszValue, "(null)") != 0))
           {
@@ -2046,12 +1793,12 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           }
 
           /* Format */
-          if(msLookupHashTable(&(map->layers[i].metadata),"wms_formatlist")==NULL && 
-             msLookupHashTable(&(map->layers[i].metadata),"wms_format")==NULL)
+          if(msLookupHashTable(&(GET_LAYER(map, i)->metadata),"wms_formatlist")==NULL && 
+             msLookupHashTable(&(GET_LAYER(map, i)->metadata),"wms_format")==NULL)
           {
               pszURL = NULL;
-              if(map->layers[i].connection)
-                  pszURL = strdup( map->layers[i].connection );
+              if(GET_LAYER(map, i)->connection)
+                  pszURL = strdup( GET_LAYER(map, i)->connection );
               else
                   pszURL = strdup( "" );
               pszValue = pszURL;
@@ -2079,15 +1826,15 @@ int msWriteMapContext(mapObj *map, FILE *stream)
               char **papszFormats;
               int numFormats, nForm;
 
-              pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+              pszValue = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                            "wms_formatlist");
               if(!pszValue)
-                  pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+                  pszValue = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                                "wms_format");
-              pszCurrent = msLookupHashTable(&(map->layers[i].metadata), 
+              pszCurrent = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                              "wms_format");
 
-              papszFormats = split(pszValue, ',', &numFormats);
+              papszFormats = msStringSplit(pszValue, ',', &numFormats);
               if(numFormats > 0 && papszFormats)
               {
                   msIO_fprintf( stream, "      <FormatList>\n");
@@ -2111,14 +1858,14 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           }
           /* Style */
           /* First check the stylelist */
-          pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+          pszValue = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                        "wms_stylelist");
-          if(pszValue == NULL || strlen(trimLeft(pszValue)) < 1)
+          if(pszValue == NULL || strlen(msStringTrimLeft(pszValue)) < 1)
           {
               /* Check if the style is in the connection URL */
               pszURL = "";
-              if(map->layers[i].connection)
-                  pszURL = strdup( map->layers[i].connection );
+              if(GET_LAYER(map, i)->connection)
+                  pszURL = strdup( GET_LAYER(map, i)->connection );
               else
                   pszURL = strdup( "" );
               pszValue = pszURL;
@@ -2132,8 +1879,8 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                       pszValue[pszChar - pszValue] = '\0';
 
                   /* Check the SLD string from the URL */
-                  if(map->layers[i].connection)
-                      pszSLD2 = strdup(map->layers[i].connection);
+                  if(GET_LAYER(map, i)->connection)
+                      pszSLD2 = strdup(GET_LAYER(map, i)->connection);
                   else
                       pszSLD2 = strdup( "" );
                   if(pszSLD2)
@@ -2215,9 +1962,9 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           {
               /* If the style information is not in the connection URL, */
               /* read the metadata. */
-              pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+              pszValue = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                            "wms_stylelist");
-              pszCurrent = msLookupHashTable(&(map->layers[i].metadata), 
+              pszCurrent = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                              "wms_style");
               msIO_fprintf( stream, "      <StyleList>\n");
               /* Loop in each style in the style list */
@@ -2238,12 +1985,12 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                   /* Write SLDURL if it is in the metadata */
                   pszStyleItem = (char*)malloc(strlen(pszStyle)+10+10);
                   sprintf(pszStyleItem, "wms_style_%s_sld", pszStyle);
-                  if(msLookupHashTable(&(map->layers[i].metadata),
+                  if(msLookupHashTable(&(GET_LAYER(map, i)->metadata),
                                        pszStyleItem) != NULL)
                   {
                       msIO_fprintf(stream, "          <SLD>\n");
                       msOWSPrintEncodeMetadata(stream, 
-                                               &(map->layers[i].metadata),
+                                               &(GET_LAYER(map, i)->metadata),
                                                NULL, pszStyleItem, 
                                                OWS_NOERR, 
      "            <OnlineResource xlink:type=\"simple\" xlink:href=\"%s\"/>\n",
@@ -2255,11 +2002,11 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                   {
                       /* If the URL is not there, check for SLDBody */
                       sprintf(pszStyleItem, "wms_style_%s_sld_body", pszStyle);
-                      if(msLookupHashTable(&(map->layers[i].metadata),
+                      if(msLookupHashTable(&(GET_LAYER(map, i)->metadata),
                                            pszStyleItem) != NULL)
                       {
                           msIO_fprintf(stream, "          <SLD>\n");
-                          msOWSPrintMetadata(stream,&(map->layers[i].metadata),
+                          msOWSPrintMetadata(stream,&(GET_LAYER(map, i)->metadata),
                                              NULL, pszStyleItem, OWS_NOERR, 
                                              "            %s\n", NULL);
                           msIO_fprintf(stream, "          </SLD>\n");
@@ -2279,7 +2026,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                           sprintf(pszStyleItem, "wms_style_%s_title",pszStyle);
                           /* Title */
                           msOWSPrintEncodeMetadata(stream, 
-                                                   &(map->layers[i].metadata), 
+                                                   &(GET_LAYER(map, i)->metadata), 
                                                    NULL, pszStyleItem, 
                                                    OWS_NOERR, 
                                         "          <Title>%s</Title>\n",
@@ -2290,7 +2037,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                           pszStyleItem = (char*)malloc(strlen(pszStyle)+10+20);
                           sprintf(pszStyleItem, "style_%s_legendurl",
                                   pszStyle);
-                          msOWSPrintURLType(stream, &(map->layers[i].metadata),
+                          msOWSPrintURLType(stream, &(GET_LAYER(map, i)->metadata),
                                             "M", pszStyleItem, OWS_NOERR, NULL,
                                             "LegendURL", NULL, " width=\"%s\"",
                                             " height=\"%s\""," format=\"%s\"",
@@ -2317,9 +2064,9 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           /* Dimension element */;
           pszCurrent = NULL;
 
-          pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+          pszValue = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                        "wms_dimensionlist");
-          pszCurrent = msLookupHashTable(&(map->layers[i].metadata), 
+          pszCurrent = msLookupHashTable(&(GET_LAYER(map, i)->metadata), 
                                          "wms_dimension");
           while(pszValue != NULL)
           {
@@ -2338,7 +2085,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
               }
 
               /* From the dimension list, extract the required dimension */
-              msOWSGetDimensionInfo(&(map->layers[i]), pszDimension, 
+              msOWSGetDimensionInfo(GET_LAYER(map, i), pszDimension, 
                                     &pszDimUserValue, &pszDimUnits, 
                                     &pszDimDefault, &pszDimNearValue, 
                                     &pszDimUnitSymbol, &pszDimMultiValue);
