@@ -36,61 +36,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.46  2006/05/26 21:46:27  tamas
- * Moving layerObj.sameconnection and msCheckConnection() internal to the MYGIS data provider.
- *
- * Revision 1.45  2005/12/14 19:13:47  sdlime
- * Patched mapfile read/write/copy routines to deal with browseformat and legendformat.
- *
- * Revision 1.44  2005/11/28 04:32:42  sdlime
- * Updated msCopyLabel() to copy encoding as well.
- *
- * Revision 1.43  2005/11/28 04:22:54  sdlime
- * Removed some unnecessary and potentially harmful if statements in msCopyStyle(). The macros should be cufficient.
- *
- * Revision 1.42  2005/06/29 14:14:23  sean
- * no longer copying layer and join items, as these will be initialized on opening of layer and joining of data (bug 1403)
- *
- * Revision 1.41  2005/06/28 16:33:23  sean
- * Fixed copy of layer processing directives with use of msLayerGetProcessing and msLayerAddProcessing (bug 1399). Also caught error in the upper bounds of looping over layer joins.
- *
- * Revision 1.40  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.39  2005/04/25 06:41:55  sdlime
- * Applied Bill's newest gradient patch, more concise in the mapfile and potential to use via MapScript.
- *
- * Revision 1.38  2005/04/15 18:44:31  sdlime
- * Updated msCopyStyle to account for the various item indexes.
- *
- * Revision 1.37  2005/04/15 17:50:20  sdlime
- * Updated style copy function to handle the new gradient parameters (bug 1305).
- *
- * Revision 1.36  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.35  2005/01/26 06:14:53  sdlime
- * Added style maxwidth/minwidth read/write/copy.
- *
- * Revision 1.34  2005/01/26 05:21:19  sdlime
- * Added support for reading/writing/copying a style width.
- *
- * Revision 1.33  2004/11/15 19:16:53  sean
- * Do not copy layerinfo, wfslayerinfo, or ogrlayerinfo in msCopyLayer (bug 1055).
- *
- * Revision 1.32  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 #include "mapsymbol.h"
 
-MS_CVSID("$Id: mapcopy.c 6255 2007-07-02 14:46:52Z sdlime $")
+MS_CVSID("$Id: mapcopy.c 6434 2007-08-01 14:23:54Z dmorissette $")
 
 #include "mapcopy.h"
 
@@ -297,6 +249,12 @@ int msCopyQueryMap(queryMapObj *dst, queryMapObj *src)
 
 int msCopyLabel(labelObj *dst, labelObj *src) 
 {
+  int i;
+
+  for(i=0; i<MS_LABEL_BINDING_LENGTH; i++) {
+    MS_COPYSTRING(dst->bindings[i].item, src->bindings[i].item);
+    dst->bindings[i].index = src->bindings[i].index; /* no way to use the macros */
+  }
 
     MS_COPYSTRING(dst->font, src->font);
     MS_COPYSTELEM(type);
@@ -320,7 +278,7 @@ int msCopyLabel(labelObj *dst, labelObj *src)
     MS_COPYSTELEM(offsetx);
     MS_COPYSTELEM(offsety);
     MS_COPYSTELEM(angle);
-    MS_COPYSTELEM(autoangle);
+    MS_COPYSTELEM(autoangle);    
     MS_COPYSTELEM(autofollow);
     MS_COPYSTELEM(buffer);
     MS_COPYSTELEM(antialias);
@@ -332,6 +290,7 @@ int msCopyLabel(labelObj *dst, labelObj *src)
     MS_COPYSTELEM(mindistance);
     MS_COPYSTELEM(partials);
     MS_COPYSTELEM(force);
+    MS_COPYSTELEM(priority);
 
     MS_COPYSTRING(dst->encoding, src->encoding);
 
@@ -364,8 +323,8 @@ int msCopyWeb(webObj *dst, webObj *src, mapObj *map)
 
     MS_COPYRECT(&(dst->extent), &(src->extent));
 
-    MS_COPYSTELEM(minscale);
-    MS_COPYSTELEM(maxscale);
+    MS_COPYSTELEM(minscaledenom);
+    MS_COPYSTELEM(maxscaledenom);
     MS_COPYSTRING(dst->mintemplate, src->mintemplate);
     MS_COPYSTRING(dst->maxtemplate, src->maxtemplate);
 
@@ -390,6 +349,12 @@ int msCopyWeb(webObj *dst, webObj *src, mapObj *map)
 
 int msCopyStyle(styleObj *dst, styleObj *src) 
 {
+  int i;
+
+  for(i=0; i<MS_STYLE_BINDING_LENGTH; i++) {
+    MS_COPYSTRING(dst->bindings[i].item, src->bindings[i].item);
+    dst->bindings[i].index = src->bindings[i].index; /* no way to use the macros */
+  }
 
     MS_COPYCOLOR(&(dst->color), &(src->color));
     MS_COPYCOLOR(&(dst->outlinecolor),&(src->outlinecolor));
@@ -410,19 +375,16 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYSTELEM(offsetx);
     MS_COPYSTELEM(offsety);
     MS_COPYSTELEM(antialias);
-    MS_COPYSTELEM(isachild);
     MS_COPYSTELEM(angle);
     MS_COPYSTELEM(minvalue);
     MS_COPYSTELEM(maxvalue);
+    MS_COPYSTELEM(opacity);
 
-    MS_COPYSTRING(dst->angleitem, src->angleitem);
-    MS_COPYSTRING(dst->sizeitem, src->sizeitem);
     MS_COPYSTRING(dst->rangeitem, src->rangeitem);
-
-    MS_COPYSTELEM(angleitemindex);
-    MS_COPYSTELEM(sizeitemindex);
     MS_COPYSTELEM(rangeitemindex);
 
+    /* TODO: add copy for bindings */
+
     return MS_SUCCESS;
 }
 
@@ -444,13 +406,31 @@ int msCopyClass(classObj *dst, classObj *src, layerObj *layer)
     }
 
     MS_COPYSTELEM(status);
-    MS_COPYSTELEM(numstyles);
 
-    for (i = 0; i < dst->numstyles; i++) {
-        if (msCopyStyle(&(dst->styles[i]), &(src->styles[i])) != MS_SUCCESS) {
+    /* free any previous styles on the dst layer*/
+    for(i=0;i<dst->numstyles;i++) { /* each style     */
+      if (dst->styles[i]!=NULL) {
+    	if( freeStyle(dst->styles[i]) == MS_SUCCESS ) {
+          msFree(dst->styles[i]);
+	}
+      }
+    }
+    msFree(dst->styles);
+    dst->numstyles = 0;
+
+    for (i = 0; i < src->numstyles; i++) {
+        if (msGrowClassStyles(dst) == NULL)
+            return MS_FAILURE;
+        if (initStyle(dst->styles[i]) != MS_SUCCESS) {
+            msSetError(MS_MEMERR, "Failed to init style.", "msCopyClass()");
+            return MS_FAILURE;
+        }
+        if (msCopyStyle(dst->styles[i], src->styles[i]) != MS_SUCCESS) {
             msSetError(MS_MEMERR, "Failed to copy style.", "msCopyClass()");
             return MS_FAILURE;
         }
+
+        dst->numstyles++;
     }
 
     if (msCopyLabel(&(dst->label), &(src->label)) != MS_SUCCESS) {
@@ -479,8 +459,8 @@ int msCopyClass(classObj *dst, classObj *src, layerObj *layer)
         msCopyHashTable(&(dst->metadata), &(src->metadata));
     }
 
-    MS_COPYSTELEM(minscale);
-    MS_COPYSTELEM(maxscale);
+    MS_COPYSTELEM(minscaledenom);
+    MS_COPYSTELEM(maxscaledenom);
     MS_COPYSTELEM(layer);
     MS_COPYSTELEM(debug);
 
@@ -537,13 +517,13 @@ int msCopyMarkerCacheMember(markerCacheMemberObj *dst,
 }
 
 /***********************************************************************
- * msCopyLabelCache()                                                  *
+ * msCopyLabelCacheSlot()                                                  *
  **********************************************************************/
 
-int msCopyLabelCache(labelCacheObj *dst, labelCacheObj *src) 
+int msCopyLabelCacheSlot(labelCacheSlotObj *dst, labelCacheSlotObj *src) 
 {
     int i;
-    MS_COPYSTELEM(numlabels);
+
     for (i = 0; i < dst->numlabels; i++) {
         msCopyLabelCacheMember(&(dst->labels[i]), &(src->labels[i]));
     }
@@ -552,12 +532,27 @@ int msCopyLabelCache(labelCacheObj *dst, labelCacheObj *src)
     for (i = 0; i < dst->nummarkers; i++) {
         msCopyMarkerCacheMember(&(dst->markers[i]), &(src->markers[i]));
     }
-
     MS_COPYSTELEM(markercachesize);
 
     return MS_SUCCESS;
 }
 
+/***********************************************************************
+ * msCopyLabelCache()                                                  *
+ **********************************************************************/
+
+int msCopyLabelCache(labelCacheObj *dst, labelCacheObj *src) 
+{
+    int p;
+    MS_COPYSTELEM(numlabels);
+
+    for (p=0; p<MS_MAX_LABEL_PRIORITY; p++) {
+        msCopyLabelCacheSlot(&(dst->slots[p]), &(src->slots[p]));
+    }
+
+    return MS_SUCCESS;
+}
+
 /***********************************************************************
  * msCopyMarkerCacheMember()                                           *
  *                                                                     *
@@ -726,26 +721,22 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     MS_COPYSTRING(dst->classitem, src->classitem);
 
     MS_COPYSTELEM(classitemindex);
-    MS_COPYSTELEM(numclasses);
 
-    for (i = 0; i < dst->numclasses; i++) {
+    for (i = 0; i < src->numclasses; i++) {
+        if (msGrowLayerClasses(dst) == NULL)
+            return MS_FAILURE;
 #ifndef __cplusplus
-        initClass(&(dst->class[i]));
-
-        return_value = msCopyClass(&(dst->class[i]), &(src->class[i]), dst);
+        initClass(dst->class[i]);
+        return_value = msCopyClass(dst->class[i], src->class[i], dst);
+#else
+        initClass(dst->_class[i]);
+        return_value = msCopyClass(dst->_class[i], src->_class[i], dst);
+#endif
         if (return_value != MS_SUCCESS) {
             msSetError(MS_MEMERR, "Failed to copy class.", "msCopyLayer()");
             return MS_FAILURE;
         }
-#else
-        initClass(&(dst->_class[i]));
-
-        return_value = msCopyClass(&(dst->_class[i]), &(src->_class[i]), dst);
-        if (return_value != MS_SUCCESS) {
-            msSetError(MS_MEMERR, "Failed to copy _class.", "msCopyLayer()");
-            return MS_FAILURE;
-        }
-#endif
+        dst->numclasses++;
     }
     MS_COPYSTRING(dst->header, src->header);
     MS_COPYSTRING(dst->footer, src->footer);
@@ -764,13 +755,13 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     MS_COPYSTELEM(annotate);
     MS_COPYSTELEM(tolerance);
     MS_COPYSTELEM(toleranceunits);
-    MS_COPYSTELEM(symbolscale);
+    MS_COPYSTELEM(symbolscaledenom);
     MS_COPYSTELEM(scalefactor);
-    MS_COPYSTELEM(minscale);
-    MS_COPYSTELEM(maxscale);
+    MS_COPYSTELEM(minscaledenom);
+    MS_COPYSTELEM(maxscaledenom);
 
-    MS_COPYSTELEM(labelminscale);
-    MS_COPYSTELEM(labelmaxscale);
+    MS_COPYSTELEM(labelminscaledenom);
+    MS_COPYSTELEM(labelmaxscaledenom);
 
     MS_COPYSTELEM(sizeunits);
     MS_COPYSTELEM(maxfeatures);
@@ -782,12 +773,7 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     MS_COPYSTELEM(postlabelcache); 
 
     MS_COPYSTRING(dst->labelitem, src->labelitem);
-    MS_COPYSTRING(dst->labelsizeitem, src->labelsizeitem);
-    MS_COPYSTRING(dst->labelangleitem, src->labelangleitem);
-
     MS_COPYSTELEM(labelitemindex);
-    MS_COPYSTELEM(labelsizeitemindex);
-    MS_COPYSTELEM(labelangleitemindex);
 
     MS_COPYSTRING(dst->tileitem, src->tileitem);
     MS_COPYSTELEM(tileitemindex);
@@ -811,7 +797,10 @@ int msCopyLayer(layerObj *dst, layerObj *src)
 
     MS_COPYSTRING(dst->connection, src->connection);
     MS_COPYSTELEM(connectiontype);
-    
+
+    MS_COPYSTRING(dst->plugin_library, src->plugin_library);
+    MS_COPYSTRING(dst->plugin_library_original, src->plugin_library_original);
+        
     /* Do not copy *layerinfo, items, or iteminfo. these are all initialized
        when the copied layer is opened */
 
@@ -834,7 +823,7 @@ int msCopyLayer(layerObj *dst, layerObj *src)
         msCopyHashTable(&(dst->metadata), &(src->metadata));
     }
 
-    MS_COPYSTELEM(transparency);
+    MS_COPYSTELEM(opacity);
     MS_COPYSTELEM(dump);
     MS_COPYSTELEM(debug);
 
@@ -876,16 +865,18 @@ int msCopyMap(mapObj *dst, mapObj *src)
     MS_COPYSTELEM(status);
     MS_COPYSTELEM(height);
     MS_COPYSTELEM(width);
-    MS_COPYSTELEM(numlayers);
 
-    for (i = 0; i < dst->numlayers; i++) {
-        initLayer(&(dst->layers[i]), dst);
+    for (i = 0; i < src->numlayers; i++) {
+        if (msGrowMapLayers(dst) == NULL)
+            return MS_FAILURE;
+        initLayer((GET_LAYER(dst, i)), dst);
 
-        return_value = msCopyLayer(&(dst->layers[i]), &(src->layers[i]));
+        return_value = msCopyLayer((GET_LAYER(dst, i)), (GET_LAYER(src, i)));
         if (return_value != MS_SUCCESS) {
             msSetError(MS_MEMERR, "Failed to copy layer.", "msCopyMap()");
             return MS_FAILURE;
         }
+        dst->numlayers++;
     }
     
     return_value = msCopyFontSet(&(dst->fontset), &(src->fontset), dst);
@@ -909,7 +900,7 @@ int msCopyMap(mapObj *dst, mapObj *src)
 
     MS_COPYSTELEM(cellsize);
     MS_COPYSTELEM(units);
-    MS_COPYSTELEM(scale);
+    MS_COPYSTELEM(scaledenom);
     MS_COPYSTELEM(resolution);
     MS_COPYSTRING(dst->shapepath, src->shapepath); 
     MS_COPYSTRING(dst->mappath, src->mappath); 
