@@ -25,130 +25,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.48.2.1  2007/02/05 05:56:19  frank
- * Fix support for OFFSITE for simple greyscale rasters (bug 2024).
- *
- * Revision 1.48  2006/02/22 03:52:04  frank
- * Incorporate range coloring support for rasters (bug 1673)
- *
- * Revision 1.47  2005/11/24 16:16:41  frank
- * fixed raster crash with some grass rasters (bug 1541)
- *
- * Revision 1.46  2005/08/25 22:01:50  frank
- * fix problem with ungeorerenced files (eg augsignals)
- *
- * Revision 1.45  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.44  2005/05/06 17:29:30  sdlime
- * Removed debuging statement from last fix.
- *
- * Revision 1.43  2005/05/06 17:19:05  frank
- * bug 985/1015 - dont classify rasters without expressions
- *
- * Revision 1.42  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.41  2005/01/23 20:40:23  frank
- * LoadGDALImage() should return int, not CPLErr.
- *
- * Revision 1.40  2005/01/19 21:20:13  frank
- * improve autoscaling - add small amount to scaling range - bug 1168
- *
- * Revision 1.39  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.38  2004/11/15 21:39:15  frank
- * fix last fix: GDALPrintPointer to CPLPrintPointer
- *
- * Revision 1.37  2004/11/15 21:28:48  frank
- * Use GDALPrintPointer() to avoid pointer conversion errors on win32. Bug 722.
- *
- * Revision 1.36  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.35  2004/10/21 02:23:24  frank
- * updated header license a bit, and added MS_CVSID
- *
- * Revision 1.34  2004/10/18 14:49:12  frank
- * implemented msAlphaBlend
- *
- * Revision 1.33  2004/10/18 13:05:14  frank
- * Fixed typos in alpha handling cleanup.
- *
- * Revision 1.32  2004/10/17 03:39:03  frank
- * added greyscale+alpha support - bug 965
- *
- * Revision 1.31  2004/09/29 17:12:13  frank
- * fixed casting issues to avoid warnings
- *
- * Revision 1.30  2004/09/03 14:24:07  frank
- * %p cannot be encoded and decoded properly on win32
- *
- * Revision 1.29  2004/05/12 19:40:19  frank
- * Fixed my last fix ... damn it.
- *
- * Revision 1.28  2004/05/11 22:35:06  frank
- * Avoid data init warning.
- *
- * Revision 1.27  2004/05/11 22:33:27  frank
- * bug 493: fixed last fix, memory corruption in some cases
- *
- * Revision 1.26  2004/05/05 04:14:39  frank
- * Fixed problem with computing dst_xsize and dst_ysize for rendering image.
- * In some subtle cases they would be one less than made sense.
- * http://mapserver.gis.umn.edu/bugs/show_bug.cgi?id=493
- *
- * Revision 1.25  2004/04/09 03:03:58  frank
- * improvement for msGetGDALBandList()
- *
- * Revision 1.24  2004/04/08 17:25:25  frank
- * added msGetGDALBandList()
- *
- * Revision 1.23  2004/03/11 23:03:56  assefa
- * Correct bug in a loop in function msDrawRasterLayerGDAL_16BitClassification
- *
- * Revision 1.22  2004/03/08 17:50:22  frank
- * completed 16bit classified support
- *
- * Revision 1.21  2004/03/05 23:18:52  frank
- * Added small TODO note.
- *
- * Revision 1.20  2004/03/05 22:58:04  frank
- * preliminary working implementation of msDrawRasterLayerGDAL_16BitClassification
- *
- * Revision 1.19  2004/03/05 19:55:10  frank
- * Dont call GDALDatasetRasterIO() in pre 1.2.0 GDAL builds
- *
- * Revision 1.18  2004/03/05 05:57:04  frank
- * support multi-band rawmode output
- *
- * Revision 1.17  2004/03/04 20:08:28  frank
- * added IMAGEMODE_BYTE (raw mode)
- *
- * Revision 1.16  2004/02/05 05:46:09  frank
- * don't call msOWSGetLayerExtent without OWS services being enabled
- *
- * Revision 1.15  2004/02/02 17:15:32  frank
- * class pens being reused inappropriately - bug 504
- *
- * Revision 1.14  2004/01/26 15:20:11  frank
- * added msGetGDALGeoTransform
- *
- * Revision 1.13  2004/01/15 19:49:23  frank
- * ensure geotransform is set on failure of GetGeotransform
- */
+ *****************************************************************************/
 
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapdrawgdal.c 5900 2007-02-05 05:56:41Z frank $")
+MS_CVSID("$Id: mapdrawgdal.c 6403 2007-07-27 16:18:41Z warmerdam $")
 
 extern int InvGeoTransform( double *gt_in, double *gt_out );
 
@@ -261,7 +145,20 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
                                               "COLOR_MATCH_THRESHOLD" )));
       }
   }
+#ifdef USE_AGG
+  else if( MS_RENDERER_AGG(image->format) )
+  {
+      gdImg = image->img.gd;
 
+      truecolor = gdImageTrueColor( gdImg );
+      if( CSLFetchNameValue( layer->processing, 
+                             "COLOR_MATCH_THRESHOLD" ) != NULL )
+      {
+          cmt = MAX(0,atoi(CSLFetchNameValue( layer->processing, 
+                                              "COLOR_MATCH_THRESHOLD" )));
+      }
+  }
+#endif
   src_xsize = GDALGetRasterXSize( hDS );
   src_ysize = GDALGetRasterYSize( hDS );
 
@@ -420,17 +317,17 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
       int s;
 
       /* change colour based on colour range? */
-      for(s=0; s<layer->class[i].numstyles; s++)
+      for(s=0; s<layer->class[i]->numstyles; s++)
       {
-          if( MS_VALID_COLOR(layer->class[i].styles[s].mincolor)
-              && MS_VALID_COLOR(layer->class[i].styles[s].maxcolor) )
+          if( MS_VALID_COLOR(layer->class[i]->styles[s]->mincolor)
+              && MS_VALID_COLOR(layer->class[i]->styles[s]->maxcolor) )
           {
               classified = TRUE;
               break;
           }
       }
       
-      if( layer->class[i].expression.string != NULL )
+      if( layer->class[i]->expression.string != NULL )
       {
           classified = TRUE;
           break;
@@ -538,8 +435,12 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   if( gdImg && !truecolor )
   {
       int iClass;
+      int iStyle;
       for( iClass = 0; iClass < layer->numclasses; iClass++ )
-          layer->class[iClass].styles[0].color.pen = MS_PEN_UNSET;
+      {
+          for (iStyle=0; iStyle<layer->class[iClass]->numstyles; iStyle++)
+            layer->class[iClass]->styles[iStyle]->color.pen = MS_PEN_UNSET;
+      }
   }
 
   /*
@@ -679,21 +580,21 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
                 /* change colour based on colour range?  Currently we 
                    only address the greyscale case properly. */
 
-                for(s=0; s<layer->class[c].numstyles; s++)
+                for(s=0; s<layer->class[c]->numstyles; s++)
                 {
-                    if( MS_VALID_COLOR(layer->class[c].styles[s].mincolor)
-                        && MS_VALID_COLOR(layer->class[c].styles[s].maxcolor) )
-                        msValueToRange(&layer->class[c].styles[s],
+                    if( MS_VALID_COLOR(layer->class[c]->styles[s]->mincolor)
+                        && MS_VALID_COLOR(layer->class[c]->styles[s]->maxcolor) )
+                        msValueToRange(layer->class[c]->styles[s],
                                        sEntry.c1 );
                 }
 
-                RESOLVE_PEN_GD(gdImg, layer->class[c].styles[0].color);
-                if( MS_TRANSPARENT_COLOR(layer->class[c].styles[0].color) )
+                RESOLVE_PEN_GD(gdImg, layer->class[c]->styles[0]->color);
+                if( MS_TRANSPARENT_COLOR(layer->class[c]->styles[0]->color) )
                     cmap[i] = -1;
-                else if( MS_VALID_COLOR(layer->class[c].styles[0].color))
+                else if( MS_VALID_COLOR(layer->class[c]->styles[0]->color))
                 {
                     /* use class color */
-                    cmap[i] = layer->class[c].styles[0].color.pen;
+                    cmap[i] = layer->class[c]->styles[0]->color.pen;
                 }
                 else /* Use raster color */
                     cmap[i] = msAddColorGD(map, gdImg, cmt,
@@ -1067,6 +968,265 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   return 0;
 }
 
+/************************************************************************/
+/*                          ParseDefaultLUT()                           */
+/************************************************************************/
+
+static int ParseDefaultLUT( const char *lut_def, GByte *lut )
+
+{
+    const char *lut_read;
+    int last_in=0, last_out=0, all_done=FALSE;
+
+/* -------------------------------------------------------------------- */
+/*      Parse definition, applying to lut on the fly.                   */
+/* -------------------------------------------------------------------- */
+    lut_read = lut_def;
+    while( !all_done )
+    {
+        int this_in=0, this_out=0;
+        int lut_i;
+
+        while( isspace(*lut_read) )
+            lut_read++;
+
+        /* if we are at end, assum 255:255 */
+        if( *lut_read == '\0' )
+        {
+            all_done = TRUE;
+            if ( last_in != 255 ){
+            	this_in = 255;
+            	this_out = 255;
+            }
+        }
+
+        /* otherwise read "in:out", and skip past */
+        else
+        {
+            this_in = atoi(lut_read);
+            while( *lut_read != ':' && *lut_read )
+                lut_read++;
+            if( *lut_read == ':' )
+                lut_read++;
+            while( isspace(*lut_read) )
+                lut_read++;
+            this_out = atoi(lut_read);
+            while( *lut_read && *lut_read != ',' && *lut_read != '\n' )
+                lut_read++;
+            if( *lut_read == ',' || *lut_read == '\n' )
+                lut_read++;
+            while( isspace(*lut_read) )
+                lut_read++;
+        }
+
+        this_in = MAX(0,MIN(255,this_in));
+        this_out = MAX(0,MIN(255,this_out));
+
+        /* apply linear values from last in:out to this in:out */
+        for( lut_i = last_in; lut_i <= this_in; lut_i++ )
+        {
+            double ratio;
+
+            if( last_in == this_in )
+                ratio = 1.0;
+            else
+                ratio = (lut_i - last_in) / (double) (this_in - last_in);
+            
+            lut[lut_i] = (int) 
+                floor(((1.0 - ratio)*last_out + ratio*this_out) + 0.5);
+        }
+        
+        last_in = this_in;
+        last_out = this_out;
+    }
+
+    return 0;
+}
+
+/************************************************************************/
+/*                          LutFromGimpLine()                           */
+/************************************************************************/
+
+static int LutFromGimpLine( char *lut_line, GByte *lut )
+
+{
+    char wrkLUTDef[1000];
+    int  i, count = 0;
+    char **tokens;
+
+    /* cleanup white space at end of line (DOS LF, etc) */
+    i = strlen(lut_line) - 1;
+    while( i > 0 && isspace(lut_line[i]) )
+        lut_line[i--] = '\0';
+
+    while( *lut_line == 10 || *lut_line == 13 )
+        lut_line++;
+    
+    /* tokenize line */
+    tokens = CSLTokenizeString( lut_line );
+    if( CSLCount(tokens) != 17 * 2 )
+    {
+        CSLDestroy( tokens );
+        msSetError(MS_MISCERR, 
+                   "GIMP curve file appears corrupt.", 
+                   "LutFromGimpLine()" );
+        return -1;
+    }
+
+    /* Convert to our own format */
+    wrkLUTDef[0] = '\0';
+    for( i = 0; i < 17; i++ )
+    {
+        if( atoi(tokens[i*2]) >= 0 )
+        {
+            if( count++ > 0 )
+                strcat( wrkLUTDef, "," );
+
+            sprintf( wrkLUTDef + strlen(wrkLUTDef), "%s:%s",
+                     tokens[i*2], tokens[i*2+1] );
+        }
+    }
+
+    CSLDestroy( tokens );
+
+    return ParseDefaultLUT( wrkLUTDef, lut );
+}
+
+/************************************************************************/
+/*                            ParseGimpLUT()                            */
+/*                                                                      */
+/*      Parse a Gimp style LUT.                                         */
+/************************************************************************/
+
+static int ParseGimpLUT( const char *lut_def, GByte *lut, int iColorIndex )
+
+{
+    int i;
+    GByte lutValue[256];
+    GByte lutColorBand[256];
+    char **lines = 
+        CSLTokenizeStringComplex( lut_def, "\n", FALSE, FALSE );
+
+    if( !EQUALN(lines[0],"# GIMP Curves File",18) 
+        || CSLCount(lines) < 6 )
+    {
+        msSetError(MS_MISCERR, 
+                   "GIMP curve file appears corrupt.", 
+                   "ParseGimpLUT()" );
+        return -1;
+    }
+
+    /*
+     * Convert the overall curve, and the color band specific curve into LUTs.
+     */
+    if( LutFromGimpLine( lines[1], lutValue ) != 0 
+        || LutFromGimpLine( lines[iColorIndex + 1], lutColorBand ) != 0 )
+    {
+        CSLDestroy( lines );
+        return -1;
+    }
+    CSLDestroy( lines );
+
+    /*
+     * Compose the two luts as if the raw value passed through the color band
+     * specific lut, and then the general value lut.  Usually one or the
+     * other will be the identity mapping, but not always.
+     */
+    
+    for( i = 0; i < 256; i++ )
+    {
+        lut[i] = lutValue[lutColorBand[i]];
+    }
+    
+    return 0;
+}
+
+/************************************************************************/
+/*                              ApplyLUT()                              */
+/*                                                                      */
+/*      Apply a LUT according to RFC 21.                                */
+/************************************************************************/
+
+static int ApplyLUT( int iColorIndex, layerObj *layer, 
+                     GByte *buffer, int buf_xsize, int buf_ysize )
+
+{
+    const char *lut_def;
+    char key[20], lut_def_fromfile[2500];
+    GByte lut[256];
+    int   err, i;
+
+/* -------------------------------------------------------------------- */
+/*      Get lut specifier from processing directives.  Do nothing if    */
+/*      none are found.                                                 */
+/* -------------------------------------------------------------------- */
+    sprintf( key, "LUT_%d", iColorIndex );
+    lut_def = msLayerGetProcessingKey( layer, key );
+    if( lut_def == NULL )
+        lut_def = msLayerGetProcessingKey( layer, "LUT" );
+    if( lut_def == NULL )
+        return 0;
+
+/* -------------------------------------------------------------------- */
+/*      Does this look like a file?  If so, read it into memory.        */
+/* -------------------------------------------------------------------- */
+    if( strspn(lut_def,"0123456789:, ") != strlen(lut_def) )
+    {
+        FILE *fp;
+        char path[MS_MAXPATHLEN];
+        int len;
+        msBuildPath(path, layer->map->mappath, lut_def);
+        fp = fopen( path, "rb" );
+        if( fp == NULL )
+        {
+            msSetError(MS_IOERR, 
+                       "Failed to open LUT file '%s'.",
+                       path,
+                       "drawGDAL()");
+            return -1;
+        }
+        
+        len = fread( lut_def_fromfile, 1, sizeof(lut_def_fromfile), fp );
+        fclose( fp );
+
+        if( len == sizeof(lut_def_fromfile) )
+        {
+            msSetError(MS_IOERR, 
+                       "LUT definition from file %s longer than maximum buffer size (%d bytes).",
+                       path, sizeof(lut_def_fromfile),
+                       "drawGDAL()");
+            return -1;
+        }
+        
+        lut_def_fromfile[len] = '\0';
+
+        lut_def = lut_def_fromfile;
+    } 
+
+/* -------------------------------------------------------------------- */
+/*      Parse the LUT description.                                      */
+/* -------------------------------------------------------------------- */
+    if( EQUALN(lut_def,"# GIMP",6) )
+    {
+        err = ParseGimpLUT( lut_def, lut, iColorIndex );
+    }
+    else
+    {
+        err = ParseDefaultLUT( lut_def, lut );
+    }
+
+    if( err != 0 )
+        return err;
+
+/* -------------------------------------------------------------------- */
+/*      Apply LUT.                                                      */
+/* -------------------------------------------------------------------- */
+    for( i = buf_xsize * buf_ysize - 1; i >= 0; i-- )
+        buffer[i] = lut[buffer[i]];
+
+    return 0;
+}
+
 /************************************************************************/
 /*                           LoadGDALImage()                            */
 /*                                                                      */
@@ -1141,7 +1301,10 @@ LoadGDALImage( GDALRasterBandH hBand, int iColorIndex,  layerObj *layer,
                         CPLGetLastErrorMsg(), "drawGDAL()" );
 
         if( eErr == CE_None )
-            return 0;
+        {
+            return ApplyLUT( iColorIndex, layer, 
+                             pabyBuffer, dst_xsize, dst_ysize );;
+        }
         else
             return -1;
     }
@@ -1218,7 +1381,11 @@ LoadGDALImage( GDALRasterBandH hBand, int iColorIndex,  layerObj *layer,
                  "unclassified scaled data.\n",
                  layer->name, dfNoDataValue );
 
-    return 0;
+/* -------------------------------------------------------------------- */
+/*      Apply LUT if there is one.                                      */
+/* -------------------------------------------------------------------- */
+    return ApplyLUT( iColorIndex, layer, 
+                             pabyBuffer, dst_xsize, dst_ysize );;
 }
 
 /************************************************************************/
@@ -1399,7 +1566,9 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
                            double *padfGeoTransform )
 
 {
+#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR)
     rectObj  rect;
+#endif
 
 /* -------------------------------------------------------------------- */
 /*      some GDAL drivers (ie. GIF) don't set geotransform on failure.  */
@@ -1816,20 +1985,20 @@ msDrawRasterLayerGDAL_16BitClassification(
             int s;
 
             /* change colour based on colour range? */
-            for(s=0; s<layer->class[c].numstyles; s++)
+            for(s=0; s<layer->class[c]->numstyles; s++)
             {
-                if( MS_VALID_COLOR(layer->class[c].styles[s].mincolor)
-                    && MS_VALID_COLOR(layer->class[c].styles[s].maxcolor) )
-                    msValueToRange(&layer->class[c].styles[s],dfOriginalValue);
+                if( MS_VALID_COLOR(layer->class[c]->styles[s]->mincolor)
+                    && MS_VALID_COLOR(layer->class[c]->styles[s]->maxcolor) )
+                    msValueToRange(layer->class[c]->styles[s],dfOriginalValue);
             }
 
-            RESOLVE_PEN_GD(gdImg, layer->class[c].styles[0].color);
-            if( MS_TRANSPARENT_COLOR(layer->class[c].styles[0].color) )
+            RESOLVE_PEN_GD(gdImg, layer->class[c]->styles[0]->color);
+            if( MS_TRANSPARENT_COLOR(layer->class[c]->styles[0]->color) )
                 cmap[i] = -1;
-            else if( MS_VALID_COLOR(layer->class[c].styles[0].color))
+            else if( MS_VALID_COLOR(layer->class[c]->styles[0]->color))
             {
                 /* use class color */
-                cmap[i] = layer->class[c].styles[0].color.pen;
+                cmap[i] = layer->class[c]->styles[0]->color.pen;
             }
         }
     }
