@@ -24,50 +24,9 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.29  2006/08/29 19:34:59  frank
- * Fix C++'ism (declare values after first statement in block).
- *
- * Revision 1.28  2006/08/29 19:23:39  frank
- * fixed syntax error in mapimagemap.c - affects vc++
- *
- * Revision 1.27  2006/08/29 02:51:24  sdlime
- * Applied patch supplied by Vilson Farias for extra commas with imagemap output (bug 770).
- *
- * Revision 1.26  2006/02/18 21:24:42  hobu
- * strlen(img->img.imagemap) is a size_t on osx.  Cast to
- * int on os x.
- *
- * Revision 1.25  2006/01/25 16:11:34  dan
- * Prevent systematic buffer overflow in imagemap code when vsnprintf()
- * is not available (bug 1613)
- *
- * Revision 1.24  2006/01/16 20:21:18  sdlime
- * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
- *
- * Revision 1.23  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.22  2005/05/19 04:09:34  sdlime
- * Removed the LINE_VERT_THRESHOLD test (bug 564) from PDF/SWF/SVG/imagemap drivers.
- *
- * Revision 1.21  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.20  2005/02/02 19:36:12  frank
- * Preliminary implementation of MS_SYMBOL_VECTOR in msDrawMarkerSymbolIM
- *
- * Revision 1.19  2004/11/23 17:09:05  frank
- * check if we have vsnprintf before using it
- *
- * Revision 1.18  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "dxfcolor.h"
  
 #include <stdarg.h>
@@ -78,7 +37,7 @@
 #include <io.h>
 #endif
 
-MS_CVSID("$Id: mapimagemap.c 5713 2006-08-29 19:34:59Z frank $")
+MS_CVSID("$Id: mapimagemap.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 #define MYDEBUG 0
 #define DEBUG_IF if (MYDEBUG > 2) 
@@ -639,7 +598,7 @@ void msCircleDrawLineSymbolIM(symbolSetObj *symbolset, imageObj* img, pointObj *
   if(style->color.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->color));
   if(style->outlinecolor.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->outlinecolor));
   
-  symbol = &(symbolset->symbol[style->symbol]);
+  symbol = symbolset->symbol[style->symbol];
   bc = style->backgroundcolor.pen;
   fc = style->color.pen;
   if(fc==-1) fc = style->outlinecolor.pen;
@@ -738,12 +697,12 @@ void msCircleDrawLineSymbolIM(symbolSetObj *symbolset, imageObj* img, pointObj *
     break;
   }  
 
-  if(symbol->stylelength > 0) {
+  if(symbol->patternlength > 0) {
     int k=0, sc;
    
     sc = fc; // start with foreground color
-    for(i=0; i<symbol->stylelength; i++) {      
-      for(j=0; j<symbol->style[i]; j++) {
+    for(i=0; i<symbol->patternlength; i++) {      
+      for(j=0; j<symbol->pattern[i]; j++) {
 	styleDashed[k] = sc;
 	k++;
       } 
@@ -796,7 +755,7 @@ DEBUG_IF printf("msCircleDrawShadeSymbolIM<BR>\n");
   if(style->color.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->color));
   if(style->outlinecolor.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->outlinecolor));
 
-  symbol = &(symbolset->symbol[style->symbol]);
+  symbol = symbolset->symbol[style->symbol];
   bc = style->backgroundcolor.pen;
   fc = style->color.pen;
   oc = style->outlinecolor.pen;
@@ -1015,7 +974,7 @@ DEBUG_IF printf("msDrawMarkerSymbolIM\n<BR>");
 /* if(style->color.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->color)); */
 /* if(style->outlinecolor.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->outlinecolor)); */
 
-  symbol = &(symbolset->symbol[style->symbol]);
+  symbol = symbolset->symbol[style->symbol];
   bc = style->backgroundcolor.pen;
   fc = style->color.pen;
   oc = style->outlinecolor.pen;
@@ -1225,7 +1184,7 @@ DEBUG_IF printf("msDrawLineSymbolIM<BR>\n");
 /* if(style->outlinecolor.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->outlinecolor)); */
 /* if(style->color.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->color)); */
 
-  symbol = &(symbolset->symbol[style->symbol]);
+  symbol = symbolset->symbol[style->symbol];
   bc = style->backgroundcolor.pen;
   fc = style->color.pen;
   if(fc==-1) fc = style->outlinecolor.pen;
@@ -1382,12 +1341,12 @@ DEBUG_IF printf("msDrawLineSymbolIM<BR>\n");
     break;
   }  
 
-  if(symbol->stylelength > 0) {
+  if(symbol->patternlength > 0) {
     int k=0, sc;
    
     sc = fc; // start with foreground color
-    for(i=0; i<symbol->stylelength; i++) {      
-      for(j=0; j<symbol->style[i]; j++) {
+    for(i=0; i<symbol->patternlength; i++) {      
+      for(j=0; j<symbol->pattern[i]; j++) {
 	styleDashed[k] = sc;
 	k++;
       } 
@@ -1438,7 +1397,7 @@ DEBUG_IF printf("msDrawShadeSymbolIM\n<BR>");
 /* if(style->color.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->color)); */
 /* if(style->outlinecolor.pen == MS_PEN_UNSET) msImageSetPenIM(img, &(style->outlinecolor)); */
 
-  symbol = &(symbolset->symbol[style->symbol]);
+  symbol = symbolset->symbol[style->symbol];
 /* bc = style->backgroundcolor.pen; */
 /* fc = style->color.pen; */
 /* oc = style->outlinecolor.pen; */
@@ -1817,7 +1776,7 @@ int msDrawTextIM(imageObj* img, pointObj labelPnt, char *string, labelObj *label
       return(-1);
 
     if(label->wrap != '\0') {
-      if((token = split(string, label->wrap, &(num_tokens))) == NULL)
+      if((token = msStringSplit(string, label->wrap, &(num_tokens))) == NULL)
 	return(-1);
 
       y -= fontPtr->h*num_tokens;
@@ -1861,7 +1820,7 @@ int msDrawTextIM(imageObj* img, pointObj labelPnt, char *string, labelObj *label
 int msDrawLabelCacheIM(imageObj* img, mapObj *map)
 {
   pointObj p;
-  int i, j, l;
+  int i, j, l, priority;
   rectObj r;
   
   labelCacheMemberObj *cachePtr=NULL;
@@ -1872,12 +1831,16 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
   int marker_offset_x, marker_offset_y;
   rectObj marker_rect;
 
-DEBUG_IF printf("msDrawLabelCacheIM\n<BR>");
-for(l=map->labelcache.numlabels-1; l>=0; l--) {
+  DEBUG_IF printf("msDrawLabelCacheIM\n<BR>");
+  for(priority=MS_MAX_LABEL_PRIORITY-1; priority>=0; priority--) {
+   labelCacheSlotObj *cacheslot;
+   cacheslot = &(map->labelcache.slots[priority]);
 
-    cachePtr = &(map->labelcache.labels[l]); /* point to right spot in the label cache */
+   for(l=cacheslot->numlabels-1; l>=0; l--) {
 
-    layerPtr = &(map->layers[cachePtr->layerindex]); /* set a couple of other pointers, avoids nasty references */
+    cachePtr = &(cacheslot->labels[l]); /* point to right spot in the label cache */
+
+    layerPtr = (GET_LAYER(map, cachePtr->layerindex)); /* set a couple of other pointers, avoids nasty references */
     labelPtr = &(cachePtr->label);
 
     if(!cachePtr->text || strlen(cachePtr->text) == 0)
@@ -1913,7 +1876,7 @@ for(l=map->labelcache.numlabels-1; l>=0; l--) {
       for(i=0; i<cachePtr->numstyles; i++) {
         if(cachePtr->styles[i].size == -1)
           cachePtr->styles[i].size = (int) msSymbolGetDefaultSize( 
-              &( map->symbolset.symbol[cachePtr->styles[i].symbol] ) );
+              map->symbolset.symbol[cachePtr->styles[i].symbol] );
 	cachePtr->styles[i].size = (int)
             ( cachePtr->styles[i].size * layerPtr->scalefactor);
       }
@@ -1934,39 +1897,9 @@ for(l=map->labelcache.numlabels-1; l>=0; l--) {
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
 
-	  if(!labelPtr->partials) { /* check against image first */
-/* if(labelInImage(img->sx, img->sy, cachePtr->poly, labelPtr->buffer) == MS_FALSE) { */
-/* cachePtr->status = MS_FALSE; */
-/* continue; // next angle */
-/* } */
-	  }
-
-	  for(i=0; i<map->labelcache.nummarkers; i++) { /* compare against points already drawn */
-	    if(l != map->labelcache.markers[i].id) { /* labels can overlap their own marker */
-	      if(intersectLabelPolygons(map->labelcache.markers[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
-
-	  if(!cachePtr->status)
-	    continue; /* next angle */
-
-	  for(i=l+1; i<map->labelcache.numlabels; i++) { /* compare against rendered labels */
-	    if(map->labelcache.labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
-
-	      if((labelPtr->mindistance != -1) && (cachePtr->classindex == map->labelcache.labels[i].classindex) && (strcmp(cachePtr->text,map->labelcache.labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-
-	      if(intersectLabelPolygons(map->labelcache.labels[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
+          /* Compare against rendered labels and markers (sets cachePtr->status) */
+          msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                     -1, -1, labelPtr->buffer, cachePtr, priority, l);
 
 	  if(cachePtr->status) /* found a suitable place for this label */
 	    break;
@@ -1984,39 +1917,9 @@ for(l=map->labelcache.numlabels-1; l>=0; l--) {
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
 
-	  if(!labelPtr->partials) { /* check against image first */
-/* if(labelInImage(img->sx, img->sy, cachePtr->poly, labelPtr->buffer) == MS_FALSE) { */
-/* cachePtr->status = MS_FALSE; */
-/* continue; // next position */
-/* } */
-	  }
-
-	  for(i=0; i<map->labelcache.nummarkers; i++) { /* compare against points already drawn */
-	    if(l != map->labelcache.markers[i].id) { /* labels can overlap their own marker */
-	      if(intersectLabelPolygons(map->labelcache.markers[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
-
-	  if(!cachePtr->status)
-	    continue; /* next position */
-
-	  for(i=l+1; i<map->labelcache.numlabels; i++) { /* compare against rendered labels */
-	    if(map->labelcache.labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
-
-	      if((labelPtr->mindistance != -1) && (cachePtr->classindex == map->labelcache.labels[i].classindex) && (strcmp(cachePtr->text,map->labelcache.labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-
-	      if(intersectLabelPolygons(map->labelcache.labels[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
+          /* Compare against rendered labels and markers (sets cachePtr->status) */
+          msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                     -1, -1, labelPtr->buffer, cachePtr, priority, l);
 
 	  if(cachePtr->status) /* found a suitable place for this label */
 	    break;
@@ -2039,39 +1942,9 @@ for(l=map->labelcache.numlabels-1; l>=0; l--) {
 
       if(!labelPtr->force) { /* no need to check anything else */
 
-	if(!labelPtr->partials) {
-/* if(labelInImage(img->sx, img->sy, cachePtr->poly, labelPtr->buffer) == MS_FALSE) */
-/* cachePtr->status = MS_FALSE; */
-	}
-
-	if(!cachePtr->status)
-	  continue; /* next label */
-
-	for(i=0; i<map->labelcache.nummarkers; i++) { /* compare against points already drawn */
-	  if(l != map->labelcache.markers[i].id) { /* labels can overlap their own marker */
-	    if(intersectLabelPolygons(map->labelcache.markers[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-	      cachePtr->status = MS_FALSE;
-	      break;
-	    }
-	  }
-	}
-
-	if(!cachePtr->status)
-	  continue; /* next label */
-
-	for(i=l+1; i<map->labelcache.numlabels; i++) { /* compare against rendered label */
-	  if(map->labelcache.labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
-	    if((labelPtr->mindistance != -1) && (cachePtr->classindex == map->labelcache.labels[i].classindex) && (strcmp(cachePtr->text, map->labelcache.labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
-	      cachePtr->status = MS_FALSE;
-	      break;
-	    }
-
-	    if(intersectLabelPolygons(map->labelcache.labels[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-	      cachePtr->status = MS_FALSE;
-	      break;
-	    }
-	  }
-	}
+        /* Compare against rendered labels and markers (sets cachePtr->status) */
+        msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                   -1, -1, labelPtr->buffer, cachePtr, priority, l);
       }
     } /* end position if-then-else */
 
@@ -2088,7 +1961,8 @@ for(l=map->labelcache.numlabels-1; l>=0; l--) {
 /* if(MS_VALID_COLOR(labelPtr->backgroundcolor)) billboardIM(img, cachePtr->poly, labelPtr); */
     msDrawTextIM(img, p, cachePtr->text, labelPtr, &(map->fontset), layerPtr->scalefactor); /* actually draw the label, we scaled it in msAddLabel */
 
-  } /* next label in cache */
+   } /* next label in cache */
+  } /* next priority */
 
   return(0);
 }
@@ -2101,6 +1975,9 @@ int msSaveImageIM(imageObj* img, char *filename, outputFormatObj *format )
 
 {
     FILE *stream;
+    char workbuffer[5000];
+    int nSize=0, size=0, iIndice=0; 
+
 DEBUG_IF printf("msSaveImageIM\n<BR>");
 
 if(filename != NULL && strlen(filename) > 0) {
@@ -2142,7 +2019,27 @@ DEBUG_IF printf("FLEN %d<BR>\n", (int)strlen(img->img.imagemap));
 	  } else {
 	    msIO_fprintf(stream, "<map name=\"%s\" width=\"%d\" height=\"%d\">\n", mapName, img->width, img->height);
     	  }
-	  msIO_fprintf(stream, img->img.imagemap);
+          nSize = sizeof(workbuffer);
+
+          size = strlen(img->img.imagemap);
+          if (size > nSize)
+          {
+              iIndice = 0;
+              while ((iIndice + nSize) <= size)
+              {
+                  snprintf(workbuffer, sizeof(workbuffer), "%s", img->img.imagemap+iIndice );
+                  workbuffer[nSize-1] = '\0';
+                  msIO_fprintf(stream, workbuffer);
+                  iIndice +=nSize-1;
+              }
+              if (iIndice < size)
+              {
+                  sprintf(workbuffer, "%s", img->img.imagemap+iIndice );
+                  msIO_fprintf(stream, workbuffer);
+              }
+          }
+          else
+            msIO_fprintf(stream, img->img.imagemap);
 	  if( strcasecmp("OFF",msGetOutputFormatOption( format, "SKIPENDTAG", "OFF" )) == 0){
 		  if (dxf == 2)
 			  msIO_fprintf(stream, "END");
