@@ -24,29 +24,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.29  2006/05/12 18:32:41  pramsey
- * Added support for postgresql joins (Bug 1570).
- *
- * Revision 1.28  2005/12/07 16:47:43  sdlime
- * Fixed potential segfault in mapjoin.c. (bug 1556)
- *
- * Revision 1.27  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.26  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.25  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapjoin.c 5433 2006-05-12 18:32:41Z pramsey $")
+MS_CVSID("$Id: mapjoin.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 #define ROW_ALLOCATION_SIZE 10
 
@@ -188,6 +170,10 @@ int msDBFJoinConnect(layerObj *layer, joinObj *join)
   msDBFJoinInfo *joininfo;
 
   if(join->joininfo) return(MS_SUCCESS); /* already open */
+  
+  if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+	return MS_FAILURE;
+  
     
   /* allocate a msDBFJoinInfo struct */
   joininfo = (msDBFJoinInfo *) malloc(sizeof(msDBFJoinInfo));
@@ -345,6 +331,9 @@ int msCSVJoinConnect(layerObj *layer, joinObj *join)
   char buffer[MS_BUFFER_LENGTH];
 
   if(join->joininfo) return(MS_SUCCESS); /* already open */
+  if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+	return MS_FAILURE;
+  
     
   /* allocate a msCSVJoinInfo struct */
   if((joininfo = (msCSVJoinInfo *) malloc(sizeof(msCSVJoinInfo))) == NULL) {
@@ -379,8 +368,8 @@ int msCSVJoinConnect(layerObj *layer, joinObj *join)
   /* load the rows */
   i = 0;
   while(fgets(buffer, MS_BUFFER_LENGTH, stream) != NULL) {  
-    trimEOL(buffer);  
-    joininfo->rows[i] = split(buffer, ',', &(join->numitems));
+    msStringTrimEOL(buffer);  
+    joininfo->rows[i] = msStringSplit(buffer, ',', &(join->numitems));
     i++;
   }
   fclose(stream);
