@@ -24,52 +24,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.86  2006/09/06 05:26:18  sdlime
- * Fixed a bug with curved labels where character advance array was not being gdFree'd. (bug 1899)
- *
- * Revision 1.85  2006/08/17 04:32:16  sdlime
- * Disable path following labels unless GD 2.0.29 or greater is available.
- *
- * Revision 1.84  2006/03/23 20:28:52  sdlime
- * Most recent patch for curved labels. (bug 1620)
- *
- * Revision 1.83  2006/03/22 23:31:20  sdlime
- * Applied latest patch for curved labels. (bug 1620)
- *
- * Revision 1.82  2006/02/18 20:59:13  sdlime
- * Initial code for curved labels. (bug 1620)
- *
- * Revision 1.81  2006/01/16 20:21:18  sdlime
- * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
- *
- * Revision 1.80  2006/01/12 05:26:23  sdlime
- * Fixed spelling error in module name and replaced a // style comment.
- *
- * Revision 1.79  2005/11/28 04:24:59  sdlime
- * Changed msAddLabel() to use msCopyStyle() rather than doing on its own. This should fix some de-allocation errors folks have been having. (bug 1398)
- *
- * Revision 1.78  2005/08/31 18:30:03  sdlime
- * Applied patch suggested to adjust TTF baselines (bug 1449).
- *
- * Revision 1.77  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.76  2005/05/19 04:09:34  sdlime
- * Removed the LINE_VERT_THRESHOLD test (bug 564) from PDF/SWF/SVG/imagemap drivers.
- *
- * Revision 1.75  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.74  2005/01/11 00:24:28  frank
- * added labelObj arg to msAddLabel()
- *
- * Revision 1.73  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
 /*
 ** maplabel.c: Routines to enable text drawing, BITMAP or TRUETYPE.
@@ -80,14 +35,15 @@
 #include <gdfontmb.h>
 #include <gdfontg.h>
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: maplabel.c 5746 2006-09-06 05:26:18Z sdlime $")
+MS_CVSID("$Id: maplabel.c 6746 2007-08-30 13:59:07Z tbonfort $")
 
 int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, labelPathObj *labelpath, char *string, double featuresize, labelObj *label )
 {
   int i;
   char wrap[2];
+  labelCacheSlotObj *cacheslot;
 
   labelCacheMemberObj *cachePtr=NULL;
   layerObj *layerPtr=NULL;
@@ -95,22 +51,31 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
 
   if(!string) return(MS_SUCCESS); /* not an error */ 
 
-  if(map->labelcache.numlabels == map->labelcache.cachesize) { /* just add it to the end */
-    map->labelcache.labels = (labelCacheMemberObj *) realloc(map->labelcache.labels, sizeof(labelCacheMemberObj)*(map->labelcache.cachesize+MS_LABELCACHEINCREMENT));
-    if(!map->labelcache.labels) {
-      msSetError(MS_MEMERR, "Realloc() error.", "msAddLabel()");
-      return(MS_FAILURE);
-    }
-    map->labelcache.cachesize += MS_LABELCACHEINCREMENT;
-  }
-
-  cachePtr = &(map->labelcache.labels[map->labelcache.numlabels]); /* set up a few pointers for clarity */
-  layerPtr = &(map->layers[layerindex]);
-  classPtr = &(map->layers[layerindex].class[classindex]);
+  layerPtr = (GET_LAYER(map, layerindex)); /* set up a few pointers for clarity */
+  classPtr = GET_LAYER(map, layerindex)->class[classindex];
 
   if( label == NULL )
       label = &(classPtr->label);
 
+  /* Validate label priority value and get ref on label cache for it */
+  if (label->priority < 1)
+      label->priority = 1;
+  else if (label->priority > MS_MAX_LABEL_PRIORITY)
+      label->priority = MS_MAX_LABEL_PRIORITY;
+
+  cacheslot = &(map->labelcache.slots[label->priority-1]);
+
+  if(cacheslot->numlabels == cacheslot->cachesize) { /* just add it to the end */
+    cacheslot->labels = (labelCacheMemberObj *) realloc(cacheslot->labels, sizeof(labelCacheMemberObj)*(cacheslot->cachesize+MS_LABELCACHEINCREMENT));
+    if(!cacheslot->labels) {
+      msSetError(MS_MEMERR, "Realloc() error.", "msAddLabel()");
+      return(MS_FAILURE);
+    }
+   cacheslot->cachesize += MS_LABELCACHEINCREMENT;
+  }
+
+  cachePtr = &(cacheslot->labels[cacheslot->numlabels]);
+
   cachePtr->layerindex = layerindex; /* so we can get back to this *raw* data if necessary */
   cachePtr->classindex = classindex;
   cachePtr->tileindex = tileindex;
@@ -139,10 +104,12 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
   if(label->type != MS_BITMAP && label->wrap != '\0') {
     wrap[0] = label->wrap;
     wrap[1] = '\0';
-    cachePtr->text = gsub(cachePtr->text, wrap, "\r\n");
+    cachePtr->text = msReplaceSubstring(cachePtr->text, wrap, "\r\n");
   }
 
-  /* copy the styles (only if there is an accompanying marker) */
+  /* copy the styles (only if there is an accompanying marker)
+   * We cannot simply keeep refs because the rendering code alters some members of the style objects
+   */
   cachePtr->styles = NULL;
   cachePtr->numstyles = 0;
   if((layerPtr->type == MS_LAYER_ANNOTATION && classPtr->numstyles > 0) || layerPtr->type == MS_LAYER_POINT) {
@@ -151,7 +118,7 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
     if (classPtr->numstyles > 0) {
       for(i=0; i<classPtr->numstyles; i++) {
 	initStyle(&(cachePtr->styles[i]));
-	msCopyStyle(&(cachePtr->styles[i]), &(classPtr->styles[i]));
+	msCopyStyle(&(cachePtr->styles[i]), classPtr->styles[i]);
       }
     }
   }
@@ -171,39 +138,137 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
     rectObj rect;
     int w, h;
 
-    if(map->labelcache.nummarkers == map->labelcache.markercachesize) { /* just add it to the end */
-      map->labelcache.markers = (markerCacheMemberObj *) realloc(map->labelcache.markers, sizeof(markerCacheMemberObj)*(map->labelcache.cachesize+MS_LABELCACHEINCREMENT));
-      if(!map->labelcache.markers) {
+    if(cacheslot->nummarkers == cacheslot->markercachesize) { /* just add it to the end */
+      cacheslot->markers = (markerCacheMemberObj *) realloc(cacheslot->markers, sizeof(markerCacheMemberObj)*(cacheslot->cachesize+MS_LABELCACHEINCREMENT));
+      if(!cacheslot->markers) {
 	msSetError(MS_MEMERR, "Realloc() error.", "msAddLabel()");
 	return(MS_FAILURE);
       }
-      map->labelcache.markercachesize+=MS_LABELCACHEINCREMENT;
+      cacheslot->markercachesize+=MS_LABELCACHEINCREMENT;
     }
 
-    i = map->labelcache.nummarkers;
+    i = cacheslot->nummarkers;
 
-    map->labelcache.markers[i].poly = (shapeObj *) malloc(sizeof(shapeObj));
-    msInitShape(map->labelcache.markers[i].poly);
+    cacheslot->markers[i].poly = (shapeObj *) malloc(sizeof(shapeObj));
+    msInitShape(cacheslot->markers[i].poly);
 
     /* TO DO: at the moment only checks the bottom style, perhaps should check all of them */
-    if(msGetMarkerSize(&map->symbolset, &(classPtr->styles[0]), &w, &h, layerPtr->scalefactor) != MS_SUCCESS)
+    if(msGetMarkerSize(&map->symbolset, classPtr->styles[0], &w, &h, layerPtr->scalefactor) != MS_SUCCESS)
       return(MS_FAILURE);
 
     rect.minx = MS_NINT(point->x - .5 * w);
     rect.miny = MS_NINT(point->y - .5 * h);
     rect.maxx = rect.minx + (w-1);
     rect.maxy = rect.miny + (h-1);
-    msRectToPolygon(rect, map->labelcache.markers[i].poly);    
-    map->labelcache.markers[i].id = map->labelcache.numlabels;
+    msRectToPolygon(rect, cacheslot->markers[i].poly);    
+    cacheslot->markers[i].id = cacheslot->numlabels;
 
-    map->labelcache.nummarkers++;
+    cacheslot->nummarkers++;
   }
 
+  cacheslot->numlabels++;
+
+  /* Maintain main labelCacheObj.numlabels only for backwards compatibility */
   map->labelcache.numlabels++;
 
   return(MS_SUCCESS);
 }
 
+
+/* msTestLabelCacheCollisions()
+**
+** Compares current label against labels already drawn and markers from cache and discards it
+** by setting cachePtr->status=MS_FALSE if it is a duplicate, collides with another label,
+** or collides with a marker.
+**
+** This function is used by the various msDrawLabelCacheXX() implementations.
+*/
+void msTestLabelCacheCollisions(labelCacheObj *labelcache, labelObj *labelPtr, 
+                                int mapwidth, int mapheight, int buffer,
+                                labelCacheMemberObj *cachePtr, int current_priority, int current_label)
+{
+    int i, p;
+
+    /* Check against image bounds first 
+    ** Pass mapwidth=-1 to skip this test
+     */
+    if(!labelPtr->partials && mapwidth > 0 && mapheight > 0) {
+        if(labelInImage(mapwidth, mapheight, cachePtr->poly, buffer) == MS_FALSE) {
+	    cachePtr->status = MS_FALSE;
+            return;
+        }
+    }
+
+    /* Compare against all rendered markers from this priority level and higher.
+    ** Labels can overlap their own marker and markers from lower priority levels
+    */
+    for (p=current_priority; p < MS_MAX_LABEL_PRIORITY; p++) {
+        labelCacheSlotObj *markerslot;
+        markerslot = &(labelcache->slots[p]);
+
+        for ( i = 0; i < markerslot->nummarkers; i++ ) {
+            if ( !(p == current_priority && current_label == markerslot->markers[i].id) ) {  /* labels can overlap their own marker */
+                if ( intersectLabelPolygons(markerslot->markers[i].poly, cachePtr->poly ) == MS_TRUE ) {
+                    cachePtr->status = MS_FALSE;  /* polys intersect */
+                    return;
+                }
+            }
+        }
+    }
+
+    /* compare against rendered labels */
+    i = current_label+1;
+
+    for(p=current_priority; p<MS_MAX_LABEL_PRIORITY; p++) {
+        labelCacheSlotObj *cacheslot;
+        cacheslot = &(labelcache->slots[p]);
+
+        for(  ; i < cacheslot->numlabels; i++) { 
+            if(cacheslot->labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
+
+                if((labelPtr->mindistance != -1) && (cachePtr->classindex == cacheslot->labels[i].classindex) && (strcmp(cachePtr->text,cacheslot->labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(cacheslot->labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
+                    cachePtr->status = MS_FALSE;
+                    return;
+                }
+
+                if(intersectLabelPolygons(cacheslot->labels[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
+                    cachePtr->status = MS_FALSE;
+                    return;
+                }
+            }
+        } /* i */
+
+        i = 0; /* Start over with 1st label of next slot */
+    } /* p */
+}
+
+
+/* msGetLabelCacheMember()
+**
+** Returns label cache members by index, making all members of the cache
+** appear as if they were stored in a single array indexed from 0 to numlabels.
+**
+** Returns NULL if requested index is out of range.
+*/
+labelCacheMemberObj *msGetLabelCacheMember(labelCacheObj *labelcache, int i)
+{
+    if(i >= 0 && i < labelcache->numlabels)
+    {
+        int p;
+        for(p=0; p<MS_MAX_LABEL_PRIORITY; p++)
+        {
+            if (i < labelcache->slots[p].numlabels )
+                return &(labelcache->slots[p].labels[i]); /* Found it */
+            else
+                i -= labelcache->slots[p].numlabels; /* Check next slots */
+        }
+    }
+
+    /* Out of range / not found */
+    return NULL;
+}
+
+
 int msInitFontSet(fontSetObj *fontset)
 {
     fontset->filename = NULL;
@@ -248,7 +313,7 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 
   fontset->map = (mapObj *)map;
 
-  path = getPath(fontset->filename);
+  path = msGetPath(fontset->filename);
 
   /* fontset->fonts = msCreateHashTable(); // create font hash */
   /* if(!fontset->fonts) { */
@@ -372,7 +437,7 @@ int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fon
       return(-1);
 
     if(label->wrap != '\0') {
-      if((token = split(string, label->wrap, &(num_tokens))) == NULL)
+      if((token = msStringSplit(string, label->wrap, &(num_tokens))) == NULL)
 	return(0);
 
       for(t=0; t<num_tokens; t++) /* what's the longest token */
@@ -681,20 +746,17 @@ int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p
   double theta, length, current_length;
   labelObj label;
   pointObj point, label_point;
-  shapeObj label_poly;
   rectObj label_rect;
   int label_width;
   int position, rot, gap, in;
   double rx, ry, size;
-
   symbolObj *symbol;
 
-  msInitShape(&label_poly);
 
-  symbol = &(symbolset->symbol[style->symbol]);
+  symbol = symbolset->symbol[style->symbol];
 
   initLabel(&label);
-  rot = (symbol->gap < 0);
+  rot = (symbol->gap <= 0);
   label.type = MS_TRUETYPE;
   label.font = symbol->font;
   /* -- rescaling symbol and gap */
@@ -742,14 +804,17 @@ int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p
       else theta = -theta;        
       if((position == MS_UR)||(position == MS_UL)) position = MS_UC;
       if((position == MS_LR)||(position == MS_LL)) position = MS_LC;
-      label.angle = MS_RAD_TO_DEG * theta;
-
+      
+      label.angle = style->angle;
+      if(rot)
+          label.angle+=MS_RAD_TO_DEG * theta;
+      
       in = 0;
       while(current_length <= length) {
         point.x = MS_NINT(p->line[i].point[j-1].x + current_length*rx);
         point.y = MS_NINT(p->line[i].point[j-1].y + current_length*ry);
 
-        label_point = get_metrics(&point, position, label_rect, 0, 0, label.angle, 0, &label_poly);
+        label_point = get_metrics(&point, (rot)?MS_CC:position, label_rect, 0, 0, label.angle, 0, NULL);
         msDrawTextGD(img, label_point, symbol->character, &label, symbolset->fontset, scalefactor);
 
         current_length += label_width + gap;
