@@ -24,66 +24,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.62  2006/08/04 04:36:12  sdlime
- * Removed cut & paste artifact (oops).
- *
- * Revision 1.61  2006/08/04 04:33:45  sdlime
- * Fixed a problem with full-image legends where layer sizeunits were not taken into account when rendering a legend. (bug 1147)
- *
- * Revision 1.60  2006/03/27 05:48:03  sdlime
- * Fixed symbol initialization error with embedded scalebars and legends. (bug 1725)
- *
- * Revision 1.59  2006/02/18 20:59:13  sdlime
- * Initial code for curved labels. (bug 1620)
- *
- * Revision 1.58  2006/01/16 20:21:18  sdlime
- * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
- *
- * Revision 1.57  2005/11/17 05:56:30  sdlime
- * Updated msDrawLegend to respect class min/max scale values. (bug 1524)
- *
- * Revision 1.56  2005/10/31 06:03:26  sdlime
- * Updated msDrawLegend() to consider layer order. (bug 1484)
- *
- * Revision 1.55  2005/09/14 00:11:35  frank
- * fixed leak of imageObj when embedding legends
- *
- * Revision 1.54  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.53  2005/02/18 18:57:26  sean
- * turn on GD alpha blending for legend icon images if specified by the target layer (bugs 490, 1250)
- *
- * Revision 1.52  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.51  2005/01/11 00:24:07  frank
- * added labelObj arg to msAddLabel()
- *
- * Revision 1.50  2004/11/22 03:43:54  sdlime
- * Added tests to mimimize the threat of recursion problems when evaluating LAYER REQUIRES or LABELREQUIRES expressions. Note that via MapScript it is possible to circumvent that test by defining layers with problems after running prepareImage. Other things crop up in that case too (symbol scaling dies) so it should be considered bad programming practice.
- *
- * Revision 1.49  2004/11/05 19:19:05  frank
- * avoid casting warning i msDrawLegendIcon call
- *
- * Revision 1.48  2004/11/04 21:33:08  frank
- * Removed unused variable.
- *
- * Revision 1.47  2004/10/28 18:16:17  dan
- * Fixed WMS GetLegendGraphic which was returning an exception (GD error)
- * when requested layer was out of scale (bug 1006)
- *
- * Revision 1.46  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ *****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: maplegend.c 5576 2006-08-04 04:36:12Z sdlime $")
+MS_CVSID("$Id: maplegend.c 6615 2007-08-17 14:00:22Z dmorissette $")
 
 #define PSF .8
 #define VMARGIN 5 /* margin at top and bottom of legend graphic */
@@ -133,7 +78,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
     if(type == MS_LAYER_POLYGON) {
       type = MS_LAYER_LINE;
       for(i=0; i<class->numstyles; i++) {
-       if(MS_VALID_COLOR(class->styles[i].color)) { /* there is a fill */
+       if(MS_VALID_COLOR(class->styles[i]->color)) { /* there is a fill */
 	  type = MS_LAYER_POLYGON;
 	  break;
         }
@@ -145,9 +90,8 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
     */     
 
     /* Bug 490 - switch alpha blending on for a layer that requires it */
-    if (lp->transparency == MS_GD_ALPHA) {
-        gdImageAlphaBlending(img, 1);
-    }
+    if(lp->opacity == MS_GD_ALPHA)
+      gdImageAlphaBlending(img, 1);
 
     switch(type) {
     case MS_LAYER_ANNOTATION:
@@ -156,7 +100,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
       marker.y = dstY + MS_NINT(height / 2.0);
 
       for(i=0; i<class->numstyles; i++)
-        msDrawMarkerSymbolGD(&map->symbolset, img, &marker, &(class->styles[i]), lp->scalefactor);          
+        msDrawMarkerSymbolGD(&map->symbolset, img, &marker, class->styles[i], lp->scalefactor);          
       break;
     case MS_LAYER_LINE:
       zigzag.line = (lineObj *)malloc(sizeof(lineObj));
@@ -175,16 +119,17 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
       zigzag.line[0].numpoints = 4;
 
       for(i=0; i<class->numstyles; i++)
-        msDrawLineSymbolGD(&map->symbolset, img, &zigzag, &(class->styles[i]), lp->scalefactor); 
+        msDrawLineSymbolGD(&map->symbolset, img, &zigzag, class->styles[i], lp->scalefactor); 
 
       free(zigzag.line[0].point);
       free(zigzag.line);	
       break;
     case MS_LAYER_CIRCLE:
     case MS_LAYER_RASTER:
+    case MS_LAYER_CHART:
     case MS_LAYER_POLYGON:
       for(i=0; i<class->numstyles; i++)     
-        msDrawShadeSymbolGD(&map->symbolset, img, &box, &(class->styles[i]), lp->scalefactor);
+        msDrawShadeSymbolGD(&map->symbolset, img, &box, class->styles[i], lp->scalefactor);
       break;
     default:
       return MS_FAILURE;
@@ -212,7 +157,7 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
   outputFormatObj *format = NULL;
   int i = 0;
 
-  if(!map->outputformat || !MS_RENDERER_GD(map->outputformat) ) {
+  if(!map->outputformat || (!MS_RENDERER_GD(map->outputformat) && !MS_RENDERER_AGG(map->outputformat) )) {
     msSetError(MS_GDERR, "Map outputformat must be set to a GD format!", "msCreateLegendIcon()");
     return(NULL);
   }
@@ -221,7 +166,12 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
   msApplyOutputFormat(&format, map->outputformat, map->legend.transparent, map->legend.interlace, MS_NOOVERRIDE);
 
   /* create image */
-  image = msImageCreateGD(width, height, map->outputformat, map->web.imagepath, map->web.imageurl);
+#ifdef USE_AGG
+  if( MS_RENDERER_AGG(map->outputformat) )
+      image = msImageCreateAGG(width, height, map->outputformat, map->web.imagepath, map->web.imageurl);        
+  else
+#endif
+      image = msImageCreateGD(width, height, map->outputformat, map->web.imagepath, map->web.imageurl);
 
   /* drop this reference to output format */
   msApplyOutputFormat( &format, NULL, MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE );
@@ -244,7 +194,7 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
       msDrawLegendIcon(map, lp, class, width, height, image->img.gd, 0, 0);
     } else {
       for (i=0; i<lp->numclasses; i++) {
-        msDrawLegendIcon(map, lp, &lp->class[i], width, height, image->img.gd, 0, 0);
+        msDrawLegendIcon(map, lp, lp->class[i], width, height, image->img.gd, 0, 0);
       }
     }
   }
@@ -279,7 +229,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
 
   if (!scale_independent) {
     map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
-    status = msCalculateScale(map->extent, map->units, map->width, map->height, map->resolution, &map->scale);
+    status = msCalculateScale(map->extent, map->units, map->width, map->height, map->resolution, &map->scaledenom);
     if(status != MS_SUCCESS) return(NULL);
   }
 
@@ -289,13 +239,13 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   ** allocate heights array
   */
   for(i=0; i<map->numlayers; i++) {
-    lp = &(map->layers[map->layerorder[i]]);
+    lp = (GET_LAYER(map, map->layerorder[i]));
 
     if((lp->status == MS_OFF) || (lp->type == MS_LAYER_QUERY)) /* skip it */
       continue;
 
     for(j=0;j<lp->numclasses;j++) {
-      if(!lp->class[j].name) continue; /* skip it */
+      if(!lp->class[j]->name) continue; /* skip it */
       n++;
     }
   }
@@ -310,28 +260,28 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   */
   n=0;
   for(i=0; i<map->numlayers; i++) { /* Need to find the longest legend label string */
-    lp = &(map->layers[map->layerorder[i]]);
+    lp = (GET_LAYER(map, map->layerorder[i]));
 
     if((lp->status == MS_OFF) || (lp->type == MS_LAYER_QUERY)) /* skip it */
       continue;
 
-    if(!scale_independent && map->scale > 0) {
-      if((lp->maxscale > 0) && (map->scale > lp->maxscale)) continue;
-      if((lp->minscale > 0) && (map->scale <= lp->minscale)) continue;
+    if(!scale_independent && map->scaledenom > 0) {
+      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
+      if((lp->maxscaledenom > 0) && (map->scaledenom <= lp->maxscaledenom)) continue;
     }
  
     for(j=0;j<lp->numclasses;j++) {
-      if(!lp->class[j].name)
+      if(!lp->class[j]->name)
 	continue; /* skip it */
 
-      if(!scale_independent && map->scale > 0) {  /* verify class scale here */
-	if((lp->class[j].maxscale > 0) && (map->scale > lp->class[j].maxscale))
+      if(!scale_independent && map->scaledenom > 0) {  /* verify class scale here */
+	if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom > lp->class[j]->maxscaledenom))
 	  continue;
-	if((lp->class[j].minscale > 0) && (map->scale <= lp->class[j].minscale))
+	if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom <= lp->class[j]->maxscaledenom))
 	  continue;
       }
 
-      if(msGetLabelSize(lp->class[j].name, &map->legend.label, &rect, &(map->fontset), 1.0, MS_FALSE) != 0)
+      if(msGetLabelSize(lp->class[j]->name, &map->legend.label, &rect, &(map->fontset), 1.0, MS_FALSE) != 0)
 	return(NULL); /* something bad happened */
 
       maxheight = MS_MAX(maxheight, MS_NINT(rect.maxy - rect.miny));
@@ -353,7 +303,12 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   msApplyOutputFormat(&format, map->outputformat, map->legend.transparent, map->legend.interlace, MS_NOOVERRIDE);
 
   /* initialize the legend image */
-  image = msImageCreateGD(size_x, size_y, format, map->web.imagepath, map->web.imageurl);
+#ifdef USE_AGG
+  if( MS_RENDERER_AGG(map->outputformat) )
+      image = msImageCreateAGG(size_x, size_y, format, map->web.imagepath, map->web.imageurl);        
+  else
+#endif
+      image = msImageCreateGD(size_x, size_y, format, map->web.imagepath, map->web.imageurl);
 
   /* drop this reference to output format */
   msApplyOutputFormat(&format, NULL, MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE);
@@ -371,19 +326,23 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
 
   msClearPenValues(map); /* just in case the mapfile has already been processed */
 
+#ifdef USE_AGG
+  if(MS_RENDERER_AGG(map->outputformat))
+      msAlphaGD2AGG(image);
+#endif
   pnt.y = VMARGIN;
     
   /* for(i=0; i<map->numlayers; i++) { */
   for(i=map->numlayers-1; i>=0; i--) {
-    lp = &(map->layers[map->layerorder[i]]); /* for brevity */
+    lp = (GET_LAYER(map, map->layerorder[i])); /* for brevity */
 
     if((lp->numclasses == 0) || (lp->status == MS_OFF) || (lp->type == MS_LAYER_QUERY))
       continue; /* skip this layer */
 
-    if(!scale_independent && map->scale > 0) {
-      if((lp->maxscale > 0) && (map->scale > lp->maxscale))
+    if(!scale_independent && map->scaledenom > 0) {
+      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom))
 	continue;
-      if((lp->minscale > 0) && (map->scale <= lp->minscale))
+      if((lp->maxscaledenom > 0) && (map->scaledenom <= lp->maxscaledenom))
 	continue;
 
       /* Should we also consider lp->symbolscale? I don't think so. Showing the "standard" size makes the most sense. */
@@ -392,25 +351,40 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
     }
 
     for(j=0; j<lp->numclasses; j++) { /* always at least 1 class */
+      char *encodedText = NULL;
 
-      if(!lp->class[j].name) continue; /* skip it */
+      if(!lp->class[j]->name) continue; /* skip it */
      
-      if(!scale_independent && map->scale > 0) {  /* verify class scale here */
-        if((lp->class[j].maxscale > 0) && (map->scale > lp->class[j].maxscale))
+      if(!scale_independent && map->scaledenom > 0) {  /* verify class scale here */
+        if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom > lp->class[j]->maxscaledenom))
           continue;
-        if((lp->class[j].minscale > 0) && (map->scale <= lp->class[j].minscale))
+        if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom <= lp->class[j]->maxscaledenom))
           continue;
       }
  
       pnt.x = HMARGIN + map->legend.keysizex + map->legend.keyspacingx;
       
       /* TODO */
-      if(msDrawLegendIcon(map, lp, &(lp->class[j]),  map->legend.keysizex,  map->legend.keysizey, image->img.gd, HMARGIN, (int) pnt.y) != MS_SUCCESS)
+#ifdef USE_AGG
+      if(MS_RENDERER_AGG(map->outputformat)) {
+          if(msDrawLegendIconAGG(map, lp, lp->class[j],  map->legend.keysizex,  map->legend.keysizey, image, HMARGIN, (int) pnt.y) != MS_SUCCESS)
+                  return NULL;
+      }
+      else
+#endif
+      if(msDrawLegendIcon(map, lp, lp->class[j],  map->legend.keysizex,  map->legend.keysizey, image->img.gd, HMARGIN, (int) pnt.y) != MS_SUCCESS)
         return NULL;
 
       pnt.y += MS_MAX(map->legend.keysizey, maxheight);
       /* TODO */
-      msDrawLabel(image, pnt, lp->class[j].name, &(map->legend.label), &map->fontset, 1.0);
+
+      if(map->legend.label.encoding &&
+         (encodedText = msGetEncodedString(lp->class[j]->name, map->legend.label.encoding)) != NULL) {
+        msDrawLabel(image, pnt, encodedText, &(map->legend.label), &map->fontset, 1.0);
+        free(encodedText);
+      }
+      else
+        msDrawLabel(image, pnt, lp->class[j]->name, &(map->legend.label), &map->fontset, 1.0);
 
       pnt.y += map->legend.keyspacingy; /* bump y for next label */
 	
@@ -418,6 +392,10 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   } /* next layer */
 
   free(heights);
+#ifdef USE_AGG
+  if(MS_RENDERER_AGG(map->outputformat))
+      msAlphaAGG2GD(image);
+#endif
   return(image);
 }
 
@@ -430,93 +408,99 @@ int msEmbedLegend(mapObj *map, gdImagePtr img)
 
   s = msGetSymbolIndex(&(map->symbolset), "legend", MS_FALSE);
   if(s == -1) {
+    if (msGrowSymbolSet(&map->symbolset) == NULL)
+        return -1;
     s = map->symbolset.numsymbols;
     map->symbolset.numsymbols++;
-    initSymbol(&(map->symbolset.symbol[s]));
+    initSymbol(map->symbolset.symbol[s]);
   } else {
-    if(map->symbolset.symbol[s].img) 
-      gdImageDestroy(map->symbolset.symbol[s].img);
+    if(map->symbolset.symbol[s]->img) 
+      gdImageDestroy(map->symbolset.symbol[s]->img);
   }
   
   /* render the legend. */
   image = msDrawLegend(map, MS_FALSE);
 
   /* steal the gdImage and free the rest of the imageObj */
-  map->symbolset.symbol[s].img = image->img.gd; 
+  map->symbolset.symbol[s]->img = image->img.gd; 
   image->img.gd = NULL;
   msFreeImage( image );
 
 
-  if(!map->symbolset.symbol[s].img) return(-1); /* something went wrong creating scalebar */
+  if(!map->symbolset.symbol[s]->img) return(-1); /* something went wrong creating scalebar */
 
-  map->symbolset.symbol[s].type = MS_SYMBOL_PIXMAP; /* intialize a few things */
-  map->symbolset.symbol[s].name = strdup("legend");  
-  map->symbolset.symbol[s].sizex = map->symbolset.symbol[s].img->sx;
-  map->symbolset.symbol[s].sizey = map->symbolset.symbol[s].img->sy;
+  map->symbolset.symbol[s]->type = MS_SYMBOL_PIXMAP; /* intialize a few things */
+  map->symbolset.symbol[s]->name = strdup("legend");  
+  map->symbolset.symbol[s]->sizex = map->symbolset.symbol[s]->img->sx;
+  map->symbolset.symbol[s]->sizey = map->symbolset.symbol[s]->img->sy;
 
   /* I'm not too sure this test is sufficient ... NFW. */
   if(map->legend.transparent == MS_ON)
-    gdImageColorTransparent(map->symbolset.symbol[s].img, 0);
+    gdImageColorTransparent(map->symbolset.symbol[s]->img, 0);
 
   switch(map->legend.position) {
   case(MS_LL):
-    point.x = MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_LR):
-    point.x = map->width - MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = map->width - MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_LC):
     point.x = MS_NINT(map->width/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_UR):
-    point.x = map->width - MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = map->width - MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_UL):
-    point.x = MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_UC):
     point.x = MS_NINT(map->width/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   }
 
   l = msGetLayerIndex(map, "__embed__legend");
   if(l == -1) {
+    if (msGrowMapLayers(map) == NULL)
+        return(-1);
     l = map->numlayers;
     map->numlayers++;
+    if(initLayer((GET_LAYER(map, l)), map) == -1) return(-1);
+    GET_LAYER(map, l)->name = strdup("__embed__legend");
+    GET_LAYER(map, l)->type = MS_LAYER_ANNOTATION;
 
-    if(initLayer(&(map->layers[l]), map) == -1) return(-1);
-    map->layers[l].name = strdup("__embed__legend");
-    map->layers[l].type = MS_LAYER_ANNOTATION;
-
-    if(initClass(&(map->layers[l].class[0])) == -1) return(-1);
-    map->layers[l].numclasses = 1; /* so we make sure to free it */
+    if (msGrowLayerClasses( GET_LAYER(map, l) ) == NULL)
+        return(-1);
+    if(initClass(GET_LAYER(map, l)->class[0]) == -1) return(-1);
+    GET_LAYER(map, l)->numclasses = 1; /* so we make sure to free it */
         
     /* update the layer order list with the layer's index. */
     map->layerorder[l] = l;
   }
 
-  map->layers[l].status = MS_ON;
+  GET_LAYER(map, l)->status = MS_ON;
 
-  map->layers[l].class[0].numstyles = 1;
-  map->layers[l].class[0].styles[0].symbol = s;
-  map->layers[l].class[0].styles[0].color.pen = -1;
-  map->layers[l].class[0].label.force = MS_TRUE;
-  map->layers[l].class[0].label.size = MS_MEDIUM; /* must set a size to have a valid label definition */
+  if (msMaybeAllocateStyle(GET_LAYER(map, l)->class[0], 0)==MS_FAILURE) return MS_FAILURE;
+  GET_LAYER(map, l)->class[0]->styles[0]->symbol = s;
+  GET_LAYER(map, l)->class[0]->styles[0]->color.pen = -1;
+  GET_LAYER(map, l)->class[0]->label.force = MS_TRUE;
+  GET_LAYER(map, l)->class[0]->label.size = MS_MEDIUM; /* must set a size to have a valid label definition */
+  GET_LAYER(map, l)->class[0]->label.priority = MS_MAX_LABEL_PRIORITY;
 
   if(map->legend.postlabelcache) /* add it directly to the image */
-    msDrawMarkerSymbolGD(&map->symbolset, img, &point, &(map->layers[l].class[0].styles[0]), 1.0);
+    msDrawMarkerSymbolGD(&map->symbolset, img, &point, GET_LAYER(map, l)->class[0]->styles[0], 1.0);
   else
     msAddLabel(map, l, 0, -1, -1, &point, NULL, " ", 1.0, NULL);
 
   /* Mark layer as deleted so that it doesn't interfere with html legends or */
   /* with saving maps */
-  map->layers[l].status = MS_DELETE;
+  GET_LAYER(map, l)->status = MS_DELETE;
 
   return(0);
 }
