@@ -15,8 +15,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -25,101 +25,16 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.27  2006/09/24 03:31:35  frank
- * Same as last fix, but for y dimension, instead of just x.
- *
- * Revision 1.26  2006/09/23 16:00:51  frank
- * Fixed computation of geotransform to match BBOX (to edges of image) not
- * map.extent (to center of edge pixels).  (bug 1916)
- *
- * Revision 1.25  2006/08/03 00:22:00  sdlime
- * Fixed problem where insertLayer does not set the layer index properly if the layer is inserted between existing layers. (bug 1838)
- *
- * Revision 1.24  2006/03/31 15:53:03  julien
- * bug 1733 Fix SLD nonsquare pixel and SLD with FE projection issue
- *
- * Revision 1.23  2005/10/26 17:40:03  frank
- * Avoid unused variable warning.
- *
- * Revision 1.22  2005/09/23 16:20:44  hobu
- * add meatier error message for invalid extents
- *
- * Revision 1.21  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.20  2004/10/28 02:24:01  frank
- * Added special putenv() case in msApplyMapConfigOptions() for MS_ERRORFILE
- *
- * Revision 1.19  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.18  2004/10/12 21:43:00  sean
- * check for NULL before inserting objects
- *
- * Revision 1.17  2004/09/29 18:27:55  frank
- * comment formatting.
- *
- * Revision 1.16  2004/09/27 13:31:34  sean
- * corrections to msInsertLayer and msRemoveLayer, a 524 byte leak remains for each mapscript layer (bug 841)
- *
- * Revision 1.15  2004/09/24 16:28:01  sean
- * mapscript can be again compiled without requiring WMS support, mapObj::loadOWSParameters will set an error in that case (bug 894).
- *
- * Revision 1.14  2004/09/10 13:47:46  sean
- * msMapSetSize returns value returned from msMapComputeGeotransform, just as msMapSetRotation
- *
- * Revision 1.13  2004/07/29 16:35:31  hobu
- * drop msRectIsValid and use MS_VALID_EXTENT
- *
- * Revision 1.12  2004/07/28 15:34:03  hobu
- * add a test after the input of an extent in msMapSetExtent to check for its validity.  Also convert tabs back to spaces.
- *
- * Revision 1.11  2004/07/28 14:24:03  hobu
- * Updated msMapSetExtent in mapobject.c to use msRectIsValid
- *
- * Revision 1.10  2004/07/27 19:03:37  sean
- * moved layer ordering functions from maputil.c to mapobject.c
- *
- * Revision 1.9  2004/07/23 14:12:07  frank
- * emit error if unimplemented georef<->pixel functions called
- *
- * Revision 1.8  2004/07/07 21:57:57  sean
- * Completed work on msInsertLayer and msRemoveLayer, added prototypes to map.h.  See bug 759.
- *
- * Revision 1.7  2004/07/07 04:34:40  sean
- * added msInsertLayer() and msRemoveLayer().  See mapserver bug 759.
- *
- * Revision 1.6  2004/06/24 17:30:11  frank
- * fixed typo in msTestConfigOption()
- *
- * Revision 1.5  2004/06/22 20:55:20  sean
- * Towards resolving issue 737 changed hashTableObj to a structure which contains a hashObj **items.  Changed all hash table access functions to operate on the target table by reference.  msFreeHashTable should not be used on the hashTableObj type members of mapserver structures, use msFreeHashItems instead.
- *
- * Revision 1.4  2004/06/14 17:34:22  frank
- * added msTestConfigOption
- *
- * Revision 1.3  2004/05/31 20:20:56  frank
- * ensure gdal.h is included for Config stuff
- *
- * Revision 1.2  2004/05/31 17:27:02  frank
- * Made msMapComputeGeotransform() return MS_SUCCESS or MS_FAILURE.
- *
- * Revision 1.1  2004/05/25 16:05:48  frank
- * New
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
 #ifdef USE_GDAL
 #  include "gdal.h"
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: mapobject.c 5762 2006-09-24 03:31:35Z frank $")
+MS_CVSID("$Id: mapobject.c 6585 2007-08-15 03:30:43Z hobu $")
 
 void freeWeb(webObj *web);
 void freeScalebar(scalebarObj *scalebar);
@@ -162,6 +77,13 @@ void msFreeMap(mapObj *map) {
   int i;
 
   if(!map) return;
+
+  /*printf("msFreeMap(): maybe freeing map at %p count=%d.\n",map, map->refcount);*/
+  if ( MS_REFCNT_DECR_IS_NOT_ZERO(map) ) { return; }
+  if (map->debug >= MS_DEBUGLEVEL_VV)
+     msDebug("msFreeMap(): freeing map at %p.",map);
+  /*printf("msFreeMap(): freeing map at %p.\n",map);*/
+  
   msCloseConnections(map);
 
   msFree(map->name);
@@ -196,8 +118,14 @@ void msFreeMap(mapObj *map) {
   freeReferenceMap(&(map->reference));
   freeLegend(&(map->legend));  
 
-  for(i=0; i<map->numlayers; i++)
-    freeLayer(&(map->layers[i]));
+  for(i=0; i<map->maxlayers; i++) {
+    if (GET_LAYER(map, i)!=NULL) {
+    	GET_LAYER(map, i)->map=NULL;
+    	if ( freeLayer((GET_LAYER(map, i))) == MS_SUCCESS) {
+	    	free(GET_LAYER(map, i));
+	}
+    }
+  }
   msFree(map->layers);
 
   if (map->layerorder)
@@ -223,7 +151,7 @@ const char *msGetConfigOption( mapObj *map, const char *key)
 /*                         msSetConfigOption()                          */
 /************************************************************************/
 
-void msSetConfigOption( mapObj *map, const char *key, const char *value)
+int msSetConfigOption( mapObj *map, const char *key, const char *value)
 
 {
     /* We have special "early" handling of this so that it will be */
@@ -231,9 +159,19 @@ void msSetConfigOption( mapObj *map, const char *key, const char *value)
     if( strcasecmp(key,"PROJ_LIB") == 0 )
         msSetPROJ_LIB( value );
 
+    /* Same for MS_ERRORFILE, we want it to kick in as early as possible
+     * to catch parsing errors */
+    if( strcasecmp(key,"MS_ERRORFILE") == 0 )
+    {
+        if (msSetErrorFile( value ) != MS_SUCCESS)
+            return MS_FAILURE;
+    }
+
     if( msLookupHashTable( &(map->configoptions), key ) != NULL )
         msRemoveHashTable( &(map->configoptions), key );
     msInsertHashTable( &(map->configoptions), key, value );
+
+    return MS_SUCCESS;
 }
 
 /************************************************************************/
@@ -276,9 +214,7 @@ void msApplyMapConfigOptions( mapObj *map )
         }
         else if( strcasecmp(key,"MS_ERRORFILE") == 0 )
         {
-            char *ms_error = (char *) malloc(strlen(value) + 40);
-            sprintf( ms_error, "MS_ERRORFILE=%s", value);
-            putenv( ms_error ); 
+            msSetErrorFile( value );
         }
         else 
         {
@@ -311,8 +247,11 @@ int msMapSetExtent( mapObj *map,
       
     map->cellsize = msAdjustExtent(&(map->extent), map->width, 
                                    map->height);
-    msCalculateScale(map->extent, map->units, map->width, map->height, 
-                     map->resolution, &(map->scale));
+
+    /* if the map size is also set, recompute scale, ignore errors? */
+    if( map->width != -1 || map->height != -1 )
+        msCalculateScale(map->extent, map->units, map->width, map->height, 
+                         map->resolution, &(map->scaledenom));
 
     return msMapComputeGeotransform( map );
 }
@@ -363,7 +302,7 @@ int msMapComputeGeotransform( mapObj * map )
     /* Do we have all required parameters? */
     if( map->extent.minx == map->extent.maxx 
         || map->width == 0 || map->height == 0 )
-        return MS_FALSE;
+        return MS_FAILURE;
 
     rot_angle = map->gt.rotation_angle * MS_PI / 180.0;
 
@@ -465,7 +404,7 @@ int msMapSetFakedExtent( mapObj *map )
     map->projection.gt.geotransform[5] *= -1;
 
     for(i=0; i<map->numlayers; i++)
-        map->layers[i].project = MS_TRUE;
+        GET_LAYER(map, i)->project = MS_TRUE;
 
     return InvGeoTransform( map->projection.gt.geotransform, 
                             map->projection.gt.invgeotransform );
@@ -493,50 +432,43 @@ int msMapRestoreRealExtent( mapObj *map )
  
 int msInsertLayer(mapObj *map, layerObj *layer, int nIndex) 
 {
-    int i;
-
     if (!layer)
     {
         msSetError(MS_CHILDERR, "Can't insert a NULL Layer", "msInsertLayer()");
         return -1;
     }
 
-    /* Possible to add another? */
-    if (map->numlayers == MS_MAXLAYERS) {
-        msSetError(MS_CHILDERR, "Maximum number of Layer, %d, has been reached",
-                   "msInsertLayer()", MS_MAXLAYERS);
+    /* Ensure there is room for a new layer */
+    if (msGrowMapLayers(map) == NULL)
         return -1;
-    }
-    /* Catch attempt to insert past end of styles array */
-    else if (nIndex >= MS_MAXLAYERS) {
-        msSetError(MS_CHILDERR, "Cannot insert Layer beyond index %d",
-                   "msInsertLayer()", MS_MAXLAYERS-1);
+    /* Catch attempt to insert past end of layers array */
+    else if (nIndex >= map->numlayers) {
+        msSetError(MS_CHILDERR, "Cannot insert layer beyond index %d",
+                   "msInsertLayer()", map->numlayers-1);
         return -1;
     }
     else if (nIndex < 0) { /* Insert at the end by default */
-        initLayer(&(map->layers[map->numlayers]), map);
-        msCopyLayer(&(map->layers[map->numlayers]), layer);
         map->layerorder[map->numlayers] = map->numlayers;
-        map->layers[map->numlayers].index = map->numlayers;
+        GET_LAYER(map, map->numlayers) = layer;
+        GET_LAYER(map, map->numlayers)->index = map->numlayers;
+	GET_LAYER(map, map->numlayers)->map = map;
+	MS_REFCNT_INCR(layer);
         map->numlayers++;
         return map->numlayers-1;
     }
-    else if (nIndex >= 0 && nIndex < MS_MAXLAYERS) {
-    
-        /* Copy layers existing at the specified nIndex or greater */
+    else if (nIndex >= 0 && nIndex < map->numlayers) {
+        /* Move existing layers at the specified nIndex or greater */
         /* to an index one higher */
+        int i;
         for (i=map->numlayers; i>nIndex; i--) {
-            if (i<map->numlayers) freeLayer(&(map->layers[i]));
-            initLayer(&(map->layers[i]), map);
-            msCopyLayer(&(map->layers[i]), &(map->layers[i-1]));
-            map->layers[i].index = i;
+	    GET_LAYER(map, i)=GET_LAYER(map, i-1);
+            GET_LAYER(map, i)->index = i;
         }
 
-        /* copy new layer to specified index */
-        freeLayer(&(map->layers[nIndex]));
-        initLayer(&(map->layers[nIndex]), map);
-        msCopyLayer(&(map->layers[nIndex]), layer);
-        map->layers[nIndex].index = nIndex;
+        /* assign new layer to specified index */
+	GET_LAYER(map, nIndex)=layer;
+        GET_LAYER(map, nIndex)->index = nIndex;
+	GET_LAYER(map, nIndex)->map = map;
 
         /* adjust layers drawing order */
         for (i=map->numlayers; i>nIndex; i--) {
@@ -549,6 +481,7 @@ int msInsertLayer(mapObj *map, layerObj *layer, int nIndex)
         map->layerorder[nIndex] = nIndex;
         
         /* increment number of layers and return */
+	MS_REFCNT_INCR(layer);
         map->numlayers++;
         return nIndex;
     }
@@ -573,26 +506,20 @@ layerObj *msRemoveLayer(mapObj *map, int nIndex)
         return NULL;
     }
     else {
-        /* allocate a copy of the removed layer */
-        layer = (layerObj *) malloc(sizeof(layerObj));
-        if (!layer) {
-            msSetError(MS_MEMERR, 
-                       "Failed to allocate layerObj to return as removed Layer",
-                       "msRemoveLayer");
-            return NULL;
-        }
-        initLayer(layer, NULL);
-        msCopyLayer(layer, &(map->layers[nIndex]));
+    	layer=GET_LAYER(map, nIndex);
+        //msCopyLayer(layer, (GET_LAYER(map, nIndex)));
         
         /* Iteratively copy the higher index layers down one index */
         for (i=nIndex; i<map->numlayers-1; i++) {
-            freeLayer(&(map->layers[i]));
-            initLayer(&(map->layers[i]), map);
-            msCopyLayer(&map->layers[i], &map->layers[i+1]);
-            map->layers[i].index = i;
+            //freeLayer((GET_LAYER(map, i)));
+            //initLayer((GET_LAYER(map, i)), map);
+            //msCopyLayer(GET_LAYER(map, i), GET_LAYER(map, i+1));
+	    GET_LAYER(map, i)=GET_LAYER(map, i+1);
+            GET_LAYER(map, i)->index = i;
         }
         /* Free the extra layer at the end */
-        freeLayer(&(map->layers[map->numlayers-1]));
+        //freeLayer((GET_LAYER(map, map->numlayers-1)));
+        GET_LAYER(map, map->numlayers-1)=NULL;
         
         /* Adjust drawing order */
         order_index = 0;
@@ -610,6 +537,8 @@ layerObj *msRemoveLayer(mapObj *map, int nIndex)
         
         /* decrement number of layers and return copy of removed layer */
         map->numlayers--;
+	layer->map=NULL;
+	MS_REFCNT_DECR(layer);
         return layer;
     }
 }
