@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapogr.cpp 5948 2007-03-03 15:09:24Z tamas $
+ * $Id: mapogr.cpp 6735 2007-08-29 14:59:04Z dmorissette $
  *
  * Name:     mapogr.cpp
  * Project:  MapServer
@@ -17,8 +17,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  * 
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  * 
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -27,150 +27,33 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  * DEALINGS IN THE SOFTWARE.
- **********************************************************************
- * $Log$
- * Revision 1.102.2.1  2007/03/03 15:09:24  tamas
- * Changed OGRLayerGetAutoStyle not to pass NULL pointer to GetRGBFromString causing access violation. (bug 1950)
- *
- * Revision 1.102  2006/09/05 20:39:07  frank
- * added support for multipoint and multiline to wkt translation (bug 1618)
- *
- * Revision 1.101  2006/09/05 15:32:38  frank
- * added support for translating multipoint to ms shapes (bug 1891)
- *
- * Revision 1.100  2006/09/04 17:05:35  dan
- * Added support for MULTIPOLYGON and MULTILINESTRING in msShapeFromWKT()
- * when going through OGR (i.e. GEOS disabled) (bug 1891)
- *
- * Revision 1.99  2006/06/01 20:49:21  dan
- * Enabled support for encrypted connection strings (MS-RFC-18, bug 1792)
- *
- * Revision 1.98  2006/05/29 20:31:20  dan
- * Make sure oPenColor is always initialized (avoid warning on MacOSX)
- *
- * Revision 1.97  2006/05/19 20:53:27  dan
- * Use lp->layerinfo for OGR connections (instead of ogrlayerinfo) (bug 331)
- *
- * Revision 1.96  2006/04/08 03:36:48  frank
- * Ensure that an error in GetNextFeature() will be properly translated
- * into a MapServer error in GetNextShape().
- *
- * Revision 1.95  2006/01/18 04:35:52  frank
- * completed implementation of shape2wkt
- *
- * Revision 1.94  2006/01/17 02:31:51  frank
- * fixed ogr wkt support - bug 1614
- *
- * Revision 1.93  2005/11/01 19:38:13  frank
- * Avoid using _2D geometry API if built with old GDAL/OGR.
- *
- * Revision 1.92  2005/11/01 05:35:50  frank
- * added preliminary implementation of OGR based WKT translation, still untested
- *
- * Revision 1.91  2005/11/01 02:34:24  frank
- * Make a few things static, trim log.
- *
- * Revision 1.90  2005/10/28 02:08:14  frank
- * a few mapogr.cpp funcs used from WFS code, exposed in map.h
- *
- * Revision 1.89  2005/10/28 01:09:42  jani
- * MS RFC 3: Layer vtable architecture (bug 1477)
- *
- * Revision 1.88  2005/06/27 19:52:22  frank
- * Avoid leak of "0 length" point arrays in geometry.
- *
- * Revision 1.87  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.86  2005/04/21 15:09:28  julien
- * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
- *
- * Revision 1.85  2005/04/14 15:17:14  julien
- * Bug 1244: Remove Z and M from point by default to gain performance.
- *
- * Revision 1.84  2005/03/29 17:56:40  frank
- * Insure that tile index reading is restarted in msOGRLayerInitItemInfo()
- * or else fastcgi repeat requests for a layer may fail on subsequent
- * renders.
- *
- * Revision 1.83  2005/03/25 07:01:27  frank
- * construct proper polygon spatial filter
- *
- * Revision 1.82  2005/03/25 05:43:43  frank
- * use msAddLineDirectly() to avoid extra alloc/copy of points
- *
- * Revision 1.81  2004/12/13 07:57:58  frank
- * bug 1126: support passing FILTER to OGR if prefixed with WHERE
- *
- * Revision 1.80  2004/12/13 03:57:17  frank
- * modifed to handle all ogr geometry types in ogrGeomPoints per bug 1124
- *
- * Revision 1.79  2004/11/17 20:40:55  dan
- * Pass config option GML_FIELDTYPES=ALWAYS_STRING to OGR so that all
- * GML attributes are returned as strings to MapServer. (bug 1043)
- *
- * Revision 1.78  2004/11/16 20:36:24  frank
- * msOGCWKT2ProjectionObj(): use SetFromUserInput() for addition capabilities
- *
- * Revision 1.77  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.76  2004/10/28 02:23:17  frank
- * disable direct error reporting from OGR
- *
- * Revision 1.75  2004/10/18 20:56:42  frank
- * fixed msOGRSpatialRef2ProjectionObj to work for non-auto: bug 949
- *
- * Revision 1.74  2004/10/09 18:22:41  sean
- * towards resolving bug 339, have implemented a mutex acquiring wrapper for
- * the loadExpressionString function.  the new msLoadExpressionString should be
- * used everywhere outside of the mapfile loading phase, and the previous
- * loadExpressionString function should be used within the mapfile loading
- * phase.
- *
- * Revision 1.73  2004/08/30 20:12:57  frank
- * added CLOSE_CONNECTION processing parameter - defer closes
- *
- * Revision 1.72  2004/07/05 17:22:00  julien
- * msOGRLayerGetAutoStyle: outlinecolor is only useful if the type is polygon
- *
- * Revision 1.71  2004/06/22 20:55:20  sean
- * Towards resolving issue 737 changed hashTableObj to a structure which contains a hashObj **items.  Changed all hash table access functions to operate on the target table by reference.  msFreeHashTable should not be used on the hashTableObj type members of mapserver structures, use msFreeHashItems instead.
- *
- * Revision 1.70  2004/05/31 20:55:14  frank
- * now reports CPLGetLastErrorMsg() if OGR Open fails
- *
- * Revision 1.69  2004/04/16 20:19:39  dan
- * Added try_addimage_if_notfound to msGetSymbolIndex() (bug 612)
- *
- * Revision 1.68  2004/03/09 17:25:23  frank
- * added msOGRCleanup(), expose msCleanup() to swig
- *
- * Revision 1.67  2004/02/03 23:45:11  assefa
- * Add utility function msOGRGeometryToShape.
- *
- * Revision 1.66  2004/01/06 21:18:02  julien
- * Make STYLEITEM AUTO work with an empty CLASS (Bug 531)
- *
- * ...
- *
- * Revision 1.1  2000/08/25 18:41:05  dan
- * Added optional OGR support
- *
  **********************************************************************/
 
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 #include "mapproject.h"
+#include "mapthread.h"
 
 #if defined(USE_OGR) || defined(USE_GDAL)
 #  include "ogr_spatialref.h"
 #  include "gdal_version.h"
 #  include "cpl_conv.h"
 #  include "cpl_string.h"
+#  include "gdal_version.h"
 #endif
 
-MS_CVSID("$Id: mapogr.cpp 5948 2007-03-03 15:09:24Z tamas $")
+MS_CVSID("$Id: mapogr.cpp 6735 2007-08-29 14:59:04Z dmorissette $")
+
+#if defined(GDAL_VERSION_NUM) && (GDAL_VERSION_NUM < 1400)
+#  define ACQUIRE_OLD_OGR_LOCK   msAcquireLock( TLOCK_OGR )
+#  define RELEASE_OLD_OGR_LOCK   msReleaseLock( TLOCK_OGR )
+#else
+#  define ACQUIRE_OLD_OGR_LOCK 
+#  define RELEASE_OLD_OGR_LOCK 
+#endif
+
+#define ACQUIRE_OGR_LOCK       msAcquireLock( TLOCK_OGR )
+#define RELEASE_OGR_LOCK       msReleaseLock( TLOCK_OGR )
 
 #ifdef USE_OGR
 
@@ -519,9 +402,9 @@ static int ogrGeomLine(OGRGeometry *poGeom, shapeObj *outshp,
   else
   {
       msSetError(MS_OGRERR, 
-                 (char*)CPLSPrintf("OGRGeometry type `%s' not supported.",
-                                   poGeom->getGeometryName()),
-                 "ogrGeomLine()");
+                 "OGRGeometry type `%s' not supported.",
+                 "ogrGeomLine()",
+                 poGeom->getGeometryName() );
       return(-1);
   }
 
@@ -589,6 +472,7 @@ static int ogrConvertGeometry(OGRGeometry *poGeom, shapeObj *outshp,
  *      MS_ANNOTATION layer - return real feature type
  * ------------------------------------------------------------------ */
     case MS_LAYER_ANNOTATION:
+    case MS_LAYER_CHART:
     case MS_LAYER_QUERY:
       switch(poGeom->getGeometryType())
       {
@@ -778,15 +662,19 @@ static int msOGRSpatialRef2ProjectionObj(OGRSpatialReference *poSRS,
   // Export OGR SRS to a PROJ4 string
   char *pszProj = NULL;
 
+  ACQUIRE_OLD_OGR_LOCK;
   if (poSRS->exportToProj4( &pszProj ) != OGRERR_NONE ||
       pszProj == NULL || strlen(pszProj) == 0)
   {
+      RELEASE_OLD_OGR_LOCK;
       msSetError(MS_OGRERR, "Conversion from OGR SRS to PROJ4 failed.",
                  "msOGRSpatialRef2ProjectionObj()");
       CPLFree(pszProj);
       return(MS_FAILURE);
   }
 
+  RELEASE_OLD_OGR_LOCK;
+
   if( debug_flag )
       msDebug( "AUTO = %s\n", pszProj );
 
@@ -820,6 +708,7 @@ int msOGCWKT2ProjectionObj( const char *pszWKT,
     char			*pszAltWKT = (char *) pszWKT;
     OGRErr  eErr;
 
+    ACQUIRE_OLD_OGR_LOCK;
     if( !EQUALN(pszWKT,"GEOGCS",6) 
         && !EQUALN(pszWKT,"PROJCS",6)
         && !EQUALN(pszWKT,"LOCAL_CS",8) )
@@ -827,6 +716,8 @@ int msOGCWKT2ProjectionObj( const char *pszWKT,
     else
         eErr = oSRS.importFromWkt( &pszAltWKT );
 
+    RELEASE_OLD_OGR_LOCK;
+
     if( eErr != OGRERR_NONE )
     {
         msSetError(MS_OGRERR, 
@@ -872,6 +763,8 @@ msOGRFileOpen(layerObj *layer, const char *connection )
  * ------------------------------------------------------------------ */
   if (!bOGRDriversRegistered)
   {
+      ACQUIRE_OGR_LOCK;
+
       OGRRegisterAll();
       CPLPushErrorHandler( CPLQuietErrorHandler );
 
@@ -883,6 +776,8 @@ msOGRFileOpen(layerObj *layer, const char *connection )
       CPLSetConfigOption("GML_FIELDTYPES","ALWAYS_STRING");
 
       bOGRDriversRegistered = MS_TRUE;
+      
+      RELEASE_OGR_LOCK;
   }
 
 /* ------------------------------------------------------------------
@@ -950,6 +845,7 @@ msOGRFileOpen(layerObj *layer, const char *connection )
   if( layer->debug )
       msDebug("msOGRFileOpen(%s)...\n", connection);
 
+
   CPLErrorReset();
   if (msTryBuildPath3(szPath, layer->map->mappath, 
                       layer->map->shapepath, pszDSName) != NULL ||
@@ -958,32 +854,38 @@ msOGRFileOpen(layerObj *layer, const char *connection )
       /* Use relative path */
       if( layer->debug )
           msDebug("OGROPen(%s)\n", szPath);
+
+      ACQUIRE_OGR_LOCK;
 #ifdef USE_SHARED_ACCESS
       poDS = OGRSFDriverRegistrar::GetRegistrar()->OpenShared( szPath );
 #else
       poDS = OGRSFDriverRegistrar::Open( szPath );
 #endif
+      RELEASE_OGR_LOCK;
   }
   else
   {
       /* pszDSName was succesful */
       if( layer->debug )
           msDebug("OGROPen(%s)\n", pszDSName);
+
+      ACQUIRE_OGR_LOCK;
 #ifdef USE_SHARED_ACCESS
       poDS = OGRSFDriverRegistrar::GetRegistrar()->OpenShared( pszDSName );
 #else
       poDS = OGRSFDriverRegistrar::Open( pszDSName );
 #endif
+      RELEASE_OGR_LOCK;
   }
 
   if( poDS == NULL )
   {
       if( strlen(CPLGetLastErrorMsg()) == 0 )
           msSetError(MS_OGRERR, 
-                     (char*)CPLSPrintf("Open failed for OGR connection `%s'.  "
-                                       "File not found or unsupported format.", 
-                                       pszDSName),
-                     "msOGRFileOpen()");
+                     "Open failed for OGR connection `%s'.  "
+                     "File not found or unsupported format.", 
+                     "msOGRFileOpen()",
+                     pszDSName );
       else
           msSetError(MS_OGRERR, 
                      "Open failed for OGR connection `%s'.\n%s\n",
@@ -1027,6 +929,7 @@ msOGRFileOpen(layerObj *layer, const char *connection )
 
   if( poLayer == NULL && EQUALN(pszLayerDef,"SELECT",6) )
   {
+      ACQUIRE_OGR_LOCK;
       poLayer = poDS->ExecuteSQL( pszLayerDef, NULL, NULL );
       if( poLayer == NULL )
       {
@@ -1034,10 +937,12 @@ msOGRFileOpen(layerObj *layer, const char *connection )
                      "ExecuteSQL(%s) failed.\n%s",
                      "msOGRFileOpen()", 
                      pszLayerDef, CPLGetLastErrorMsg() );
-          delete poDS;
+          OGRDataSource::DestroyDataSource( poDS );
           CPLFree( pszLayerDef );
+          RELEASE_OGR_LOCK;
           return NULL;
       }
+      RELEASE_OGR_LOCK;
       nLayerIndex = -1;
   }
 
@@ -1047,7 +952,9 @@ msOGRFileOpen(layerObj *layer, const char *connection )
                  "msOGRFileOpen()", 
                  pszLayerDef, connection );
       CPLFree( pszLayerDef );
-      delete poDS;
+      ACQUIRE_OGR_LOCK;
+      OGRDataSource::DestroyDataSource( poDS );
+      RELEASE_OGR_LOCK;
       return NULL;
   }
 
@@ -1085,8 +992,9 @@ static int msOGRFileClose(layerObj *layer, msOGRFileInfo *psInfo )
 
   CPLFree(psInfo->pszFname);
 
+  ACQUIRE_OGR_LOCK;
   if (psInfo->poLastFeature)
-      delete psInfo->poLastFeature;
+      OGRFeature::DestroyFeature( psInfo->poLastFeature );
 
   /* If nLayerIndex == -1 then the layer is an SQL result ... free it */
   if( psInfo->nLayerIndex == -1 )
@@ -1111,7 +1019,7 @@ static int msOGRFileClose(layerObj *layer, msOGRFileInfo *psInfo )
   }
       
 #else
-  delete psInfo->poDS;
+  OGRDataSource::DestroyDataSource( psInfo->poDS );
 #endif
 
   // Free current tile if there is one.
@@ -1120,6 +1028,8 @@ static int msOGRFileClose(layerObj *layer, msOGRFileInfo *psInfo )
 
   CPLFree(psInfo);
 
+  RELEASE_OGR_LOCK;
+
   return MS_SUCCESS;
 }
 
@@ -1149,6 +1059,8 @@ static int msOGRFileWhichShapes(layerObj *layer, rectObj rect,
  * region and matches the layer's FILTER expression, but there is currently
  * no _efficient_ way to do that with OGR.
  * ------------------------------------------------------------------ */
+  ACQUIRE_OGR_LOCK;
+
   OGRPolygon oSpatialFilter;
   OGRLinearRing oRing;
 
@@ -1181,6 +1093,7 @@ static int msOGRFileWhichShapes(layerObj *layer, rectObj rect,
                      "msOGRFileWhichShapes()",
                      layer->filter.string+6, layer->name, 
                      CPLGetLastErrorMsg() );
+          RELEASE_OGR_LOCK;
           return MS_FAILURE;
       }
   }
@@ -1192,6 +1105,8 @@ static int msOGRFileWhichShapes(layerObj *layer, rectObj rect,
  * ------------------------------------------------------------------ */
   psInfo->poLayer->ResetReading();
 
+  RELEASE_OGR_LOCK;
+
   return MS_SUCCESS;
 }
 
@@ -1260,10 +1175,11 @@ msOGRFileNextShape(layerObj *layer, shapeObj *shape,
   msFreeShape(shape);
   shape->type = MS_SHAPE_NULL;
 
+  ACQUIRE_OGR_LOCK;
   while (shape->type == MS_SHAPE_NULL)
   {
       if( poFeature )
-          delete poFeature;
+          OGRFeature::DestroyFeature(poFeature);
 
       if( (poFeature = psInfo->poLayer->GetNextFeature()) == NULL )
       {
@@ -1271,10 +1187,14 @@ msOGRFileNextShape(layerObj *layer, shapeObj *shape,
           {
               msSetError(MS_OGRERR, "%s", "msOGRFileNextShape()",
                          CPLGetLastErrorMsg() );
+              RELEASE_OGR_LOCK;
               return MS_FAILURE;
           }
           else
+          {
+              RELEASE_OGR_LOCK;
               return MS_DONE;  // No more features to read
+          }
       }
 
       if(layer->numitems > 0) 
@@ -1283,7 +1203,8 @@ msOGRFileNextShape(layerObj *layer, shapeObj *shape,
           shape->numvalues = layer->numitems;
           if(!shape->values)
           {
-              delete poFeature;
+              OGRFeature::DestroyFeature(poFeature);
+              RELEASE_OGR_LOCK;
               return(MS_FAILURE);
           }
       }
@@ -1305,7 +1226,8 @@ msOGRFileNextShape(layerObj *layer, shapeObj *shape,
           else
           {
               msFreeShape(shape);
-              delete poFeature;
+              OGRFeature::DestroyFeature(poFeature);
+              RELEASE_OGR_LOCK;
               return MS_FAILURE; // Error message already produced.
           }
       }
@@ -1320,9 +1242,11 @@ msOGRFileNextShape(layerObj *layer, shapeObj *shape,
 
   // Keep ref. to last feature read in case we need style info.
   if (psInfo->poLastFeature)
-      delete psInfo->poLastFeature;
+      OGRFeature::DestroyFeature(psInfo->poLastFeature);
   psInfo->poLastFeature = poFeature;
 
+  RELEASE_OGR_LOCK;
+
   return MS_SUCCESS;
 }
 
@@ -1352,8 +1276,10 @@ msOGRFileGetShape(layerObj *layer, shapeObj *shape, long record,
   msFreeShape(shape);
   shape->type = MS_SHAPE_NULL;
 
+  ACQUIRE_OGR_LOCK;
   if( (poFeature = psInfo->poLayer->GetFeature(record)) == NULL )
   {
+      RELEASE_OGR_LOCK;
       return MS_FAILURE;
   }
 
@@ -1361,6 +1287,7 @@ msOGRFileGetShape(layerObj *layer, shapeObj *shape, long record,
   if (ogrConvertGeometry(poFeature->GetGeometryRef(), shape,
                          layer->type) != MS_SUCCESS)
   {
+      RELEASE_OGR_LOCK;
       return MS_FAILURE; // Error message already produced.
   }
   
@@ -1369,6 +1296,7 @@ msOGRFileGetShape(layerObj *layer, shapeObj *shape, long record,
       msSetError(MS_OGRERR, 
                  "Requested feature is incompatible with layer type",
                  "msOGRLayerGetShape()");
+      RELEASE_OGR_LOCK;
       return MS_FAILURE;
   }
 
@@ -1379,16 +1307,23 @@ msOGRFileGetShape(layerObj *layer, shapeObj *shape, long record,
   {
       shape->values = msOGRGetValues(layer, poFeature);
       shape->numvalues = layer->numitems;
-      if(!shape->values) return(MS_FAILURE);
+      if(!shape->values)
+      {
+          RELEASE_OGR_LOCK;
+          return(MS_FAILURE);
+      }
+
   }   
 
   shape->index = poFeature->GetFID();
 
   // Keep ref. to last feature read in case we need style info.
   if (psInfo->poLastFeature)
-      delete psInfo->poLastFeature;
+      OGRFeature::DestroyFeature(psInfo->poLastFeature);
   psInfo->poLastFeature = poFeature;
 
+  RELEASE_OGR_LOCK;
+
   return MS_SUCCESS;
 }
 
@@ -1424,6 +1359,7 @@ int msOGRFileReadTile( layerObj *layer, msOGRFileInfo *psInfo,
 /*      We want to start from the beginning even if this file is        */
 /*      shared between layers or renders.                               */
 /* -------------------------------------------------------------------- */
+    ACQUIRE_OGR_LOCK;
     if( targetTile == -2 )
     {
         psInfo->poLayer->ResetReading();
@@ -1449,6 +1385,7 @@ int msOGRFileReadTile( layerObj *layer, msOGRFileInfo *psInfo,
 
     if( poFeature == NULL )
     {
+        RELEASE_OGR_LOCK;
         if( targetTile == -1 )
             return MS_DONE;
         else
@@ -1460,8 +1397,10 @@ int msOGRFileReadTile( layerObj *layer, msOGRFileInfo *psInfo,
     
     nFeatureId = poFeature->GetFID();
 
-    delete poFeature;
+    OGRFeature::DestroyFeature(poFeature);
                         
+    RELEASE_OGR_LOCK;
+
 /* -------------------------------------------------------------------- */
 /*      Open the new tile file.                                         */
 /* -------------------------------------------------------------------- */
@@ -1595,6 +1534,7 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
   if (layer->projection.numargs > 0 && 
       EQUAL(layer->projection.args[0], "auto"))
   {
+      ACQUIRE_OGR_LOCK;
       OGRSpatialReference *poSRS = psInfo->poLayer->GetSpatialRef();
 
       if (msOGRSpatialRef2ProjectionObj(poSRS,
@@ -1603,6 +1543,7 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
       {
           errorObj *ms_error = msGetErrorObj();
 
+          RELEASE_OGR_LOCK;
           msSetError(MS_OGRERR, 
                      "%s  "
                      "PROJECTION AUTO cannot be used for this "
@@ -1615,6 +1556,7 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
           layer->layerinfo = NULL;
           return(MS_FAILURE);
       }
+      RELEASE_OGR_LOCK;
   }
 #endif
 
@@ -2017,12 +1959,15 @@ int msOGRLayerGetExtent(layerObj *layer, rectObj *extent)
  * For tile indexes layers we assume it is sufficient to get the
  * extents of the tile index.
  * ------------------------------------------------------------------ */
+  ACQUIRE_OGR_LOCK;
   if (psInfo->poLayer->GetExtent(&oExtent, TRUE) != OGRERR_NONE)
   {
+      RELEASE_OGR_LOCK;
     msSetError(MS_MISCERR, "Unable to get extents for this layer.", 
                "msOGRLayerGetExtent()");
     return(MS_FAILURE);
   }
+  RELEASE_OGR_LOCK;
 
   extent->minx = oExtent.MinX;
   extent->miny = oExtent.MinY;
@@ -2063,7 +2008,7 @@ static int msOGRGetSymbolId(symbolSetObj *symbolset, const char *pszSymbolId,
     int   nSymbol = -1;
 
     if (pszSymbolId && pszSymbolId[0] != '\0' &&
-        (params = split(pszSymbolId, '.', &numparams))!=NULL)
+        (params = msStringSplit(pszSymbolId, '.', &numparams))!=NULL)
     {
         for(int j=0; j<numparams && nSymbol == -1; j++)
         {
@@ -2118,11 +2063,12 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
 /* ------------------------------------------------------------------
  * Read shape or reuse ref. to last shape read.
  * ------------------------------------------------------------------ */
+  ACQUIRE_OGR_LOCK;
   if (psInfo->poLastFeature == NULL || 
       psInfo->poLastFeature->GetFID() != record)
   {
       if (psInfo->poLastFeature)
-          delete psInfo->poLastFeature;
+          OGRFeature::DestroyFeature(psInfo->poLastFeature);
 
       psInfo->poLastFeature = psInfo->poLayer->GetFeature(record);
   }
@@ -2132,6 +2078,10 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
  * the only members we don't touch are name, expression, and join/query stuff
  * ------------------------------------------------------------------ */
   resetClassStyle(c);
+  if (msMaybeAllocateStyle(c, 0)) {
+      RELEASE_OGR_LOCK;
+      return(MS_FAILURE);
+  }
 
   // __TODO__ label cache incompatible with styleitem feature.
   layer->labelcache = MS_OFF;
@@ -2276,24 +2226,33 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
               if (bIsBrush || layer->type == MS_LAYER_POLYGON)
               {
                   // This is a multipart symbology, so pen defn goes in the
-                  // overlaysymbol params (also set outlinecolor just in case)
-                  c->styles[0].outlinecolor = c->styles[1].outlinecolor = 
-                      oPenColor;
-                  c->styles[1].size = nPenSize;
-                  c->styles[1].symbol = nPenSymbol;
-                  c->numstyles = 2;
+                  // overlaysymbol params 
+                  if (msMaybeAllocateStyle(c, 1))
+                  {
+                      RELEASE_OGR_LOCK;
+                      return(MS_FAILURE);
+                  }
+
+                  c->styles[1]->outlinecolor = oPenColor;
+                  c->styles[1]->size = nPenSize;
+                  c->styles[1]->symbol = nPenSymbol;
               }
               else
               {
                   // Single part symbology
+                  if (msMaybeAllocateStyle(c, 0))
+                  {
+                      RELEASE_OGR_LOCK;
+                      return(MS_FAILURE);
+                  }
+
                   if(layer->type == MS_LAYER_POLYGON)
-                      c->styles[0].outlinecolor = c->styles[0].color = 
+                      c->styles[0]->outlinecolor = c->styles[0]->color = 
                           oPenColor;
                   else
-                      c->styles[0].color = oPenColor;
-                  c->styles[0].symbol = nPenSymbol;
-                  c->styles[0].size = nPenSize;
-                  c->numstyles = 1;
+                      c->styles[0]->color = oPenColor;
+                  c->styles[0]->symbol = nPenSymbol;
+                  c->styles[0]->size = nPenSize;
               }
 
           }
@@ -2304,11 +2263,18 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
               const char *pszBrushName = poBrushStyle->Id(bIsNull);
               if (bIsNull) pszBrushName = NULL;
 
+              /* We need 1 style */
+              if (msMaybeAllocateStyle(c, 0))
+              {
+                  RELEASE_OGR_LOCK;
+                  return(MS_FAILURE);
+              }
+
               // Check for Brush Pattern "ogr-brush-1": the invisible fill
               // If that's what we have then set fill color to -1
               if (pszBrushName && strstr(pszBrushName, "ogr-brush-1") != NULL)
               {
-                  MS_INIT_COLOR(c->styles[0].color, -1, -1, -1);
+                  MS_INIT_COLOR(c->styles[0]->color, -1, -1, -1);
               }
               else
               {
@@ -2316,14 +2282,14 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
                   const char *pszColor = poBrushStyle->ForeColor(bIsNull);
                   if (!bIsNull && poBrushStyle->GetRGBFromString(pszColor,r,g,b,t))
                   {
-                      MS_INIT_COLOR(c->styles[0].color, r, g, b);
+                      MS_INIT_COLOR(c->styles[0]->color, r, g, b);
                       // msDebug("** BRUSH COLOR = %d %d %d (%d)**\n", r,g,b,c->color);
                   }
 
                   pszColor = poBrushStyle->BackColor(bIsNull);
                   if (!bIsNull && poBrushStyle->GetRGBFromString(pszColor,r,g,b,t))
                   {
-                      MS_INIT_COLOR(c->styles[0].backgroundcolor, r, g, b);
+                      MS_INIT_COLOR(c->styles[0]->backgroundcolor, r, g, b);
                   }
 
                   // Symbol name mapping:
@@ -2333,23 +2299,29 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
               
                   const char *pszName = poBrushStyle->Id(bIsNull);
                   if (bIsNull)
-                  pszName = NULL;
-                  c->styles[0].symbol = msOGRGetSymbolId(&(map->symbolset), 
+                      pszName = NULL;
+                  c->styles[0]->symbol = msOGRGetSymbolId(&(map->symbolset), 
                                                          pszName, NULL);
               }
-              c->numstyles = 1;
           }
           else if (poStylePart->GetType() == OGRSTCSymbol)
           {
               OGRStyleSymbol *poSymbolStyle = (OGRStyleSymbol*)poStylePart;
 
+              /* We need 1 style */
+              if (msMaybeAllocateStyle(c, 0))
+              {
+                  RELEASE_OGR_LOCK;
+                  return(MS_FAILURE);
+              }
+
               const char *pszColor = poSymbolStyle->Color(bIsNull);
               if (!bIsNull && poSymbolStyle->GetRGBFromString(pszColor,r,g,b,t))
               {
-                  MS_INIT_COLOR(c->styles[0].color, r, g, b);
+                  MS_INIT_COLOR(c->styles[0]->color, r, g, b);
               }
 
-              c->styles[0].size = (int)poSymbolStyle->Size(bIsNull);
+              c->styles[0]->size = (int)poSymbolStyle->Size(bIsNull);
 
               // Symbol name mapping:
               // First look for the native symbol name, then the ogr-...
@@ -2359,10 +2331,9 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
               if (bIsNull)
                   pszName = NULL;
 
-              c->styles[0].symbol = msOGRGetSymbolId(&(map->symbolset),
+              c->styles[0]->symbol = msOGRGetSymbolId(&(map->symbolset),
                                                     pszName, 
                                                     "default-marker");
-              c->numstyles = 1;
           }
 
           delete poStylePart;
@@ -2374,6 +2345,7 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
 
   }
 
+  RELEASE_OGR_LOCK;
   return MS_SUCCESS;
 #else
 /* ------------------------------------------------------------------
@@ -2395,12 +2367,14 @@ void msOGRCleanup( void )
 
 {
 #if defined(USE_OGR)
+    ACQUIRE_OGR_LOCK;
     if( bOGRDriversRegistered == MS_TRUE )
     {
         delete OGRSFDriverRegistrar::GetRegistrar();
         CPLFinderClean();
         bOGRDriversRegistered = MS_FALSE;
     }
+    RELEASE_OGR_LOCK;
 #endif
 }
 
@@ -2450,10 +2424,12 @@ shapeObj *msOGRShapeFromWKT(const char *string)
     if(!string) 
         return NULL;
     
+    ACQUIRE_OLD_OGR_LOCK;
     if( OGR_G_CreateFromWkt( (char **)&string, NULL, &hGeom ) != OGRERR_NONE )
     {
         msSetError(MS_OGRERR, "Failed to parse WKT string.", 
                    "msOGRShapeFromWKT()" );
+        RELEASE_OLD_OGR_LOCK;
         return NULL;
     }
 
@@ -2468,12 +2444,14 @@ shapeObj *msOGRShapeFromWKT(const char *string)
                               wkbFlatten(OGR_G_GetGeometryType(hGeom)) )
                               == MS_FAILURE )
     {
+        RELEASE_OLD_OGR_LOCK;
         free( shape );
         return NULL;
     }
 
     OGR_G_DestroyGeometry( hGeom );
 
+    RELEASE_OLD_OGR_LOCK;
     return shape;
 #else
   msSetError(MS_OGRERR, "OGR support is not available.","msOGRShapeFromWKT()");
@@ -2494,6 +2472,7 @@ char *msOGRShapeToWKT(shapeObj *shape)
     if(!shape) 
         return NULL;
 
+    ACQUIRE_OLD_OGR_LOCK;
     if( shape->type == MS_SHAPE_POINT && shape->numlines == 1
         && shape->line[0].numpoints == 1 )
     {
@@ -2616,6 +2595,7 @@ char *msOGRShapeToWKT(shapeObj *shape)
         CPLFree( pszOGRWkt );
     }
 
+    RELEASE_OLD_OGR_LOCK;
     return wkt;
 #else
     msSetError(MS_OGRERR, "OGR support is not available.", "msOGRShapeToWKT()");
