@@ -24,242 +24,15 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.67  2006/08/23 18:28:00  dan
- * Use OWS_DEFAULT_SCHEMAS_LOCATION #define instead of hardcoded string (bug 1873)
- *
- * Revision 1.66  2006/08/23 17:52:06  dan
- * schemas.opengeospatial.net has been shutdown, use schemas.opengis.net
- * instead as the default schema repository for OGC services (bug 1873)
- *
- * Revision 1.65  2006/06/06 14:21:01  frank
- * ensure msOWSDispatch() always available
- *
- * Revision 1.64  2006/05/09 14:33:41  assefa
- * WFS client/OWS : correct path concatenation logic for temporary gml file
- * created (bug 1770).
- *
- * Revision 1.63  2006/03/15 18:00:22  assefa
- * Use flag SOS_SVR instead of OGC_SOS (Bug 1712).
- *
- * Revision 1.62  2006/03/14 04:08:34  assefa
- * Add disptach call to SOS service.
- *
- * Revision 1.61  2006/02/14 03:38:47  julien
- * Update to MapContext 1.1.0, add dimensions support in context bug 1581
- *
- * Revision 1.60  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.59  2005/05/24 18:52:45  julien
- * Bug 1149: From WMS 1.1.1, SRS are given in individual tags.
- *
- * Revision 1.58  2005/04/12 23:13:20  sean
- * use non static strings for temp values in msOWSGetLayerExtent() and msOWSGetEPSGProj() (bug 1311).
- *
- * Revision 1.57  2005/02/25 21:32:12  frank
- * fix msOWSGetLayerExtent() for bug 1118
- *
- * Revision 1.56  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.55  2004/12/21 15:59:14  dan
- * Do not include port number in online resource if it's http/80 (bug 1075)
- *
- * Revision 1.54  2004/11/28 02:51:02  frank
- * blow an error if any WxS services are enabled without PROJ
- *
- * Revision 1.53  2004/11/25 06:19:05  dan
- * Add trailing "?" or "&" to connection string when required in WFS
- * client layers using GET method (bug 1082)
- *
- * Revision 1.52  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.51  2004/11/15 21:10:38  dan
- * No need to call msLayerOpen/Close before/after msLayerGetExtent() any more
- * (bug 1051)
- *
- * Revision 1.50  2004/11/09 17:59:05  julien
- * Allow use of msOWSPrintURLType with no metadata. (For bug 1011)
- *
- * Revision 1.49  2004/10/29 22:18:54  assefa
- * Use ows_schama_location metadata. The default value if metadata is not found
- * is http://schemas.opengeospatial.net
- *
- * Revision 1.48  2004/10/27 19:18:25  julien
- * msOWSPrintURLType: Encode string before asssigning end buffer. (Bug 944)
- *
- * Revision 1.47  2004/10/26 21:54:00  dan
- * Poor attempt at clarifying the function header docs for msOWSPrintURLType()
- *
- * Revision 1.46  2004/10/26 15:19:00  julien
- * msOWSPrintURLType: use default values inside the given format. (Bug 944)
- *
- * Revision 1.45  2004/10/25 17:30:38  julien
- * Print function for OGC URLs components. msOWSPrintURLType() (Bug 944)
- *
- * Revision 1.44  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.43  2004/10/04 17:02:28  frank
- * fixed handling of default_value in msOWSPrintValidateMetadata
- *
- * Revision 1.42  2004/10/01 19:14:47  frank
- * use msIO_ calls
- *
- * Revision 1.41  2004/09/30 13:01:43  julien
- * Fix a typo in contact information encoding
- *
- * Revision 1.40  2004/09/28 20:43:27  frank
- * avoid warnings
- *
- * Revision 1.39  2004/09/27 19:11:15  dan
- * Fixed small leak in msOWSParseVersionString()
- *
- * Revision 1.38  2004/09/25 17:16:31  julien
- * Don't encode XML tag (Bug 897)
- * Don't compile mapgml.c function if not necessary (Bug 896)
- *
- * Revision 1.37  2004/09/23 19:18:10  julien
- * Encode all metadata and parameter printed in an XML document (Bug 802)
- *
- * Revision 1.36  2004/08/03 23:26:24  dan
- * Cleanup OWS version tests in the code, mapwms.c (bug 799)
- *
- * Revision 1.35  2004/08/03 22:12:34  dan
- * Cleanup OWS version tests in the code, started with mapcontext.c (bug 799)
- *
- * Revision 1.34  2004/06/22 20:55:20  sean
- * Towards resolving issue 737 changed hashTableObj to a structure which contains a hashObj **items.  Changed all hash table access functions to operate on the target table by reference.  msFreeHashTable should not be used on the hashTableObj type members of mapserver structures, use msFreeHashItems instead.
- *
- * Revision 1.33  2004/05/14 05:32:44  sdlime
- * An assert(FALSE) in mapows.c was failing at build, changed to MS_FALSE...
- *
- * Revision 1.32  2004/05/12 20:59:48  dan
- * Fixed typo in OWS namespace code in msOWSGetLayerExtent() and added an
- * assert() in msOWSLookupMetadata() to catch that in the future (bug 661)
- *
- * Revision 1.31  2004/05/03 03:45:42  dan
- * Include map= param in default onlineresource of GetCapabilties if it
- * was explicitly set in QUERY_STRING (bug 643)
- *
- * Revision 1.30  2004/04/23 16:17:53  frank
- * avoid const warnings
- *
- * Revision 1.29  2004/04/21 13:02:42  sdlime
- * Updated msOWSPrintMetadataList() so that NULL values for startTag/endTag
- * are valid.
- *
- * Revision 1.28  2004/04/20 05:41:20  sdlime
- * Getting very close to a usable WCS implementation. Still need to add 
- * domain and range set to DescribeCoverage, and need to be able to interpret
- * requests based on them. However, we'll keep it simple for now, operating
- * on bands and some temporal subsetting.
- *
- * Revision 1.26  2004/04/14 07:31:40  dan
- * Removed msOWSGetMetadata(), replaced by msOWSLookupMetadata()
- *
- * Revision 1.25  2004/04/14 05:14:54  dan
- * Added ability to pass a default value to msOWSPrintMetadataList() (bug 616)
- *
- * Revision 1.24  2004/04/14 04:54:30  dan
- * Created msOWSLookupMetadata() and added namespaces lookup in all
- * msOWSPrint*Metadata() functions. Also pass namespaces=NULL everywhere
- * that calls those functions for now to avoid breaking something just
- * before the release. (bug 615, 568)
- *
- * Revision 1.23  2004/03/30 00:04:49  dan
- * Cleaned up changelog
- *
- * Revision 1.22  2004/02/26 18:38:08  frank
- * fixed last change
- *
- * Revision 1.21  2004/02/26 16:13:24  frank
- * Correct msOWSGetLayerExtent() error message.
- *
- * Revision 1.20  2004/02/26 16:08:49  frank
- * Added check for wcs_extent.
- *
- * Revision 1.19  2004/02/24 06:20:37  sdlime
- * Added msOWSGetMetadata() function.
- *
- * Revision 1.18  2004/02/05 04:40:01  sdlime
- * Added WCS to the OWS request broker function. The WCS request handler 
- * just returns MS_DONE for now.
- *
- * Revision 1.17  2003/09/19 21:54:19  assefa
- * Add support fot the Post request.
- *
- * Revision 1.16  2003/07/29 20:24:18  dan
- * Fixed problem with invalid BoundingBox tag in WMS capabilities (bug 34)
- *
- * Revision 1.15  2003/02/05 04:40:10  sdlime
- * Removed shapepath as an argument from msLayerOpen and msSHPOpenFile. The 
- * shapefile opening routine now expects just a filename. So, you must use 
- * msBuildPath or msBuildPath3 to create a full qualified filename. Relatively
- * simple change, but required lots of changes. Demo still works...
- *
- * Revision 1.14  2003/01/10 06:39:06  sdlime
- * Moved msEncodeHTMLEntities() and msDecodeHTMLEntities() from mapows.c to
- * mapstring.c so they can be used a bit more freely.
- *
- * Revision 1.13  2002/12/20 03:43:03  frank
- * ensure this builds without libcurl
- *
- * Revision 1.12  2002/12/19 06:30:59  dan
- * Enable caching WMS/WFS request using tmp filename built from URL
- *
- * Revision 1.11  2002/12/19 05:17:09  dan
- * Report WFS exceptions, and do not fail on WFS requests returning 0 features
- *
- * Revision 1.10  2002/12/18 16:45:49  dan
- * Fixed WFS capabilities to validate against schema
- *
- * Revision 1.9  2002/12/13 00:57:31  dan
- * Modified WFS implementation to behave more as a real vector data source
- *
- * Revision 1.8  2002/11/20 21:22:32  dan
- * Added msOWSGetSchemasLocation() for use by both WFS and WMS Map Context
- *
- * Revision 1.7  2002/11/20 17:17:21  julien
- * Support version 0.1.2 of MapContext
- * Remove warning from tags
- * Encode and decode all url
- *
- * Revision 1.6  2002/11/19 02:27:04  dan
- * Fixed unterminated buffer in msEncodeHTMLEntities()
- *
- * Revision 1.5  2002/11/07 21:16:45  julien
- * Fix warning in ContactInfo
- *
- * Revision 1.4  2002/11/07 15:46:45  julien
- * Print ContactInfo just if necessary
- *
- * Revision 1.3  2002/10/28 20:31:20  dan
- * New support for WMS Map Context (from Julien)
- *
- * Revision 1.2  2002/10/22 20:03:57  julien
- * Add the mapcontext support
- *
- * Revision 1.2  2002/10/04 21:29:41  dan
- * WFS: Added GetCapabilities and basic GetFeature (still some work to do)
- *
- * Revision 1.1  2002/09/03 03:19:51  dan
- * Set the bases for WFS Server support + moved some WMS/WFS stuff to mapows.c
- *
- **********************************************************************/
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
 #include <ctype.h> /* isalnum() */
 #include <stdarg.h> 
 #include <assert.h>
 
-MS_CVSID("$Id: mapows.c 5639 2006-08-23 18:28:00Z dan $")
+MS_CVSID("$Id: mapows.c 6734 2007-08-29 14:41:10Z dmorissette $")
 
 /*
 ** msOWSDispatch() is the entry point for any OWS request (WMS, WFS, ...)
@@ -271,36 +44,65 @@ MS_CVSID("$Id: mapows.c 5639 2006-08-23 18:28:00Z dan $")
 */
 int msOWSDispatch(mapObj *map, cgiRequestObj *request)
 {
-    int status = MS_DONE;
+    int i, status = MS_DONE;
+    const char *service=NULL;
 
     if (!request)
       return status;
 
+    for( i=0; i<request->NumParams; i++ ) 
+    {
+        if(strcasecmp(request->ParamNames[i], "VERSION") == 0)
+            service = request->ParamValues[i];
+    }
+
 #ifdef USE_WMS_SVR
     if ((status = msWMSDispatch(map, request)) != MS_DONE )
         return status;
+#else
+    if( service != NULL && strcasecmp(service,"WMS") == 0 )
+        msSetError( MS_WMSERR, 
+                    "SERVICE=WMS requested, but WMS support not configured in MapServer.", 
+                    "msOWSDispatch()" );
 #endif
+
 #ifdef USE_WFS_SVR
     if ((status = msWFSDispatch(map, request)) != MS_DONE )
         return status;
+#else
+    if( service != NULL && strcasecmp(service,"WFS") == 0 )
+        msSetError( MS_WFSERR, 
+                    "SERVICE=WFS requested, but WFS support not configured in MapServer.", 
+                    "msOWSDispatch()" );
 #endif
+
 #ifdef USE_WCS_SVR
     if ((status = msWCSDispatch(map, request)) != MS_DONE )
         return status;
+#else
+    if( service != NULL && strcasecmp(service,"WCS") == 0 )
+        msSetError( MS_WCSERR, 
+                    "SERVICE=WCS requested, but WCS support not configured in MapServer.", 
+                    "msOWSDispatch()" );
 #endif
 
 #ifdef USE_SOS_SVR
     if ((status = msSOSDispatch(map, request)) != MS_DONE )
         return status;
+#else
+    if( service != NULL && strcasecmp(service,"SOS") == 0 )
+        msSetError( MS_SOSERR, 
+                    "SERVICE=SOS requested, but SOS support not configured in MapServer.", 
+                    "msOWSDispatch()" );
 #endif
 
     return MS_DONE;  /* Not a WMS/WFS request... let MapServer handle it */
 }
 
-#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR)
+#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR) || defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
 
 #if !defined(USE_PROJ)
-#error "PROJ.4 is required for WMS, WFS and WCS Server Support."
+#error "PROJ.4 is required for WMS, WFS, WCS and SOS Server Support."
 #endif
 
 /*
@@ -335,22 +137,22 @@ int msOWSMakeAllLayersUnique(mapObj *map)
       int count=1;
       for(j=i+1; j<map->numlayers; j++)
       {
-          if (map->layers[i].name == NULL || map->layers[j].name == NULL)
+          if (GET_LAYER(map, i)->name == NULL || GET_LAYER(map, j)->name == NULL)
           {
               msSetError(MS_MISCERR, 
                          "At least one layer is missing a name in map file.", 
                          "msOWSMakeAllLayersUnique()");
               return MS_FAILURE;
           }
-          if (strcasecmp(map->layers[i].name, map->layers[j].name) == 0 &&
-              msRenameLayer(&(map->layers[j]), ++count) != MS_SUCCESS)
+          if (strcasecmp(GET_LAYER(map, i)->name, GET_LAYER(map, j)->name) == 0 &&
+              msRenameLayer((GET_LAYER(map, j)), ++count) != MS_SUCCESS)
           {
               return MS_FAILURE;
           }
       }
 
       /* Don't forget to rename the first layer if duplicates were found */
-      if (count > 1 && msRenameLayer(&(map->layers[i]), 1) != MS_SUCCESS)
+      if (count > 1 && msRenameLayer((GET_LAYER(map, i)), 1) != MS_SUCCESS)
       {
           return MS_FAILURE;
       }
@@ -513,6 +315,34 @@ const char *msOWSGetSchemasLocation(mapObj *map)
     return schemas_location;
 }
 
+/* msOWSGetLanguage()
+**
+** returns the language via MAP/WEB/METADATA/ows_language
+**
+** Use value of "ows_language" metadata, if not set then
+** return "undefined" as a default
+*/
+const char *msOWSGetLanguage(mapObj *map, const char *context)
+{
+    const char *language;
+
+    /* if this is an exception, MapServer always returns Exception
+       messages in en-US
+    */
+    if (context == "exception") {
+      language = MS_ERROR_LANGUAGE;
+    }
+    /* if not, fetch language from mapfile metadata */
+    else {
+      language = msLookupHashTable(&(map->web.metadata), "ows_language");
+
+      if (language == NULL) {
+        language = "undefined";
+      }
+    }
+    return language;
+}
+
 /* msOWSParseVersionString()
 **
 ** Parse a version string in the format "a.b.c" or "a.b" and return an
@@ -528,7 +358,7 @@ int msOWSParseVersionString(const char *pszVersion)
     if (pszVersion)
     {
         int nVersion = 0;
-        digits = split(pszVersion, '.', &numDigits);
+        digits = msStringSplit(pszVersion, '.', &numDigits);
         if (digits == NULL || numDigits < 2 || numDigits > 3)
         {
             msSetError(MS_WMSERR, 
@@ -820,9 +650,9 @@ int msOWSPrintGroupMetadata(FILE *stream, mapObj *map, char* pszGroupName,
 
     for (i=0; i<map->numlayers; i++)
     {
-       if (map->layers[i].group && (strcmp(map->layers[i].group, pszGroupName) == 0) && &(map->layers[i].metadata))
+       if (GET_LAYER(map, i)->group && (strcmp(GET_LAYER(map, i)->group, pszGroupName) == 0) && &(GET_LAYER(map, i)->metadata))
        {
-         if((value = msOWSLookupMetadata(&(map->layers[i].metadata), namespaces, name)))
+         if((value = msOWSLookupMetadata(&(GET_LAYER(map, i)->metadata), namespaces, name)))
          { 
             encoded = msEncodeHTMLEntities(value);
             msIO_fprintf(stream, format, encoded);
@@ -1141,7 +971,7 @@ int msOWSPrintMetadataList(FILE *stream, hashTableObj *metadata,
       char **keywords;
       int numkeywords;
       
-      keywords = split(value, ',', &numkeywords);
+      keywords = msStringSplit(value, ',', &numkeywords);
       if(keywords && numkeywords > 0) {
         int kw;
 	    if(startTag) msIO_fprintf(stream, "%s", startTag);
@@ -1174,7 +1004,7 @@ int msOWSPrintEncodeMetadataList(FILE *stream, hashTableObj *metadata,
       char **keywords;
       int numkeywords;
       
-      keywords = split(value, ',', &numkeywords);
+      keywords = msStringSplit(value, ',', &numkeywords);
       if(keywords && numkeywords > 0) {
         int kw;
 	    if(startTag) msIO_fprintf(stream, "%s", startTag);
@@ -1208,7 +1038,7 @@ int msOWSPrintEncodeParamList(FILE *stream, const char *name,
     int numitems = 0, i;
 
     if(value && strlen(value) > 0)
-        items = split(value, delimiter, &numitems);
+        items = msStringSplit(value, delimiter, &numitems);
     else
     {
         if (action_if_not_found == OWS_WARN)
@@ -1218,7 +1048,7 @@ int msOWSPrintEncodeParamList(FILE *stream, const char *name,
         }
 
         if (default_value)
-            items = split(default_value, delimiter, &numitems);
+            items = msStringSplit(default_value, delimiter, &numitems);
     }
 
     if(items && numitems > 0)
@@ -1315,24 +1145,19 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
                             int nVersion, hashTableObj *metadata, 
                             const char *namespaces )
 {
-  int bEnableContact = 0;
-
   /* contact information is a required element in 1.0.7 but the */
   /* sub-elements such as ContactPersonPrimary, etc. are not! */
   /* In 1.1.0, ContactInformation becomes optional. */
   if (nVersion > OWS_1_0_0) 
   {
-    if(msOWSLookupMetadata(metadata, namespaces, "contactperson") ||
-       msOWSLookupMetadata(metadata, namespaces, "contactorganization")) 
-    {
-      if(bEnableContact == 0)
-      {
-          msIO_fprintf(stream, "%s<ContactInformation>\n", tabspace);
-          bEnableContact = 1;
-      }
+    msIO_fprintf(stream, "%s<ContactInformation>\n", tabspace); 
 
       /* ContactPersonPrimary is optional, but when present then all its  */
       /* sub-elements are mandatory */
+
+    if(msOWSLookupMetadata(metadata, namespaces, "contactperson") ||
+       msOWSLookupMetadata(metadata, namespaces, "contactorganization")) 
+    {
       msIO_fprintf(stream, "%s  <ContactPersonPrimary>\n", tabspace);
 
       msOWSPrintEncodeMetadata(stream, metadata, namespaces, "contactperson", 
@@ -1343,23 +1168,15 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
       msIO_fprintf(stream, "%s  </ContactPersonPrimary>\n", tabspace);
     }
 
-    if(bEnableContact == 0)
+    if(msOWSLookupMetadata(metadata, namespaces, "contactposition"))
     {
-
-        if(msOWSPrintEncodeMetadata(stream, metadata, namespaces, "contactposition",
-                           OWS_NOERR, 
-     "    <ContactInformation>\n      <ContactPosition>%s</ContactPosition>\n",
-                              NULL) != 0)
-            bEnableContact = 1;
-    }
-    else
-    {
-
-        msOWSPrintEncodeMetadata(stream, metadata, namespaces, "contactposition", 
+      msOWSPrintEncodeMetadata(stream, metadata, namespaces, "contactposition", 
                     OWS_NOERR, "      <ContactPosition>%s</ContactPosition>\n",
                            NULL);
     }
 
+      /* ContactAdress is optional, but when present then all its  */
+      /* sub-elements are mandatory */
     if(msOWSLookupMetadata( metadata, namespaces, "addresstype" ) || 
        msOWSLookupMetadata( metadata, namespaces, "address" ) || 
        msOWSLookupMetadata( metadata, namespaces, "city" ) ||
@@ -1367,14 +1184,6 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
        msOWSLookupMetadata( metadata, namespaces, "postcode" ) ||
        msOWSLookupMetadata( metadata, namespaces, "country" )) 
     {
-      if(bEnableContact == 0)
-      {
-          msIO_fprintf(stream, "%s<ContactInformation>\n", tabspace);
-          bEnableContact = 1;
-      }
-
-      /* ContactAdress is optional, but when present then all its  */
-      /* sub-elements are mandatory */
       msIO_fprintf(stream, "%s  <ContactAddress>\n", tabspace);
 
       msOWSPrintEncodeMetadata(stream, metadata, namespaces,"addresstype", OWS_WARN,
@@ -1392,17 +1201,7 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
       msIO_fprintf(stream, "%s  </ContactAddress>\n", tabspace);
     }
 
-
-    if(bEnableContact == 0)
-    {
-        if(msOWSPrintEncodeMetadata(stream, metadata, namespaces, 
-                                    "contactvoicetelephone", OWS_NOERR,
-                                    "    <ContactInformation>\n"
-                   "      <ContactVoiceTelephone>%s</ContactVoiceTelephone>\n",
-                                    NULL) != 0)
-            bEnableContact = 1;
-    }
-    else
+    if(msOWSLookupMetadata(metadata, namespaces, "contactvoicetelephone"))
     {
         msOWSPrintEncodeMetadata(stream, metadata, namespaces, 
                                  "contactvoicetelephone", OWS_NOERR,
@@ -1410,16 +1209,7 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
                            NULL);
     }
 
-    if(bEnableContact == 0)
-    {
-        if(msOWSPrintEncodeMetadata(stream, metadata,
-                           namespaces, "contactfacsimiletelephone", OWS_NOERR,
-                              "    <ContactInformation>\n     "
-                 "<ContactFacsimileTelephone>%s</ContactFacsimileTelephone>\n",
-                                    NULL) != 0)
-            bEnableContact = 1;
-    }
-    else
+    if(msOWSLookupMetadata(metadata, namespaces, "contactfacsimiletelephone"))
     {
         msOWSPrintEncodeMetadata(stream, metadata, 
                            namespaces, "contactfacsimiletelephone", OWS_NOERR,
@@ -1427,28 +1217,14 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
                                  NULL);
     }
 
-    if(bEnableContact == 0)
-    {
-        if(msOWSPrintEncodeMetadata(stream, metadata, 
-                           namespaces, "contactelectronicmailaddress", OWS_NOERR,
-                              "    <ContactInformation>\n     "
-           "<ContactElectronicMailAddress>%s</ContactElectronicMailAddress>\n",
-                                    NULL) != 0)
-            bEnableContact = 1;
-    }
-    else
+    if(msOWSLookupMetadata(metadata, namespaces, "contactelectronicmailaddress"))
     {
         msOWSPrintEncodeMetadata(stream, metadata, 
                            namespaces, "contactelectronicmailaddress", OWS_NOERR,
          "  <ContactElectronicMailAddress>%s</ContactElectronicMailAddress>\n",
                                  NULL);
     }
-
-
-    if(bEnableContact == 1)
-    {
-        msIO_fprintf(stream, "%s</ContactInformation>\n", tabspace);
-    }
+    msIO_fprintf(stream, "%s</ContactInformation>\n", tabspace);
   }
 }
 
@@ -1469,7 +1245,7 @@ int msOWSGetLayerExtent(mapObj *map, layerObj *lp, const char *namespaces, rectO
     char **tokens;
     int n;
 
-    tokens = split(value, ' ', &n);
+    tokens = msStringSplit(value, ' ', &n);
     if (tokens==NULL || n != 4) {
       msSetError(MS_WMSERR, "Wrong number of arguments for EXTENT metadata.",
                  "msOWSGetLayerExtent()");
@@ -1521,7 +1297,7 @@ int msOWSExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
         {
             layerObj *lp;
 
-            lp = &(map->layers[pasReqInfo[iReq].nLayerId]);
+            lp = GET_LAYER(map, pasReqInfo[iReq].nLayerId);
 
             if (lp->connectiontype == MS_WFS)
                 msWFSUpdateRequestInfo(lp, &(pasReqInfo[iReq]));
