@@ -25,46 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.39  2006/01/16 20:37:15  sdlime
- * Changed label size calls to not adjust baseline offset.
- *
- * Revision 1.38  2006/01/16 20:21:18  sdlime
- * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
- *
- * Revision 1.37  2005/12/14 01:21:38  assefa
- * Add dash line support (bug 492).
- *
- * Revision 1.36  2005/12/12 17:02:07  assefa
- * Segfault on annotation layer when no style is set (Bug 1559)
- *
- * Revision 1.35  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.34  2005/05/19 04:09:34  sdlime
- * Removed the LINE_VERT_THRESHOLD test (bug 564) from PDF/SWF/SVG/imagemap drivers.
- *
- * Revision 1.33  2005/04/21 04:34:03  dan
- * Fixed old problem with labels occasionally drawn upside down (bug 564)
- *
- * Revision 1.32  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.31  2004/12/07 00:38:51  dan
- * Fixed segmentation fault when generating PDF output (bug 1105)
- *
- * Revision 1.30  2004/11/04 21:46:36  frank
- * Commented out unused variable.
- *
- * Revision 1.29  2004/11/04 21:06:09  frank
- * centralize 'stdout binary mode setting' for win32, add for gdal output
- *
- * Revision 1.28  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
 #ifdef USE_PDF
 
@@ -72,14 +33,14 @@
 #if !defined(_WIN32)
 #include <zlib.h>
 #endif
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mappdf.c 5110 2006-01-16 20:37:15Z sdlime $")
+MS_CVSID("$Id: mappdf.c 6693 2007-08-23 12:21:12Z dmorissette $")
 
 /* -------------------------------------------------------------------- */
 /*      prototypes.                                                     */
 /* -------------------------------------------------------------------- */
-/*see map.h*/
+/*see mapserver.h*/
 
 /************************************************************************/
 /*                        PDF *initializeDocument                       */
@@ -126,7 +87,7 @@ int msLoadFontSetPDF(fontSetObj *fontset, PDF *pdf)
     }
 
     i = 0;
-    path = getPath(fontset->filename);
+    path = msGetPath(fontset->filename);
 
     while(fgets(buffer, MS_BUFFER_LENGTH, stream))
     { /* while there's something to load */
@@ -321,21 +282,21 @@ void drawPolylinePDF(PDF *pdf, shapeObj *p, colorObj  *c, double width)
      int i, j;
     int optlistlength=14;
     char *optlist=NULL;
-     for(i=0;i<s->stylelength;i++) {
-       if(!s->style[i]) /*in case length is zero*/ 
+     for(i=0;i<s->patternlength;i++) {
+       if(!s->pattern[i]) /*in case length is zero*/ 
          optlistlength+=2;
        else
-        if(s->style[i]>0)
-         optlistlength+=(int)(log10(s->style[i]))+2;
+        if(s->pattern[i]>0)
+         optlistlength+=(int)(log10(s->pattern[i]))+2;
         else{
-        	msSetError(MS_SYMERR, "Symbol styles must be positive", "drawDashedPolylinePDF()");
-        	return;
-        	}
+          msSetError(MS_SYMERR, "Symbol patterns must be positive", "drawDashedPolylinePDF()");
+          return;
+        }
      }
      optlist = (char*)malloc(optlistlength*sizeof(char));
      sprintf(optlist,"dasharray={");
-     for(i=0;i<s->stylelength;i++)
-       sprintf(optlist,"%s %d",optlist,s->style[i]);
+     for(i=0;i<s->patternlength;i++)
+       sprintf(optlist,"%s %d",optlist,s->pattern[i]);
      sprintf(optlist,"%s }",optlist);
      PDF_setdashpattern(pdf,optlist);
      
@@ -385,7 +346,7 @@ void msDrawLineSymbolPDF(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
     PDF *pdf;
 
     if(style->size == -1) {
-        size = msSymbolGetDefaultSize( &( symbolset->symbol[style->symbol] ) );
+        size = msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
         size = MS_NINT(size*scalefactor);
     }
     else
@@ -406,8 +367,8 @@ void msDrawLineSymbolPDF(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
         return;
 
 #if PDFLIB_MAJORVERSION >= 6
-     if(symbolset && symbolset->symbol[style->symbol].stylelength)
-       drawDashedPolylinePDF(pdf, p, &(symbolset->symbol[style->symbol]), 
+     if(symbolset && symbolset->symbol[style->symbol]->patternlength)
+       drawDashedPolylinePDF(pdf, p, symbolset->symbol[style->symbol], 
                              &(style->color),size);
      else
 #endif
@@ -516,7 +477,7 @@ int msDrawLabelPDF(imageObj *image, pointObj labelPnt, char *string,
 int msDrawLabelCachePDF(imageObj *image, mapObj *map)
 {
     pointObj p;
-    int i, j, l;
+    int i, j, l, priority;
     rectObj r;
     labelCacheMemberObj *cachePtr=NULL;
     layerObj *layerPtr=NULL;
@@ -529,7 +490,7 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
     
     imageObj    *imagetmp = NULL;
 
-    /* -------------------------------------------------------------------- */
+/* -------------------------------------------------------------------- */
 /*      if not PDF, return.                                             */
 /* -------------------------------------------------------------------- */
     if (image == NULL || map == NULL || !MS_DRIVER_PDF(image->format))
@@ -546,13 +507,19 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
         msDrawLabelCacheGD(imagetmp->img.gd, map);
         return 0;
     }
-    for(l=map->labelcache.numlabels-1; l>=0; l--)
+
+    for(priority=MS_MAX_LABEL_PRIORITY-1; priority>=0; priority--)
     {
+      labelCacheSlotObj *cacheslot;
+      cacheslot = &(map->labelcache.slots[priority]);
+
+      for(l=cacheslot->numlabels-1; l>=0; l--)
+      {
         /* point to right spot in cache */
-        cachePtr = &(map->labelcache.labels[l]);
+        cachePtr = &(cacheslot->labels[l]);
 
         /* set a couple of other pointers, avoids nasty references */
-        layerPtr = &(map->layers[cachePtr->layerindex]);
+        layerPtr = (GET_LAYER(map, cachePtr->layerindex));
         /* classPtr = &(cachePtr->class); */
         labelPtr = &(cachePtr->label);
 
@@ -617,65 +584,10 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
                     if(draw_marker)
                         msRectToPolygon(marker_rect, cachePtr->poly);
 
-                    /*check against image first*/
-                    if(!labelPtr->partials)
-                    {
-                        if(labelInImage(map->width,
-                                        map->width,
-                                        cachePtr->poly,
-                                        labelPtr->buffer) == MS_FALSE)
-                        {
-                            cachePtr->status = MS_FALSE;
-                            /*go to next angle*/
-                            continue;
-                        }
-                    }
-
-                    /* compare against points already drawn*/
-                    for(i=0; i<map->labelcache.nummarkers; i++)
-                    {
-                        /* labels can overlap their own marker*/
-                        if(l != map->labelcache.markers[i].id)
-                        {
-                            /* see if polys intersect */
-                            if(intersectLabelPolygons(map->labelcache.markers[i].poly,
-                                                      cachePtr->poly) == MS_TRUE)
-                            {
-                                cachePtr->status = MS_FALSE;
-                                break;
-                            }
-                        }
-                    }
-
-                    /*go to next angle*/
-                    if(!cachePtr->status)
-                        continue;
-
-                    /* compare against rendered labels*/
-                    for(i=l+1; i<map->labelcache.numlabels; i++)
-                    {
-                        /* compare bounding polygons and check for duplicates */
-                        if(map->labelcache.labels[i].status == MS_TRUE)
-                        {
-                            /* check if label is a duplicate */
-                            if((labelPtr->mindistance != -1) &&
-                                (cachePtr->classindex == map->labelcache.labels[i].classindex) &&
-                                (strcmp(cachePtr->text,map->labelcache.labels[i].text) == 0) &&
-                                (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance))
-                            {
-                                cachePtr->status = MS_FALSE;
-                                break;
-                            }
-
-                            /* see if polys intersect */
-                            if(intersectLabelPolygons(map->labelcache.labels[i].poly,
-                                                      cachePtr->poly) == MS_TRUE)
-                            {
-                                cachePtr->status = MS_FALSE;
-                                break;
-                            }
-                        }
-                    }
+                    /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
+                    msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                               map->width, map->height, 
+                                               labelPtr->buffer, cachePtr, priority, l);
 
                     /*found a suitable place for this label*/
                     if(cachePtr->status)
@@ -706,65 +618,10 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
                     if(draw_marker)
                         msRectToPolygon(marker_rect, cachePtr->poly);
 
-                    /*check against image first*/
-                    if(!labelPtr->partials)
-                    {
-                        if(labelInImage(map->width,
-                                        map->height,
-                                        cachePtr->poly,
-                                        labelPtr->buffer) == MS_FALSE)
-                        {
-                            cachePtr->status = MS_FALSE;
-                            /*next position*/
-                            continue;
-                        }
-                    }
-
-                    /* compare against points already drawn*/
-                    for(i=0; i<map->labelcache.nummarkers; i++)
-                    {
-                        /* labels can overlap their own marker*/
-                        if(l != map->labelcache.markers[i].id)
-                        {
-                            /* test if polys intersect */
-                            if(intersectLabelPolygons(map->labelcache.markers[i].poly,
-                                                      cachePtr->poly) == MS_TRUE)
-                            {
-                                cachePtr->status = MS_FALSE;
-                                break;
-                            }
-                        }
-                    }
-
-                    if(!cachePtr->status)
-                        continue; /*go to next position*/
-
-                    /* compare against rendered labels*/
-                    for(i=l+1; i<map->labelcache.numlabels; i++)
-                    {
-                        /* compare bounding polygons and check for duplicates */
-                        if(map->labelcache.labels[i].status == MS_TRUE)
-                        {
-
-                            /* check if label is a duplicate */
-                            if((labelPtr->mindistance != -1) &&
-                               (cachePtr->classindex == map->labelcache.labels[i].classindex) &&
-                               (strcmp(cachePtr->text,map->labelcache.labels[i].text) == 0) &&
-                               (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance))
-                            {
-                                cachePtr->status = MS_FALSE;
-                                break;
-                            }
-
-                            /* polys intersect? */
-                            if(intersectLabelPolygons(map->labelcache.labels[i].poly,
-                                                      cachePtr->poly) == MS_TRUE)
-                            {
-                                cachePtr->status = MS_FALSE;
-                                break;
-                            }
-                        }
-                    }
+                    /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
+                    msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                               map->width, map->height, 
+                                               labelPtr->buffer, cachePtr, priority, l);
 
                     /* found a suitable place for this label*/
                     if(cachePtr->status)
@@ -811,62 +668,10 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
             if(!labelPtr->force)
             { /* no need to check anything else*/
 
-                if(!labelPtr->partials)
-                {
-                    if(labelInImage(map->width,
-                                    map->height,
-                                    cachePtr->poly,
-                                    labelPtr->buffer) == MS_FALSE)
-                        cachePtr->status = MS_FALSE;
-                }
-
-                if(!cachePtr->status)
-                    continue; /*goto next label*/
-
-                /*compare against points already drawn*/
-                for(i=0; i<map->labelcache.nummarkers; i++)
-                {
-                    /* labels can overlap their own marker*/
-                    if(l != map->labelcache.markers[i].id)
-                    {
-                        /* check if polys intersect */
-                        if(intersectLabelPolygons(map->labelcache.markers[i].poly,
-                                                  cachePtr->poly) == MS_TRUE)
-                        {
-                            cachePtr->status = MS_FALSE;
-                            break;
-                        }
-                    }
-                }
-
-                if(!cachePtr->status)
-                    continue; /*goto next label*/
-
-                /* compare against rendered label*/
-                for(i=l+1; i<map->labelcache.numlabels; i++)
-                {
-                    /* compare bounding polygons and check for duplicates */
-                    if(map->labelcache.labels[i].status == MS_TRUE)
-                    {
-                        /* check if label is a duplicate */
-                        if((labelPtr->mindistance != -1) &&
-                           (cachePtr->classindex == map->labelcache.labels[i].classindex) &&
-                           (strcmp(cachePtr->text, map->labelcache.labels[i].text) == 0) &&
-                           (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point))
-                                                        <= labelPtr->mindistance))
-                        {
-                            cachePtr->status = MS_FALSE;
-                            break;
-                        }
-
-                        if(intersectLabelPolygons(map->labelcache.labels[i].poly,
-                                                  cachePtr->poly) == MS_TRUE)
-                        {
-                            cachePtr->status = MS_FALSE;
-                            break;
-                        }
-                    }
-                }
+                /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
+                msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                           map->width, map->height, 
+                                           labelPtr->buffer, cachePtr, priority, l);
             }
         } /* end position if-then-else */
 
@@ -893,7 +698,8 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
         msDrawTextPDF(image, p, cachePtr->text,
                       labelPtr, &(map->fontset), layerPtr->scalefactor);
 
-    } /* next in cache */
+      } /* next in cache */
+    } /* next priority */
 
     return(0);
 }
@@ -909,7 +715,7 @@ void msDrawShadeSymbolPDF(symbolSetObj *symbolset, imageObj *image,
     PDF *pdf;
 
     if(style->size == -1) {
-        size = msSymbolGetDefaultSize( &( symbolset->symbol[style->symbol] ) );
+        size = msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
         size = MS_NINT(size*scalefactor);
     }
     else
@@ -964,7 +770,7 @@ void msDrawMarkerSymbolPDF(symbolSetObj *symbolset, imageObj *image,
                                 (float)(style->color.blue/255));
 
     /*  set up the symbol scale and type */
-    symbol = &(symbolset->symbol[style->symbol]);
+    symbol = symbolset->symbol[style->symbol];
     if(style->size == -1) {
         size = msSymbolGetDefaultSize( symbol );
         size = MS_NINT(size*scalefactor);
@@ -1399,7 +1205,9 @@ int msDrawTextPDF(imageObj *image, pointObj labelPnt, char *string,
     float phi = label->angle;
     colorObj  sColor;
     char *wrappedString;
-    char *fontKey;
+    char *fontPath = NULL;
+    char *fontOutlineAlias = NULL;
+    int nFontOutlineLen = 0;
     PDF *pdf;
 /* -------------------------------------------------------------------- */
 /*      if not PDF, return.                                             */
@@ -1433,9 +1241,9 @@ int msDrawTextPDF(imageObj *image, pointObj labelPnt, char *string,
         return(-1);
     }
 
-    font = msLookupHashTable(&(fontset->fonts), label->font);
+    fontPath = msLookupHashTable(&(fontset->fonts), label->font);
 
-    if(!font)
+    if(!fontPath)
     {
         msSetError(MS_TTFERR, "Requested font (%s) not found.", "msDrawTextPDF()",
                    label->font);
@@ -1494,9 +1302,15 @@ int msDrawTextPDF(imageObj *image, pointObj labelPnt, char *string,
     PDF_setlinewidth(pdf,.3);
 
 /*set up font handling*/
-    fontKey = label->font;
+    /*set an alias for the font outline file*/
+    nFontOutlineLen = strlen(fontPath + 3);
+    fontOutlineAlias = (char *)malloc(sizeof(char)*(nFontOutlineLen));
+    sprintf(fontOutlineAlias, "f1=%s",fontPath);
+    PDF_set_parameter(pdf, "FontOutline", fontOutlineAlias);
+    free(fontOutlineAlias);
 
-    font = PDF_findfont(pdf, fontKey, "winansi",1);
+    /*load font using alias*/
+    font = PDF_load_font(pdf, "f1" ,0 ,"winansi", NULL);
     PDF_setfont(pdf,font,size+2);
 
     if (phi!=0){
