@@ -24,118 +24,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.74  2006/08/13 14:26:52  sdlime
- * Fixed typo in connection encryption addition.
- *
- * Revision 1.73  2006/08/11 16:58:01  dan
- * Added ability to encrypt tokens (passwords, etc.) in database connection
- * strings (MS-RFC-18, bug 1792)
- *
- * Revision 1.72  2006/05/09 22:35:41  pramsey
- * Added quotes around field names being retrieved by query requests. (Bug 1536)
- *
- * Revision 1.71  2006/05/03 22:35:41  pramsey
- * Added schema separation and search path awareness to the LayerRetrievePK. (towards Bug 1571)
- *
- * Revision 1.70  2006/05/02 16:03:36  frank
- * keep track of mycursor and close in layerclose (bug 1757)
- *
- * Revision 1.69  2006/04/28 16:07:53  pramsey
- * Removed HTML tags from PostGIS error messages (Bug 1572)
- *
- * Revision 1.68  2006/02/25 19:11:41  pramsey
- * Added checks for MS_FAILURE around ParseData calls.
- * Patch from Tamas Szekeres
- *
- * Revision 1.67  2006/01/31 10:26:24  umberto
- * Revert behaviour to pre-1.61: do not allow for use of the FILTERITEM attribute (bug 1629)
- *
- * Revision 1.66  2006/01/19 16:04:04  sean
- * move gBYTE_ORDER inside the pg layerinfo structure to allow for differently byte ordered connections (bug 1587).
- *
- *
- * Revision 1.65  2005/12/14 21:25:47  sean
- * test for the case of a selection w/out srid for postgis data and fix from Daryl (bug 1443)
- *
- * Revision 1.64  2005/12/14 17:24:45  sdlime
- * Fixed documentation URL in the generic PostGIS error message. (bug 1558)
- *
- * Revision 1.63  2005/12/11 00:16:29  sean
- * postgis layer test cases and fix for broken view and sub-select layers (bug 1443)
- *
- * Revision 1.62  2005/10/28 01:09:42  jani
- * MS RFC 3: Layer vtable architecture (bug 1477)
- *
- * Revision 1.61  2005/09/15 20:25:39  pramsey
- * Added cases to allow the connector to use both the FILTERITEM and FILTER tags, if present.
- *
- * Revision 1.60  2005/07/16 21:12:59  jerryp
- * Fixed memory leaks.
- *
- * Revision 1.59  2005/07/16 19:07:39  jerryp
- * Bug 1420: PostGIS connector no longer needs two layer close functions.
- *
- * Revision 1.58  2005/06/23 05:11:38  jerryp
- * Fixed a buffer overflow (bug 1392).
- *
- * Revision 1.57  2005/06/23 04:45:19  jerryp
- * Fixed yet another buffer overflow (still bug 1391).
- *
- * Revision 1.56  2005/06/23 04:34:18  jerryp
- * Fixed a buffer overflow (bug 1391).
- *
- * Revision 1.55  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.54  2005/06/07 01:38:50  jerryp
- * Field length call in msPOSTGISLayerRetrievePK was using wrong pqsql function.
- *
- * Revision 1.53  2005/04/26 03:32:46  frank
- * Avoid casting warning.
- *
- * Revision 1.52  2005/04/15 21:10:59  pramsey
- * Applied patch for bug 1306
- *
- * Revision 1.51  2005/03/08 19:39:51  assefa
- * Corrected memeory leak introduced in revision 1.50.
- * Changes in Revison 1.50 were necessary due to portability issue : the
- * code as it was did not build on windows using MSVC.
- *
- * Revision 1.50  2005/03/07 14:55:37  assefa
- * Correct problems when building on Windows.
- *
- * Revision 1.49  2005/03/04 21:54:21  pramsey
- * Provided masking of passwords in the error reporting (bug 703).
- *
- * Revision 1.48  2005/03/04 20:04:34  pramsey
- * Added the ability to identify a tables primary key for use as a unique identifier (Bug 1234)
- *
- * Revision 1.46  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.45  2005/02/06 18:07:16  dave
- * Rolling back the patch.
- *
- * Revision 1.43  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.42  2004/11/05 19:26:05  frank
- * avoid type casting warnings
- *
- * Revision 1.41  2004/10/28 18:26:54  frank
- * comment out the super-noisy debug message in msPOSTGISLayerNextShape().
- *
- * Revision 1.40  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-/* $Id: mappostgis.c 5585 2006-08-13 14:26:52Z sdlime $ */
+/* $Id: mappostgis.c 6477 2007-08-06 15:18:07Z unicoletti $ */
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 #include "maptime.h"
 
 #ifndef FLT_MAX
@@ -155,7 +48,7 @@
 #include <string.h>
 #include <ctype.h> /* tolower() */
 
-MS_CVSID("$Id: mappostgis.c 5585 2006-08-13 14:26:52Z sdlime $")
+MS_CVSID("$Id: mappostgis.c 6477 2007-08-06 15:18:07Z unicoletti $")
 
 typedef struct ms_POSTGIS_layer_info_t
 {
@@ -1171,6 +1064,7 @@ int msPOSTGISLayerGetShapeRandom(layerObj *layer, shapeObj *shape, long *record)
 
                 case MS_LAYER_ANNOTATION:
                 case MS_LAYER_QUERY:
+                case MS_LAYER_CHART:
                     result = dont_force(wkb, shape);
                     break;
 
@@ -1374,6 +1268,7 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
 
             case MS_LAYER_ANNOTATION:
             case MS_LAYER_QUERY:
+            case MS_LAYER_CHART:
                 result = dont_force(wkb,shape);
                 break;
 
@@ -1762,12 +1657,12 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
     pos_sep = strstr(table_name, ".");
     if(pos_sep) 
     {
-      length = pos_sep - table_name;
+      length = strlen(table_name) - strlen(pos_sep);
       schema = (char *)malloc(length + 1);
       strncpy(schema, table_name, length);
       schema[length] = 0;
 
-      length = (int)pos_sep + strlen(pos_sep);
+      length = strlen(pos_sep);
       table = (char *)malloc(length);
       strncpy(table, pos_sep + 1, length - 1);
       table[length - 1] = 0;
@@ -2101,13 +1996,13 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
       tmpstimestring = strdup(timestring);
     else
     {
-        atimes = split (timestring, ',', &numtimes);
+        atimes = msStringSplit (timestring, ',', &numtimes);
         if (atimes == NULL || numtimes < 1)
           return MS_FALSE;
 
         if (numtimes >= 1)
         {
-            tokens = split(atimes[0],  '/', &ntmp);
+            tokens = msStringSplit(atimes[0],  '/', &ntmp);
             if (ntmp == 2) /* ranges */
             {
                 tmpstimestring = strdup(tokens[0]);
@@ -2169,7 +2064,18 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
         if(lp->filteritem) free(lp->filteritem);
         lp->filteritem = strdup(timefield);
         if (&lp->filter)
-          freeExpression(&lp->filter);
+        {
+            /* if the filter is set and it's a string type, concatenate it with
+               the time. If not just free it */
+            if (lp->filter.type == MS_EXPRESSION)
+            {
+                strcat(buffer, "(");
+                strcat(buffer, lp->filter.string);
+                strcat(buffer, ") and ");
+            }
+            else
+              freeExpression(&lp->filter);
+        }
         
 
         strcat(buffer, "(");
@@ -2235,20 +2141,20 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
         return MS_TRUE;
     }
     
-    atimes = split (timestring, ',', &numtimes);
+    atimes = msStringSplit (timestring, ',', &numtimes);
     if (atimes == NULL || numtimes < 1)
       return MS_FALSE;
 
     if (numtimes >= 1)
     {
         /* check to see if we have ranges by parsing the first entry */
-        tokens = split(atimes[0],  '/', &ntmp);
+        tokens = msStringSplit(atimes[0],  '/', &ntmp);
         if (ntmp == 2) /* ranges */
         {
             msFreeCharArray(tokens, ntmp);
             for (i=0; i<numtimes; i++)
             {
-                tokens = split(atimes[i],  '/', &ntmp);
+                tokens = msStringSplit(atimes[i],  '/', &ntmp);
                 if (ntmp == 2)
                 {
                     if (strlen(buffer) > 0)
@@ -2449,7 +2355,16 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
               free(lp->filteritem);
             lp->filteritem = strdup(timefield);     
             if (&lp->filter)
-              freeExpression(&lp->filter);
+            {
+                if (lp->filter.type == MS_EXPRESSION)
+                {
+                    strcat(buffer, "(");
+                    strcat(buffer, lp->filter.string);
+                    strcat(buffer, ") and ");
+                }
+                else
+                  freeExpression(&lp->filter);
+            }
             loadExpressionString(&lp->filter, buffer);
         }
 
