@@ -24,44 +24,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.91  2006/02/02 01:00:11  sdlime
- * Moved the code to free previous query results a bit higher in the msQueryByAttribute() function.
- *
- * Revision 1.90  2006/02/02 00:53:16  sdlime
- * Fixed bug when doing repeated queries where you make a layer not queryable then it is possible the results for that layer can persist. (bug 1550)
- *
- * Revision 1.89  2006/01/01 02:42:39  sdlime
- * The bounds that are part of the result cache are projected so there is no need to do any projection when computing the overall result set bounds.
- *
- * Revision 1.88  2005/12/21 23:31:19  sdlime
- * Fixed an error that occurs when doing attribute queries against layers with a FILTER. The code to cache the existing filter blindly called strdup on a value that could be NULL.
- *
- * Revision 1.87  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.86  2005/04/27 16:52:35  assefa
- * Bug 1335 : missing call to msInitShape in function msQueryByShape.
- *
- * Revision 1.85  2005/04/15 19:32:33  julien
- * Bug 1103: Set the default tolerance value based on the layer type.
- *
- * Revision 1.84  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.83  2004/11/11 19:06:36  sdlime
- * Applied patch for bug 1003 (mapshape and projected layers).
- *
- * Revision 1.82  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapquery.c 5158 2006-02-02 01:00:11Z sdlime $")
+MS_CVSID("$Id: mapquery.c 6600 2007-08-16 14:48:27Z dmorissette $")
 
 /*
 ** msIsLayerQueryable()  returns MS_TRUE/MS_FALSE
@@ -76,7 +43,7 @@ int msIsLayerQueryable(layerObj *lp)
   if(lp->template && strlen(lp->template) > 0) return MS_TRUE;
 
   for(i=0; i<lp->numclasses; i++) {
-    if(lp->class[i].template && strlen(lp->class[i].template) > 0)
+    if(lp->class[i]->template && strlen(lp->class[i]->template) > 0)
         return MS_TRUE;
   }
 
@@ -126,17 +93,17 @@ int msSaveQuery(mapObj *map, char *filename) {
 
   /* count the number of layers with results */
   for(i=0; i<map->numlayers; i++)
-    if(map->layers[i].resultcache) n++;
+    if(GET_LAYER(map, i)->resultcache) n++;
   fwrite(&n, sizeof(int), 1, stream);
 
   /* now write the result set for each layer */
   for(i=0; i<map->numlayers; i++) {
-    if(map->layers[i].resultcache) {
+    if(GET_LAYER(map, i)->resultcache) {
       fwrite(&i, sizeof(int), 1, stream); /* layer index */
-      fwrite(&(map->layers[i].resultcache->numresults), sizeof(int), 1, stream); /* number of results */
-      fwrite(&(map->layers[i].resultcache->bounds), sizeof(rectObj), 1, stream); /* bounding box */
-      for(j=0; j<map->layers[i].resultcache->numresults; j++)
-	fwrite(&(map->layers[i].resultcache->results[j]), sizeof(resultCacheMemberObj), 1, stream); /* each result */
+      fwrite(&(GET_LAYER(map, i)->resultcache->numresults), sizeof(int), 1, stream); /* number of results */
+      fwrite(&(GET_LAYER(map, i)->resultcache->bounds), sizeof(rectObj), 1, stream); /* bounding box */
+      for(j=0; j<GET_LAYER(map, i)->resultcache->numresults; j++)
+	fwrite(&(GET_LAYER(map, i)->resultcache->results[j]), sizeof(resultCacheMemberObj), 1, stream); /* each result */
     }
   }
 
@@ -171,17 +138,17 @@ int msLoadQuery(mapObj *map, char *filename) {
     }
     
     /* inialize the results for this layer */
-    map->layers[j].resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
+    GET_LAYER(map, j)->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
 
-    fread(&(map->layers[j].resultcache->numresults), sizeof(int), 1, stream); /* number of results    */
-    map->layers[j].resultcache->cachesize = map->layers[j].resultcache->numresults;
+    fread(&(GET_LAYER(map, j)->resultcache->numresults), sizeof(int), 1, stream); /* number of results    */
+    GET_LAYER(map, j)->resultcache->cachesize = GET_LAYER(map, j)->resultcache->numresults;
 
-    fread(&(map->layers[j].resultcache->bounds), sizeof(rectObj), 1, stream); /* bounding box */
+    fread(&(GET_LAYER(map, j)->resultcache->bounds), sizeof(rectObj), 1, stream); /* bounding box */
 
-    map->layers[j].resultcache->results = (resultCacheMemberObj *) malloc(sizeof(resultCacheMemberObj)*map->layers[j].resultcache->numresults);
+    GET_LAYER(map, j)->resultcache->results = (resultCacheMemberObj *) malloc(sizeof(resultCacheMemberObj)*GET_LAYER(map, j)->resultcache->numresults);
 
-    for(k=0; k<map->layers[j].resultcache->numresults; k++)
-      fread(&(map->layers[j].resultcache->results[k]), sizeof(resultCacheMemberObj), 1, stream); /* each result */
+    for(k=0; k<GET_LAYER(map, j)->resultcache->numresults; k++)
+      fread(&(GET_LAYER(map, j)->resultcache->results[k]), sizeof(resultCacheMemberObj), 1, stream); /* each result */
   }
 
   fclose(stream);
@@ -202,7 +169,7 @@ int _msQueryByIndex(mapObj *map, int qlayer, int tileindex, int shapeindex,
     return(MS_FAILURE);
   }
 
-  lp = &(map->layers[qlayer]);
+  lp = (GET_LAYER(map, qlayer));
 
   if(!msIsLayerQueryable(lp)) {
     msSetError(MS_QUERYERR, "Requested layer has no templates defined.", "msQueryByIndex()"); 
@@ -240,14 +207,14 @@ int _msQueryByIndex(mapObj *map, int qlayer, int tileindex, int shapeindex,
     return(MS_FAILURE);
   }
 
-  shape.classindex = msShapeGetClass(lp, &shape, map->scale);
-  if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex].status == MS_OFF))) { /* not a valid shape */
+  shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+  if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
     msSetError(MS_NOTFOUND, "Shape %d not valid against layer classification.", "msQueryByIndex()", shapeindex);
     msFreeShape(&shape);
     return(MS_FAILURE);
   }
     
-  if(!(lp->template) && !(lp->class[shape.classindex].template)) { /* no valid template */
+  if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
     msFreeShape(&shape);
     msSetError(MS_NOTFOUND, "Shape does not have a valid template, no way to present results.", "msQueryByIndex()"); 
     return(MS_FAILURE);
@@ -297,7 +264,7 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
     return(MS_FAILURE);
   }
 
-  lp = &(map->layers[qlayer]);
+  lp = (GET_LAYER(map, qlayer));
 
   /* free any previous search results, do now in case one of the following tests fails */
   if(lp->resultcache) {
@@ -325,7 +292,7 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
   }
 
   /* apply the passed query parameters */
-  if(qitem) 
+  if(qitem && qitem[0] != '\0') 
     lp->filteritem = strdup(qitem);
   else
     lp->filteritem = NULL;
@@ -366,13 +333,13 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
   
   while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-    shape.classindex = msShapeGetClass(lp, &shape, map->scale);    
-    if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex].status == MS_OFF))) { /* not a valid shape */
+    shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);    
+    if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
       msFreeShape(&shape);
       continue;
     }
     
-    if(!(lp->template) && !(lp->class[shape.classindex].template)) { /* no valid template */
+    if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
       msFreeShape(&shape);
       continue;
     }
@@ -452,7 +419,7 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
     start = stop = qlayer;
 
   for(l=start; l>=stop; l--) {
-    lp = &(map->layers[l]);
+    lp = (GET_LAYER(map, l));
 
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
@@ -464,9 +431,9 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
     if(!msIsLayerQueryable(lp)) continue;
     if(lp->status == MS_OFF) continue;
 
-    if(map->scale > 0) {
-      if((lp->maxscale > 0) && (map->scale > lp->maxscale)) continue;
-      if((lp->minscale > 0) && (map->scale <= lp->minscale)) continue;
+    if(map->scaledenom > 0) {
+      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
+      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }    
 
     /* Raster layers are handled specially. */
@@ -509,13 +476,13 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
 
     while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-      shape.classindex = msShapeGetClass(lp, &shape, map->scale);
-      if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex].status == MS_OFF))) { /* not a valid shape */
+      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+      if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	msFreeShape(&shape);
 	continue;
       }
 
-      if(!(lp->template) && !(lp->class[shape.classindex].template)) { /* no valid template */
+      if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
 	msFreeShape(&shape);
 	continue;
       }
@@ -566,7 +533,7 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
  
   /* was anything found? */
   for(l=start; l>=stop; l--) {    
-    if(map->layers[l].resultcache && map->layers[l].resultcache->numresults > 0)
+    if(GET_LAYER(map, l)->resultcache && GET_LAYER(map, l)->resultcache->numresults > 0)
       return(MS_SUCCESS);
   }
  
@@ -606,7 +573,7 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
     msSetError(MS_QUERYERR, "Invalid selection layer index.", "msQueryByFeatures()");
     return(MS_FAILURE);
   }
-  slp = &(map->layers[slayer]);
+  slp = (GET_LAYER(map, slayer));
   if(!slp->resultcache) {
     msSetError(MS_QUERYERR, "Selection layer has not been queried.", "msQueryByFeatures()");
     return(MS_FAILURE);
@@ -623,7 +590,7 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
   for(l=start; l>=stop; l--) {
     if(l == slayer) continue; /* skip the selection layer */
     
-    lp = &(map->layers[l]);
+    lp = (GET_LAYER(map, l));
 
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
@@ -635,9 +602,9 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
     if(!msIsLayerQueryable(lp)) continue;    
     if(lp->status == MS_OFF) continue;
     
-    if(map->scale > 0) {
-      if((lp->maxscale > 0) && (map->scale > lp->maxscale)) continue;
-      if((lp->minscale > 0) && (map->scale <= lp->minscale)) continue;
+    if(map->scaledenom > 0) {
+      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
+      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
 
     /* Get the layer tolerance
@@ -725,13 +692,13 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
 	/* check for dups when there are multiple selection shapes */
 	if(i > 0 && is_duplicate(lp->resultcache, shape.index, shape.tileindex)) continue;
 
-	shape.classindex = msShapeGetClass(lp, &shape, map->scale);
-	if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex].status == MS_OFF))) { /* not a valid shape */
+	shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+	if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	  msFreeShape(&shape);
 	  continue;
 	}
 
-	if(!(lp->template) && !(lp->class[shape.classindex].template)) { /* no valid template */
+	if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
 	  msFreeShape(&shape);
 	  continue;
 	}
@@ -806,7 +773,7 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
   for(l=start; l>=stop; l--) {  
     if(l == slayer) continue; /* skip the selection layer */
 
-    if(map->layers[l].resultcache && map->layers[l].resultcache->numresults > 0)
+    if(GET_LAYER(map, l)->resultcache && GET_LAYER(map, l)->resultcache->numresults > 0)
       return(MS_SUCCESS);
   }
 
@@ -836,7 +803,7 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
     start = stop = qlayer;
 
   for(l=start; l>=stop; l--) {
-    lp = &(map->layers[l]);    
+    lp = (GET_LAYER(map, l));    
 
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
@@ -848,9 +815,9 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
     if(!msIsLayerQueryable(lp)) continue;
     if(lp->status == MS_OFF) continue;
 
-    if(map->scale > 0) {
-      if((lp->maxscale > 0) && (map->scale > lp->maxscale)) continue;
-      if((lp->minscale > 0) && (map->scale <= lp->minscale)) continue;
+    if(map->scaledenom > 0) {
+      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
+      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
 
     /* Raster layers are handled specially.  */
@@ -916,13 +883,13 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
 
     while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-      shape.classindex = msShapeGetClass(lp, &shape, map->scale);
-      if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex].status == MS_OFF))) { /* not a valid shape */
+      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+      if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	msFreeShape(&shape);
 	continue;
       }
 
-      if(!(lp->template) && !(lp->class[shape.classindex].template)) { /* no valid template */
+      if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
 	msFreeShape(&shape);
 	continue;
       }
@@ -963,7 +930,7 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
 
   /* was anything found? */
   for(l=start; l>=stop; l--) {    
-    if(map->layers[l].resultcache && map->layers[l].resultcache->numresults > 0)
+    if(GET_LAYER(map, l)->resultcache && GET_LAYER(map, l)->resultcache->numresults > 0)
       return(MS_SUCCESS);
   }
  
@@ -996,7 +963,7 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
   msComputeBounds(selectshape); /* make sure an accurate extent exists */
  
   for(l=start; l>=stop; l--) { /* each layer */
-    lp = &(map->layers[l]);
+    lp = (GET_LAYER(map, l));
 
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
@@ -1008,9 +975,9 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
     if(!msIsLayerQueryable(lp)) continue;
     if(lp->status == MS_OFF) continue;
     
-    if(map->scale > 0) {
-      if((lp->maxscale > 0) && (map->scale > lp->maxscale)) continue;
-      if((lp->minscale > 0) && (map->scale <= lp->minscale)) continue;
+    if(map->scaledenom > 0) {
+      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
+      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
 
     /* Raster layers are handled specially. */
@@ -1076,13 +1043,13 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
 
     while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-      shape.classindex = msShapeGetClass(lp, &shape, map->scale);
-      if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex].status == MS_OFF))) { /* not a valid shape */
+      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+      if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	msFreeShape(&shape);
 	continue;
       }
 
-      if(!(lp->template) && !(lp->class[shape.classindex].template)) { /* no valid template */
+      if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
 	msFreeShape(&shape);
 	continue;
       }
@@ -1142,7 +1109,7 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
 
   /* was anything found? */
   for(l=start; l>=stop; l--) {    
-    if(map->layers[l].resultcache && map->layers[l].resultcache->numresults > 0)
+    if(GET_LAYER(map, l)->resultcache && GET_LAYER(map, l)->resultcache->numresults > 0)
       return(MS_SUCCESS);
   }
  
@@ -1150,6 +1117,227 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
   return(MS_FAILURE);
 }
 
+
+/************************************************************************/
+/*                            msQueryByOperator                         */
+/*                                                                      */
+/*      uary using a shape and a valid operator : all queries are       */
+/*      done using geos.                                                */
+/************************************************************************/
+int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
+                      int geos_operator)
+{
+#ifdef USE_GEOS
+    int start, stop=0, l;
+    shapeObj shape;
+    layerObj *lp;
+    char status;
+    rectObj searchrect;
+
+    msInitShape(&shape);
+
+    /*todo call should be moved somewhere else*/
+    msGEOSSetup();
+
+    /* this should not be a necessary test for uries using geos*/
+    /*
+      if(selectshape->type != MS_SHAPE_POLYGON) {
+      msSetError(MS_QUERYERR, "Search shape MUST be a polygon.", "msQueryByShape()"); 
+      return(MS_FAILURE);
+      }
+    */
+
+    if(qlayer < 0 || qlayer >= map->numlayers)
+      start = map->numlayers-1;
+    else
+      start = stop = qlayer;
+
+    msComputeBounds(selectshape); /* make sure an accurate extent exists */
+ 
+    for(l=start; l>=stop; l--) { /* each layer */
+      lp = (GET_LAYER(map, l));
+
+      /* free any previous search results, do it now in case one of the next few tests fail */
+      if(lp->resultcache) {
+        if(lp->resultcache->results) free(lp->resultcache->results);
+        free(lp->resultcache);
+        lp->resultcache = NULL;
+      }
+
+      if(!msIsLayerQueryable(lp)) continue;
+      if(lp->status == MS_OFF) continue;
+    
+      if(map->scaledenom > 0) {
+        if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
+        if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
+      }
+
+      /* Raster layers are handled specially. */
+      if( lp->type == MS_LAYER_RASTER )
+      {
+        if( msRasterQueryByShape( map, lp, selectshape )
+            == MS_FAILURE )
+          return MS_FAILURE;
+
+        continue;
+      }
+
+    
+      /* open this layer */
+      status = msLayerOpen(lp);
+      if(status != MS_SUCCESS) return(MS_FAILURE);
+
+      /* build item list (no annotation) */
+      status = msLayerWhichItems(lp, MS_TRUE, MS_FALSE, NULL);
+      if(status != MS_SUCCESS) return(MS_FAILURE);
+
+/* identify target shapes */
+      searchrect.minx = map->extent.minx;
+      searchrect.miny = map->extent.miny;
+      searchrect.maxx = map->extent.maxx;
+      searchrect.maxy = map->extent.maxy;
+      
+#ifdef USE_PROJ
+    if(lp->project && msProjectionsDiffer(&(lp->projection), &(map->projection)))
+      msProjectRect(&(map->projection), &(lp->projection), &searchrect); /* project the searchrect to source coords */
+    else
+      lp->project = MS_FALSE;
+#endif
+    status = msLayerWhichShapes(lp, searchrect);
+    if(status == MS_DONE) { /* no overlap */
+      msLayerClose(lp);
+      continue;
+    } else if(status != MS_SUCCESS) {
+      msLayerClose(lp);
+      return(MS_FAILURE);
+    }
+
+      lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
+      lp->resultcache->results = NULL;
+      lp->resultcache->numresults = lp->resultcache->cachesize = 0;
+      lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
+
+      while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
+
+        shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+        if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
+          msFreeShape(&shape);
+          continue;
+        }
+
+        if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
+          msFreeShape(&shape);
+          continue;
+        }
+
+#ifdef USE_PROJ
+        if(lp->project && msProjectionsDiffer(&(lp->projection), &(map->projection)))
+          msProjectShape(&(lp->projection), &(map->projection), &shape);
+        else
+          lp->project = MS_FALSE;
+#endif
+
+        switch(geos_operator) 
+        { 
+          case MS_GEOS_EQUALS:
+            status = msGEOSEquals(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+        
+          case MS_GEOS_DISJOINT:
+            status = msGEOSDisjoint(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+
+          case MS_GEOS_TOUCHES:
+            status = msGEOSTouches(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+
+          case MS_GEOS_OVERLAPS:
+            status = msGEOSOverlaps(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+
+          case MS_GEOS_CROSSES:
+            status = msGEOSCrosses(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+        
+          case MS_GEOS_INTERSECTS:
+            status = msGEOSIntersects(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+
+          case MS_GEOS_WITHIN:
+            status = msGEOSWithin(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+          
+          case MS_GEOS_CONTAINS:
+            status = msGEOSContains(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            break;
+
+            /*beyond is opposite of dwith use in filter encoding
+              see ticket 2105*/
+          case MS_GEOS_BEYOND:
+            status = msGEOSWithin(&shape, selectshape);
+            if (status != MS_TRUE && status != MS_FALSE)
+              status = MS_FALSE;
+            else
+              status = !status;
+          break;
+
+          default:
+            msSetError(MS_QUERYERR, "Unknown GEOS Operator.", "msQueryByOperator()"); 
+            return(MS_FAILURE);
+        }
+
+        if(status == MS_TRUE) {
+          addResult(lp->resultcache, shape.classindex, shape.index, shape.tileindex);
+	
+          if(lp->resultcache->numresults == 1)
+            lp->resultcache->bounds = shape.bounds;
+          else
+            msMergeRect(&(lp->resultcache->bounds), &shape.bounds);
+        }
+
+        msFreeShape(&shape);
+      } /* next shape */
+
+      if(status != MS_DONE) return(MS_FAILURE);
+
+      msLayerClose(lp);
+    } /* next layer */
+
+    /*todo call should be moved somewhere else*/
+    msGEOSCleanup();
+
+    /* was anything found? */
+    for(l=start; l>=stop; l--) {    
+      if(GET_LAYER(map, l)->resultcache && GET_LAYER(map, l)->resultcache->numresults > 0)
+        return(MS_SUCCESS);
+    }
+
+ 
+    msSetError(MS_NOTFOUND, "No matching record(s) found.", "msQueryByOperator()"); 
+    return(MS_FAILURE);
+#else
+  msSetError(MS_GEOSERR, "GEOS support is not available.", "msQueryByOperator()");
+  return(MS_FAILURE);
+#endif
+
+}
+
 /* msGetQueryResultBounds()
  *
  * Compute the BBOX of all query results, returns the number of layers found
@@ -1163,7 +1351,7 @@ int msGetQueryResultBounds(mapObj *map, rectObj *bounds)
   
   for(i=0; i<map->numlayers; i++) {
     layerObj *lp;
-    lp = &(map->layers[i]);
+    lp = (GET_LAYER(map, i));
 
     if(!lp->resultcache) continue;
     if(lp->resultcache->numresults <= 0) continue;
@@ -1199,7 +1387,7 @@ void msQueryFree(mapObj *map, int qlayer)
     start = stop = qlayer;
 
   for(l=start; l>=stop; l--) {
-    lp = &(map->layers[l]);
+    lp = (GET_LAYER(map, l));
         
     if(lp->resultcache) {
       if(lp->resultcache->results) 
