@@ -24,93 +24,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.23  2006/06/23 20:39:19  frank
- * fix time filter propogation to tileindex layer
- *
- * Revision 1.22  2006/06/22 17:32:25  frank
- * Use backtics version of LayerSetTimeFilter so that time
- * queries against layers with direct shapefile tileindexes
- * will work properly.
- *
- * Revision 1.21  2005/10/28 01:09:42  jani
- * MS RFC 3: Layer vtable architecture (bug 1477)
- *
- * Revision 1.20  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.19  2005/04/21 15:09:28  julien
- * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
- *
- * Revision 1.18  2005/04/15 19:32:33  julien
- * Bug 1103: Set the default tolerance value based on the layer type.
- *
- * Revision 1.17  2005/04/14 15:17:14  julien
- * Bug 1244: Remove Z and M from point by default to gain performance.
- *
- * Revision 1.16  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.15  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.14  2004/11/03 16:40:24  frank
- * modified raster queries to properly set the classindex in the resultcache.
- *
- * Revision 1.13  2004/11/02 02:08:30  frank
- * Changed to use "value_list" instead of "values" for a list of all
- * pixel values.  [values] has a special substitution rule in
- * maptemplate.c, so we need to avoid it.
- *
- * Revision 1.12  2004/11/02 01:33:58  frank
- * Added logic to reproject location back to map coordiantes before doing
- * the distance or "in shape" tests.  This also ensures results are returned
- * in map coordianates.
- * http://mapserver.gis.umn.edu/bugs/show_bug.cgi?id=1021
- *
- * Revision 1.11  2004/11/01 21:32:01  frank
- * Added pointer to raster query classification bug.
- *
- * Revision 1.10  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.9  2004/10/13 20:09:24  frank
- * generate a respectible error in msRASTERLayerOpen()
- *
- * Revision 1.8  2004/09/28 20:44:52  frank
- * avoid casting warning
- *
- * Revision 1.7  2004/05/18 16:52:48  frank
- * ensure things build clean without USE_GDAL
- *
- * Revision 1.6  2004/05/13 03:27:07  frank
- * fixed bug in query rect, return point geometries
- *
- * Revision 1.5  2004/05/12 20:55:49  frank
- * don't wipe query results cache between tiles
- *
- * Revision 1.4  2004/04/27 16:53:52  frank
- * added max result support
- *
- * Revision 1.3  2004/04/08 20:32:18  frank
- * now substantially complete
- *
- * Revision 1.2  2004/04/06 18:45:56  frank
- * initial implementation working
- *
- * Revision 1.1  2004/03/09 21:11:08  frank
- * New
- *
- */
+ ****************************************************************************/
 
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: maprasterquery.c 5514 2006-06-23 20:39:19Z frank $")
+MS_CVSID("$Id: maprasterquery.c 6847 2007-09-06 02:30:53Z warmerdam $")
 
 int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile, 
                           long record);
@@ -391,9 +312,9 @@ static void msRasterQueryAddPixel( layerObj *layer, pointObj *location,
         else
         {
             rlinfo->qc_class[rlinfo->query_results] = p_class;
-            red   = layer->class[p_class].styles[0].color.red;
-            green = layer->class[p_class].styles[0].color.green;
-            blue  = layer->class[p_class].styles[0].color.blue;
+            red   = layer->class[p_class]->styles[0]->color.red;
+            green = layer->class[p_class]->styles[0]->color.green;
+            blue  = layer->class[p_class]->styles[0]->color.blue;
         }
     }
 
@@ -594,7 +515,10 @@ msRasterQueryByRectLow(mapObj *map, layerObj *layer, GDALDatasetH hDS,
     eErr = GDALDatasetRasterIO( hDS, GF_Read, 
                                 nWinXOff, nWinYOff, nWinXSize, nWinYSize,
                                 pafRaster, nWinXSize, nWinYSize, GDT_Float32,
-                                nBandCount, panBandMap, 0, 0, 0 );
+                                nBandCount, panBandMap, 
+                                4 * nBandCount, 
+                                4 * nBandCount * nWinXSize, 
+                                4 );
 #else
     /*
      * The above could actually be implemented for pre-1.2.0 GDALs
@@ -1274,8 +1198,8 @@ int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile,
             else if( EQUAL(layer->items[i],"class") && rlinfo->qc_class ) 
             {
                 int p_class = rlinfo->qc_class[record];
-                if( layer->class[p_class].name != NULL )
-                    sprintf( szWork, "%.999s", layer->class[p_class].name );
+                if( layer->class[p_class]->name != NULL )
+                    sprintf( szWork, "%.999s", layer->class[p_class]->name );
                 else
                     sprintf( szWork, "%d", p_class );
             }
@@ -1397,7 +1321,9 @@ int msRASTERLayerSetTimeFilter(layerObj *layer, const char *timestring,
 /*      Otherwise we invoke the tileindex layers SetTimeFilter          */
 /*      method.                                                         */
 /* -------------------------------------------------------------------- */
-    return msLayerSetTimeFilter( layer->map->layers + tilelayerindex,
+	if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+		return MS_FAILURE;
+    return msLayerSetTimeFilter( layer->GET_LAYER(map,tilelayerindex),
                                  timestring, timefield );
 }
 
