@@ -24,117 +24,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.67  2006/03/15 20:31:18  frank
- * Fixup comment for last change.
- *
- * Revision 1.66  2006/03/15 20:29:12  frank
- * Fix inter-tile cracking problem (Bug 1715).
- *
- * Revision 1.65  2005/12/15 16:37:28  frank
- * improve error reporting if proj not compiled in
- *
- * Revision 1.64  2005/11/01 16:53:40  frank
- * fixed MapToSource calculation: bug 1509
- *
- * Revision 1.63  2005/10/26 18:02:37  frank
- * No point in compiling in resampling functions without USE_GDAL.
- *
- * Revision 1.62  2005/10/13 04:02:28  frank
- * Better computation of sDummyMap.cellsize (bug 1493)
- *
- * Revision 1.61  2005/10/07 16:34:53  frank
- * added OVERSAMPLE_RATIO PROCESSING directive
- *
- * Revision 1.60  2005/10/03 14:39:42  frank
- * use alpha blending for nodata antialiasing in bil/aver RGBA case
- *
- * Revision 1.59  2005/09/21 20:12:21  frank
- * Removed prototype bicubic code.
- *
- * Revision 1.58  2005/09/21 20:01:50  frank
- * Fixed bug with RGB offsite support in bilinear/average resampling.
- *
- * Revision 1.57  2005/09/21 01:26:03  frank
- * fixed problem with nearest resampling of float data
- *
- * Revision 1.56  2005/09/21 01:18:10  frank
- * Preliminary RFC4 (alternate resampling kernals) support
- *
- * Revision 1.55  2005/07/21 22:18:16  frank
- * Bug 1372: ensure that raw buffers support multiple bands
- *
- * Revision 1.54  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.53  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.52  2004/10/18 14:49:12  frank
- * implemented msAlphaBlend
- *
- * Revision 1.51  2004/09/28 20:29:18  frank
- * avoid double to int casting warnings
- *
- * Revision 1.50  2004/09/03 15:35:02  frank
- * pass correct map into msImageCreate()
- *
- * Revision 1.49  2004/07/28 21:44:21  assefa
- * Function msImageCreate has an additional argument (map object).
- *
- * Revision 1.48  2004/05/25 15:56:26  frank
- * added rotation/geotransform support
- *
- * Revision 1.47  2004/04/30 19:13:16  dan
- * Fixed problem compiling without GDAL (use of TRUE and CPLAssert)
- *
- * Revision 1.46  2004/03/04 20:08:28  frank
- * added IMAGEMODE_BYTE (raw mode)
- *
- * Revision 1.45  2004/01/30 16:55:42  assefa
- * Fixed a problem while compiling on windows.
- *
- * Revision 1.44  2004/01/29 18:10:16  frank
- * msTransformMapToSource() improved to support using an internal grid if
- * the outer edge has some failures.  Also, now grows the region if some points
- * fail to account for the poor resolution of the grid.
- * Also added LOAD_FULL_RES_IMAGE and LOAD_WHOLE_IMAGE processing directives
- * when computing source image window and resolution.
- *
- * Revision 1.43  2004/01/24 09:50:51  frank
- * Check pj_transform() return values for HUGE_VAL, and handle cases where
- * not all points transform successfully, but some do.
- *
- * Revision 1.42  2003/02/28 20:58:41  frank
- * added preliminary support for the COLOR_MATCH_THRESHOLD
- *
- * Revision 1.41  2003/02/28 20:02:54  frank
- * fixed sizing of srcImage
- *
- * Revision 1.40  2003/02/25 21:41:19  frank
- * fixed bug with asymmetric rounding around zero
- *
- * Revision 1.39  2003/02/24 21:22:52  frank
- * Restructured the source window quite a bit so that input images with a 
- * rotated (or sheared) geotransform would work properly.  Pass RAW_WINDOW to 
- * draw func.
- *
- * Revision 1.38  2003/01/21 04:25:44  frank
- * moved InvGeoTransform to top to predeclare
- *
- * Revision 1.37  2003/01/21 04:16:41  frank
- * allways ensure InvGeoTransform is available
- *
- * Revision 1.36  2003/01/21 04:15:13  frank
- * shift USE_PROJ ifdefs to avoid build warnings without PROJ.4
- */
+ *****************************************************************************/
 
 #include <assert.h>
 #include "mapresample.h"
 
-MS_CVSID("$Id: mapresample.c 5273 2006-03-15 20:32:49Z frank $")
+MS_CVSID("$Id: mapresample.c 6881 2007-09-10 20:21:02Z warmerdam $")
 
 #ifndef MAX
 #  define MIN(a,b)      ((a<b) ? a : b)
@@ -275,6 +170,41 @@ msNearestRasterResampler( imageObj *psSrcImage, colorObj offsite,
                     }
                 }
             }
+#ifdef USE_AGG
+            else if( MS_RENDERER_AGG(psSrcImage->format) )
+            {
+                if( !gdImageTrueColor(psSrcImage->img.gd) )
+                {
+                    nValue = panCMap[srcImg->pixels[nSrcY][nSrcX]];
+
+                    if( nValue == -1 )
+                        continue;
+
+                    nSetPoints++;
+                    dstImg->pixels[nDstY][nDstX] = nValue; 
+                }
+                else
+                {
+                    int nValue = srcImg->tpixels[nSrcY][nSrcX];
+                    int gd_alpha = gdTrueColorGetAlpha(nValue);
+
+                    if( gd_alpha == 0 )
+                    {
+                        nSetPoints++;
+                        dstImg->tpixels[nDstY][nDstX] = nValue;
+                    }
+                    else if( gd_alpha == 127 )
+                        /* overlay is transparent, do nothing */;
+                    else
+                    {
+                        nSetPoints++;
+                        dstImg->tpixels[nDstY][nDstX] = 
+                            gdAlphaBlend( dstImg->tpixels[nDstY][nDstX], 
+                                          nValue );
+                    }
+                }
+            }
+#endif
             else if( MS_RENDERER_RAWDATA(psSrcImage->format) )
             {
                 int band;
@@ -394,6 +324,31 @@ static void msSourceSample( imageObj *psSrcImage, int iSrcX, int iSrcY,
             }
         }
     }
+#ifdef USE_AGG
+    else if( MS_RENDERER_AGG(psSrcImage->format) )
+    {
+        if( !gdImageTrueColor(psSrcImage->img.gd) )
+        {
+            padfPixelSum[0] += 
+                (dfWeight * psSrcImage->img.gd->pixels[iSrcY][iSrcX]);
+            *pdfWeightSum += dfWeight;
+        }
+        else
+        {
+            int nValue = psSrcImage->img.gd->tpixels[iSrcY][iSrcX];
+            int gd_alpha = gdTrueColorGetAlpha(nValue);
+
+            if( gd_alpha != 127 )
+            {
+                padfPixelSum[0] += dfWeight * gdTrueColorGetRed(nValue);
+                padfPixelSum[1] += dfWeight * gdTrueColorGetGreen(nValue);
+                padfPixelSum[2] += dfWeight * gdTrueColorGetBlue(nValue);
+                
+                *pdfWeightSum += dfWeight;
+            }
+        }
+    }
+#endif
     else
     {
         int band;
@@ -556,6 +511,44 @@ msBilinearRasterResampler( imageObj *psSrcImage, colorObj offsite,
                 padfPixelSum[i] /= dfWeightSum;
 
             if( MS_RENDERER_GD(psSrcImage->format) )
+            {
+                if( !gdImageTrueColor(psSrcImage->img.gd) )
+                {
+                    int nResult = panCMap[(int) padfPixelSum[0]];
+                    if( nResult != -1 )
+                    {                        
+                        nSetPoints++;
+                        dstImg->pixels[nDstY][nDstX] = nResult;
+                    }
+                }
+                else
+                {
+                    nSetPoints++;
+                    if( dfWeightSum > 0.99 )
+                        dstImg->tpixels[nDstY][nDstX] = 
+                            gdTrueColor( (int) padfPixelSum[0], 
+                                         (int) padfPixelSum[1], 
+                                         (int) padfPixelSum[2] );
+                    else
+                    {
+                        int gd_color;
+                        int gd_alpha = (int) (127 - 127.9 * dfWeightSum);
+
+                        gd_alpha = MAX(0,MIN(127,gd_alpha));
+                        gd_color = gdTrueColorAlpha(
+                            (int) padfPixelSum[0], 
+                            (int) padfPixelSum[1], 
+                            (int) padfPixelSum[2], 
+                            gd_alpha );
+                        
+                        dstImg->tpixels[nDstY][nDstX] = 
+                            msAlphaBlend( dstImg->tpixels[nDstY][nDstX],
+                                          gd_color );
+                    }
+                }
+            }
+#ifdef USE_AGG
+            else if( MS_RENDERER_AGG(psSrcImage->format) )
             {
                 if( !gdImageTrueColor(psSrcImage->img.gd) )
                 {
@@ -592,6 +585,7 @@ msBilinearRasterResampler( imageObj *psSrcImage, colorObj offsite,
                     }
                 }
             }
+#endif
             else if( MS_RENDERER_RAWDATA(psSrcImage->format) )
             {
                 int band;
@@ -802,7 +796,7 @@ msAverageRasterResampler( imageObj *psSrcImage, colorObj offsite,
                     else
                     {
                         int gd_color;
-                        int gd_alpha = 127 - 127.9 * dfAlpha01;
+                        int gd_alpha = (int) (127 - 127.9 * dfAlpha01);
 
                         gd_alpha = MAX(0,MIN(127,gd_alpha));
                         gd_color = gdTrueColorAlpha(
@@ -818,6 +812,46 @@ msAverageRasterResampler( imageObj *psSrcImage, colorObj offsite,
                     }
                 }
             }
+#ifdef USE_AGG
+            else if( MS_RENDERER_AGG(psSrcImage->format) )
+            {
+                if( !gdImageTrueColor(psSrcImage->img.gd) )
+                {
+                    int nResult = panCMap[(int) padfPixelSum[0]];
+                    if( nResult != -1 )
+                    {                        
+                        nSetPoints++;
+                        dstImg->pixels[nDstY][nDstX] = nResult;
+                    }
+                }
+                else
+                {
+                    nSetPoints++;
+                    if( dfAlpha01 > 0.99 )
+                        dstImg->tpixels[nDstY][nDstX] = 
+                            gdTrueColor( (int) padfPixelSum[0], 
+                                         (int) padfPixelSum[1], 
+                                         (int) padfPixelSum[2] );
+                    else
+                    {
+                        int gd_color;
+                        int gd_alpha = (int) (127 - 127.9 * dfAlpha01);
+
+                        gd_alpha = MAX(0,MIN(127,gd_alpha));
+                        gd_color = gdTrueColorAlpha(
+                            (int) padfPixelSum[0], 
+                            (int) padfPixelSum[1], 
+                            (int) padfPixelSum[2], 
+                            gd_alpha );
+                        
+                        dstImg->tpixels[nDstY][nDstX] = 
+                            msAlphaBlend( dstImg->tpixels[nDstY][nDstX],
+                                          gd_color );
+                        
+                    }
+                }
+            }
+#endif
             else if( MS_RENDERER_RAWDATA(psSrcImage->format) )
             {
                 int band;
@@ -1162,8 +1196,7 @@ static int msApproxTransformer( void *pCBData, int nPoints,
     if( dfError > psATInfo->dfMaxError )
     {
         bSuccess = 
-            psATInfo->pfnBaseTransformer( psATInfo->pBaseCBData, nMiddle, 
-                                          x, y, panSuccess );
+            msApproxTransformer( psATInfo, nMiddle, x, y, panSuccess );
             
         if( !bSuccess )
         {
@@ -1173,10 +1206,8 @@ static int msApproxTransformer( void *pCBData, int nPoints,
         }
 
         bSuccess = 
-            psATInfo->pfnBaseTransformer( psATInfo->pBaseCBData, 
-                                          nPoints - nMiddle, 
-                                          x+nMiddle, y+nMiddle, 
-                                          panSuccess+nMiddle );
+            msApproxTransformer( psATInfo, nPoints - nMiddle, 
+                                 x+nMiddle, y+nMiddle, panSuccess+nMiddle );
 
         if( !bSuccess )
         {
@@ -1621,6 +1652,16 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
         sDummyMap.imagecolor.green = 17;
         sDummyMap.imagecolor.blue = 191;
     }
+#ifdef USE_AGG
+    else if( MS_RENDERER_AGG(sDummyMap.outputformat) 
+        && !gdImageTrueColor( image->img.gd ) )
+    {
+        sDummyMap.outputformat->transparent = MS_TRUE;
+        sDummyMap.imagecolor.red = 117;
+        sDummyMap.imagecolor.green = 17;
+        sDummyMap.imagecolor.blue = 191;
+    }
+#endif
 /* -------------------------------------------------------------------- */
 /*      If we are working in RGB mode ensure we produce an RGBA         */
 /*      image so the transparency can be preserved.                     */
@@ -1638,7 +1679,21 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
         sDummyMap.imagecolor.green = map->imagecolor.green;
         sDummyMap.imagecolor.blue = map->imagecolor.blue;
     }
+#ifdef USE_AGG
+    else if( MS_RENDERER_AGG(sDummyMap.outputformat) 
+             && gdImageTrueColor( image->img.gd ) )
+    {
+        assert( sDummyMap.outputformat->imagemode == MS_IMAGEMODE_RGB
+                || sDummyMap.outputformat->imagemode == MS_IMAGEMODE_RGBA );
 
+        sDummyMap.outputformat->transparent = MS_TRUE;
+        sDummyMap.outputformat->imagemode = MS_IMAGEMODE_RGBA;
+
+        sDummyMap.imagecolor.red = map->imagecolor.red;
+        sDummyMap.imagecolor.green = map->imagecolor.green;
+        sDummyMap.imagecolor.blue = map->imagecolor.blue;
+    }
+#endif
 /* -------------------------------------------------------------------- */
 /*      Setup a dummy map object we can use to read from the source     */
 /*      raster, with the newly established extents, and resolution.     */
@@ -1694,7 +1749,27 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
         for( iColor = nColorCount; iColor < 256; iColor++ )
             anCMap[iColor] = -1;
     }
+#ifdef USE_AGG
+    else if( MS_RENDERER_AGG(srcImage->format)
+        && !gdImageTrueColor( srcImage->img.gd ) )
+    {
+        int  iColor, nColorCount;
 
+        anCMap[0] = -1; /* color zero is always transparent */
+
+        nColorCount = gdImageColorsTotal( srcImage->img.gd );
+        for( iColor = 1; iColor < nColorCount; iColor++ )
+        {
+            anCMap[iColor] = 
+                msAddColorAGG( map, image->img.gd, 0, 
+                              gdImageRed( srcImage->img.gd, iColor ),
+                              gdImageGreen( srcImage->img.gd, iColor ),
+                              gdImageBlue( srcImage->img.gd, iColor ) );
+        }
+        for( iColor = nColorCount; iColor < 256; iColor++ )
+            anCMap[iColor] = -1;
+    }
+#endif
 /* -------------------------------------------------------------------- */
 /*      Setup transformations between our source image, and the         */
 /*      target map image.                                               */
