@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsvg.c 6084 2007-05-03 16:05:17Z Assefa $
+ * $Id: mapsvg.c 6614 2007-08-17 13:22:13Z dmorissette $
  *
  * Name:     mapsvg.c
  * Project:  MapServer
@@ -28,79 +28,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.22  2006/01/16 20:21:18  sdlime
- * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
- *
- * Revision 1.21  2005/10/26 17:53:57  frank
- * Avoid warning about unused function.
- *
- * Revision 1.20  2005/09/26 19:47:14  assefa
- * Correct typo bugs when outputing symbols (Bug 1407)
- *
- * Revision 1.19  2005/07/05 14:36:33  assefa
- * Correct a bug : using msFree on a wrong pointer : Bug 1406.
- *
- * Revision 1.18  2005/06/22 15:16:25  assefa
- * Correct problems with multipolygons (Bug 1390).
- * Remove unecessary spaces when generating the SVG.
- * Contribution to these 2 issues by Makoto Niwa niwa@orkney.co.jp.
- *
- * Revision 1.17  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.16  2005/05/19 04:09:34  sdlime
- * Removed the LINE_VERT_THRESHOLD test (bug 564) from PDF/SWF/SVG/imagemap drivers.
- *
- * Revision 1.15  2005/04/21 04:34:03  dan
- * Fixed old problem with labels occasionally drawn upside down (bug 564)
- *
- * Revision 1.14  2005/04/13 15:55:55  dan
- * Added missing include stdarg.h (bug 1315)
- *
- * Revision 1.13  2005/04/07 21:49:24  frank
- * use MS_PI instead of M_PI, available everywhere
- *
- * Revision 1.12  2005/04/07 17:23:16  assefa
- * Remove #ifdef USE_SVG. It was added during development.
- *
- * Revision 1.11  2005/03/07 15:58:10  assefa
- * Use gzip function to output svgz.
- *
- * Revision 1.10  2005/03/02 04:23:36  assefa
- * Partial support of symbols (vector and ellipse).
- *
- * Revision 1.9  2005/02/21 22:22:32  assefa
- * Add code for GOSVG_LATLONG_TRANSFORMATION.
- *
- * Revision 1.8  2005/02/18 18:14:11  dan
- * New patch to fix url strings broken by previous patch to bug 1238
- *
- * Revision 1.7  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.6  2005/02/11 22:14:08  frank
- * Fixed imagePolyline declaration.
- *
- * Revision 1.5  2005/02/09 20:43:49  assefa
- * Add SVG utility function msSaveImagetoFpSVG.
- *
- * Revision 1.4  2005/02/09 18:30:50  assefa
- * goSVG specific metadata added.
- *
- * Revision 1.3  2005/02/05 00:07:14  assefa
- * Add support for lables with encoding.
- *
- * Revision 1.2  2005/02/03 00:07:39  assefa
- * Add partial text and raster support.
- *
- * Revision 1.1  2005/02/01 23:18:54  assefa
- * Initial version : svg output.
- *
- *
- */
+ ****************************************************************************/
 
 
 #include <stdarg.h>
@@ -109,9 +37,9 @@
 #ifdef USE_ZLIB
 #include "zlib.h"
 #endif
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapsvg.c 6084 2007-05-03 16:05:17Z Assefa $")
+MS_CVSID("$Id: mapsvg.c 6614 2007-08-17 13:22:13Z dmorissette $")
 
 
 static int msIO_fprintfgz(FILE *fp, int bCompressed, const char *format, ... )
@@ -415,7 +343,7 @@ static void imagePolyline(FILE *fp, int bCompressed, shapeObj *p,
                 else
                   sprintf(szTmp, "%d", symbolstyle[k]);
                 
-                pszDashArray = strcatalloc(pszDashArray, szTmp);
+                pszDashArray = msStringConcatenate(pszDashArray, szTmp);
             }
             
             msIO_fprintfgz(fp, bCompressed, "<polyline fill=\"none\" stroke=\"#%02x%02x%02x\" stroke-width=\"%d\" stroke-dasharray=\"%s\" points=\"",color->red, color->green, 
@@ -597,7 +525,7 @@ MS_DLL_EXPORT void msDrawLineSymbolSVG(symbolSetObj *symbolset,
     if(p->numlines <= 0) return;
 
     if(style->size == -1) {
-      size = (int)msSymbolGetDefaultSize( &( symbolset->symbol[style->symbol] ) );
+      size = (int)msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
     }
     else
       size = style->size;
@@ -624,7 +552,7 @@ MS_DLL_EXPORT void msDrawLineSymbolSVG(symbolSetObj *symbolset,
     ox = MS_NINT(style->offsetx*scalefactor);
     oy = (style->offsety == -99) ? -99 : (int)(style->offsety*scalefactor);
 
-    symbol = &(symbolset->symbol[style->symbol]);
+    symbol = symbolset->symbol[style->symbol];
 
 
     /* TODO : all lines are draw as filled lines.  */
@@ -635,11 +563,11 @@ MS_DLL_EXPORT void msDrawLineSymbolSVG(symbolSetObj *symbolset,
     if(style->symbol == 0) 
       imagePolyline(image->img.svg->stream, image->img.svg->compressed, 
                     p, &style->color, width,
-                    symbol->stylelength,  symbol->style);
+                    symbol->patternlength,  symbol->pattern);
     else
       imagePolyline(image->img.svg->stream, image->img.svg->compressed, 
                     p, &style->color, (int)size,
-                    symbol->stylelength,  symbol->style);
+                    symbol->patternlength,  symbol->pattern);
         return;
     
 
@@ -688,11 +616,11 @@ static void imageFillPolygon(FILE *fp, int bCompressed,
     {
         if (i == 0)
         {
-            pszDashArray = strcatalloc(pszDashArray,"");
+            pszDashArray = msStringConcatenate(pszDashArray,"");
             if (symbolstylelength > 0 && symbolstyle)
             {
                 sprintf(szTmp, "stroke-dasharray=\"");
-                pszDashArray = strcatalloc(pszDashArray, szTmp);
+                pszDashArray = msStringConcatenate(pszDashArray, szTmp);
                 for (k=0; k<symbolstylelength; k++)
                 {
                     if (k < symbolstylelength-1)
@@ -700,7 +628,7 @@ static void imageFillPolygon(FILE *fp, int bCompressed,
                     else
                       sprintf(szTmp, "%d\"", symbolstyle[k]);
 
-                    pszDashArray = strcatalloc(pszDashArray, szTmp);
+                    pszDashArray = msStringConcatenate(pszDashArray, szTmp);
                 }
             }
             if (max > 2)
@@ -783,11 +711,11 @@ void msDrawShadeSymbolSVG(symbolSetObj *symbolset, imageObj *image,
       return;
 
 
-    symbol = &(symbolset->symbol[style->symbol]);
+    symbol = symbolset->symbol[style->symbol];
 
     if(style->size == -1) 
     {
-        size = (int)msSymbolGetDefaultSize( &( symbolset->symbol[style->symbol] ) );
+        size = (int)msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
         size = MS_NINT(size*scalefactor);
     }
     else
@@ -816,7 +744,7 @@ void msDrawShadeSymbolSVG(symbolSetObj *symbolset, imageObj *image,
       psOutlineColor = &sOc;
 
     imageFillPolygon(image->img.svg->stream, image->img.svg->compressed ,p, psFillColor, psOutlineColor, size,
-                     symbol->stylelength,  symbol->style);
+                     symbol->patternlength,  symbol->pattern);
 
 }
 
@@ -939,24 +867,24 @@ static void drawSVGText(FILE *fp, int bCompressed, int x, int y,
     /*    char *next; */
     /*    int len, i, ch; */
 
-    pszFontStyleString = strcatalloc(pszFontStyleString, "");
-    pszFontWeightString = strcatalloc(pszFontWeightString, "");
-    pszAngleString = strcatalloc(pszAngleString, "");
-    pszAngleAnchorString = strcatalloc(pszAngleAnchorString, "");
+    pszFontStyleString = msStringConcatenate(pszFontStyleString, "");
+    pszFontWeightString = msStringConcatenate(pszFontWeightString, "");
+    pszAngleString = msStringConcatenate(pszAngleString, "");
+    pszAngleAnchorString = msStringConcatenate(pszAngleAnchorString, "");
 
     if (pszFontStyle)
     {
         sprintf(szTmp, " font-style=\"%s\"", pszFontStyle);
-        pszFontStyleString = strcatalloc(pszFontStyleString, szTmp);
+        pszFontStyleString = msStringConcatenate(pszFontStyleString, szTmp);
     }
 
     if (pszFontWeight)
     {
         sprintf(szTmp, " font-weight=\"%s\"", pszFontWeight);
-        pszFontWeightString = strcatalloc(pszFontWeightString, szTmp);
+        pszFontWeightString = msStringConcatenate(pszFontWeightString, szTmp);
     }
  
-    pszFillString =  strcatalloc(pszFillString, "");
+    pszFillString =  msStringConcatenate(pszFillString, "");
     if (psColor)
     {
         if (MS_VALID_COLOR(*psColor))
@@ -965,15 +893,15 @@ static void drawSVGText(FILE *fp, int bCompressed, int x, int y,
         else
           sprintf(szTmp," fill=\"none\"");
 
-        pszFillString = strcatalloc(pszFillString, szTmp);
+        pszFillString = msStringConcatenate(pszFillString, szTmp);
     }
 
-    pszStrokeString =  strcatalloc(pszStrokeString, "");
+    pszStrokeString =  msStringConcatenate(pszStrokeString, "");
     if (psOutlineColor && MS_VALID_COLOR(*psOutlineColor))
     {
         sprintf(szTmp, " stroke=\"#%02x%02x%02x\" stroke-width=\"0.5\"",
                 psOutlineColor->red, psOutlineColor->green  , psOutlineColor->blue);
-        pszStrokeString = strcatalloc(pszStrokeString, szTmp);
+        pszStrokeString = msStringConcatenate(pszStrokeString, szTmp);
     }
 
     /* angle */
@@ -981,7 +909,7 @@ static void drawSVGText(FILE *fp, int bCompressed, int x, int y,
     {
         sprintf(szTmp, " transform=\"rotate(%f %d %d)\"",
                 -dfAngle, x, y);
-        pszAngleString = strcatalloc(pszAngleString, szTmp);
+        pszAngleString = msStringConcatenate(pszAngleString, szTmp);
     }
 
     /* anchor point */
@@ -989,24 +917,24 @@ static void drawSVGText(FILE *fp, int bCompressed, int x, int y,
         nAnchorPosition == MS_LL)
     {
         sprintf(szTmp, " text-anchor=\"end\"");
-        pszAngleAnchorString = strcatalloc(pszAngleAnchorString, szTmp);
+        pszAngleAnchorString = msStringConcatenate(pszAngleAnchorString, szTmp);
     }
     else if (nAnchorPosition == MS_UC || nAnchorPosition == MS_CC ||
              nAnchorPosition == MS_LC)
     {
         sprintf(szTmp, " text-anchor=\"middle\"");
-        pszAngleAnchorString = strcatalloc(pszAngleAnchorString, szTmp);
+        pszAngleAnchorString = msStringConcatenate(pszAngleAnchorString, szTmp);
     }
     else if (nAnchorPosition == MS_UR || nAnchorPosition == MS_CR ||
              nAnchorPosition == MS_LR)
     {
         sprintf(szTmp, " text-anchor=\"start\"");
-        pszAngleAnchorString = strcatalloc(pszAngleAnchorString, szTmp);
+        pszAngleAnchorString = msStringConcatenate(pszAngleAnchorString, szTmp);
     }
 
 /*    if (bEncoding)
     {
-        pszTmpStr = strcatalloc(pszTmpStr, string);
+        pszTmpStr = msStringConcatenate(pszTmpStr, string);
         next = pszTmpStr;
         for (i=0; *next; i++)
         {
@@ -1017,7 +945,7 @@ static void drawSVGText(FILE *fp, int bCompressed, int x, int y,
             next += len;
         
             sprintf(szTmp, "&#x%x;",ch);
-            pszFinalString = strcatalloc(pszFinalString, szTmp);
+            pszFinalString = msStringConcatenate(pszFinalString, szTmp);
         }
     }
     else*/
@@ -1071,10 +999,7 @@ int msDrawTextSVG(imageObj *image, pointObj labelPnt, char *string,
       return (0);
 
     if(label->encoding != NULL ) 
-    { /* converting the label encoding */
-        
-        string = msGetEncodedString(string, label->encoding);
-        if(string == NULL) return(-1);
+    { /* label encoding is set. expect UTF-8 as input */
         bEncoding = 1;
     }
 
@@ -1155,7 +1080,7 @@ int msDrawTextSVG(imageObj *image, pointObj labelPnt, char *string,
 /*        - font-weight (normal | bold | bolder | lighter | 100 | 200 ..*/
 /* -------------------------------------------------------------------- */
         /* parse font string :  */
-        aszFontsParts = split(label->font, '_', &nFontParts);
+        aszFontsParts = msStringSplit(label->font, '_', &nFontParts);
  
         pszFontFamily = aszFontsParts[0];
         if (nFontParts == 3)
@@ -1197,7 +1122,7 @@ int msDrawTextSVG(imageObj *image, pointObj labelPnt, char *string,
 int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
 {
   pointObj p;
-  int i, j, l;
+  int i, j, l, priority;
   rectObj r;
   
   labelCacheMemberObj *cachePtr=NULL;
@@ -1215,11 +1140,15 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
       return (0);
 
   
-  for(l=map->labelcache.numlabels-1; l>=0; l--) {
+  for(priority=MS_MAX_LABEL_PRIORITY-1; priority>=0; priority--) {
+   labelCacheSlotObj *cacheslot;
+   cacheslot = &(map->labelcache.slots[priority]);
 
-    cachePtr = &(map->labelcache.labels[l]); /* point to right spot in the label cache */
+   for(l=cacheslot->numlabels-1; l>=0; l--) {
 
-    layerPtr = &(map->layers[cachePtr->layerindex]); /* set a couple of other pointers, avoids nasty references */
+    cachePtr = &(cacheslot->labels[l]); /* point to right spot in the label cache */
+
+    layerPtr = (GET_LAYER(map, cachePtr->layerindex)); /* set a couple of other pointers, avoids nasty references */
     labelPtr = &(cachePtr->label);
 
     if(!cachePtr->text || strlen(cachePtr->text) == 0)
@@ -1262,39 +1191,11 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
 
-	  if(!labelPtr->partials) { /* check against image first */
-	    if(labelInImage(map->width, map->height, cachePtr->poly, labelPtr->buffer) == MS_FALSE) {
-	      cachePtr->status = MS_FALSE;
-	      continue; /* next angle */
-	    }
-	  }
+          /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
+          msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                     map->width, map->height, 
+                                     labelPtr->buffer, cachePtr, priority, l);
 
-	  for(i=0; i<map->labelcache.nummarkers; i++) { /* compare against points already drawn */
-	    if(l != map->labelcache.markers[i].id) { /* labels can overlap their own marker */
-	      if(intersectLabelPolygons(map->labelcache.markers[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
-
-	  if(!cachePtr->status)
-	    continue; /* next angle */
-
-	  for(i=l+1; i<map->labelcache.numlabels; i++) { /* compare against rendered labels */
-	    if(map->labelcache.labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
-
-	      if((labelPtr->mindistance != -1) && (cachePtr->classindex == map->labelcache.labels[i].classindex) && (strcmp(cachePtr->text,map->labelcache.labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-
-	      if(intersectLabelPolygons(map->labelcache.labels[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
 
 	  if(cachePtr->status) /* found a suitable place for this label */
 	    break;
@@ -1312,39 +1213,10 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
 
-	  if(!labelPtr->partials) { /* check against image first */
-	    if(labelInImage(map->width, map->height, cachePtr->poly, labelPtr->buffer) == MS_FALSE) {
-	      cachePtr->status = MS_FALSE;
-	      continue; /* next position */
-	    }
-	  }
-
-	  for(i=0; i<map->labelcache.nummarkers; i++) { /* compare against points already drawn */
-	    if(l != map->labelcache.markers[i].id) { /* labels can overlap their own marker */
-	      if(intersectLabelPolygons(map->labelcache.markers[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
-
-	  if(!cachePtr->status)
-	    continue; /* next position */
-
-	  for(i=l+1; i<map->labelcache.numlabels; i++) { /* compare against rendered labels */
-	    if(map->labelcache.labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
-
-	      if((labelPtr->mindistance != -1) && (cachePtr->classindex == map->labelcache.labels[i].classindex) && (strcmp(cachePtr->text,map->labelcache.labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-
-	      if(intersectLabelPolygons(map->labelcache.labels[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-		cachePtr->status = MS_FALSE;
-		break;
-	      }
-	    }
-	  }
+          /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
+          msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                     map->width, map->height, 
+                                     labelPtr->buffer, cachePtr, priority, l);
 
 	  if(cachePtr->status) /* found a suitable place for this label */
 	    break;
@@ -1366,39 +1238,10 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
 
       if(!labelPtr->force) { /* no need to check anything else */
 
-	if(!labelPtr->partials) {
-	  if(labelInImage(map->width, map->height, cachePtr->poly, labelPtr->buffer) == MS_FALSE)
-	    cachePtr->status = MS_FALSE;
-	}
-
-	if(!cachePtr->status)
-	  continue; /* next label */
-
-	for(i=0; i<map->labelcache.nummarkers; i++) { /* compare against points already drawn */
-	  if(l != map->labelcache.markers[i].id) { /* labels can overlap their own marker */
-	    if(intersectLabelPolygons(map->labelcache.markers[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */
-	      cachePtr->status = MS_FALSE;
-	      break;
-	    }
-	  }
-	}
-
-	if(!cachePtr->status)
-	  continue; /* next label */
-
-	for(i=l+1; i<map->labelcache.numlabels; i++) { /* compare against rendered label */
-	  if(map->labelcache.labels[i].status == MS_TRUE) { /* compare bounding polygons and check for duplicates */
-	    if((labelPtr->mindistance != -1) && (cachePtr->classindex == map->labelcache.labels[i].classindex) && (strcmp(cachePtr->text, map->labelcache.labels[i].text) == 0) && (msDistancePointToPoint(&(cachePtr->point), &(map->labelcache.labels[i].point)) <= labelPtr->mindistance)) { /* label is a duplicate */
-	      cachePtr->status = MS_FALSE;
-	      break;
-	    }
-
-	    if(intersectLabelPolygons(map->labelcache.labels[i].poly, cachePtr->poly) == MS_TRUE) { /* polys intersect */          
-	      cachePtr->status = MS_FALSE;
-	      break;
-	    }
-	  }
-	}
+        /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
+        msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
+                                   map->width, map->height, 
+                                   labelPtr->buffer, cachePtr, priority, l);
       }
     } /* end position if-then-else */
 
@@ -1416,7 +1259,8 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
     /* if(MS_VALID_COLOR(labelPtr->backgroundcolor)) billboardGD(img, cachePtr->poly, labelPtr); */
     msDrawTextSVG(image, p, cachePtr->text, labelPtr, &(map->fontset), layerPtr->scalefactor); /* actually draw the label */
 
-  } /* next label */
+   } /* next label */
+  } /* next priority */
 
   
   return(0);
@@ -1455,11 +1299,11 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
    
     if(!p) return;
 
-    symbol = &(symbolset->symbol[style->symbol]);
+    symbol = symbolset->symbol[style->symbol];
 
  
     if(style->size == -1) {
-    size = msSymbolGetDefaultSize( &( symbolset->symbol[style->symbol] ) );
+    size = msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
     size = MS_NINT(size*scalefactor);
     } else
       size = MS_NINT(style->size*scalefactor);
@@ -1507,7 +1351,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
 /*        - font-weight (normal | bold | bolder | lighter | 100 | 200 ..*/
 /* -------------------------------------------------------------------- */
             /* parse font string :  */
-            aszFontsParts = split(symbol->font, '_', &nFontParts);
+            aszFontsParts = msStringSplit(symbol->font, '_', &nFontParts);
  
             pszFontFamily = aszFontsParts[0];
             if (nFontParts == 3)
@@ -1547,26 +1391,26 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
             y = MS_NINT(p->y  + style->offsety);
 
             /*TODO : style->angle */
-            pszFill = strcatalloc(pszFill,"");
+            pszFill = msStringConcatenate(pszFill,"");
             if (MS_VALID_COLOR(style->color) && symbol->filled)
             {
                 sprintf(szTmp, "fill=\"#%02x%02x%02x\"",style->color.red, 
                         style->color.green,
                         style->color.blue);
-                pszFill = strcatalloc(pszFill, szTmp);
+                pszFill = msStringConcatenate(pszFill, szTmp);
             }
             else
             {
-                pszFill = strcatalloc(pszFill,"fill=\"none\"");
+                pszFill = msStringConcatenate(pszFill,"fill=\"none\"");
                 bFillSetToNone =1;
             }
-            pszStroke = strcatalloc(pszStroke, "");
+            pszStroke = msStringConcatenate(pszStroke, "");
             if (MS_VALID_COLOR(style->outlinecolor))
             {
                 sprintf(szTmp, "stroke=\"#%02x%02x%02x\"",style->outlinecolor.red, 
                         style->outlinecolor.green,
                         style->outlinecolor.blue);
-                pszStroke = strcatalloc(pszStroke, szTmp);
+                pszStroke = msStringConcatenate(pszStroke, szTmp);
             } 
             else if (bFillSetToNone)
             {
@@ -1574,7 +1418,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
                 the outline color is not set, set the stroke to black. 
                 This is the way the gd outputs reacts to this case */
                 sprintf(szTmp, "stroke=\"#%02x%02x%02x\"",0,0,0);
-                pszStroke = strcatalloc(pszStroke, szTmp);
+                pszStroke = msStringConcatenate(pszStroke, szTmp);
             }
             
 
@@ -1590,28 +1434,28 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
             offset_x = MS_NINT(p->x - d*.5*symbol->sizex +  style->offsetx);
             offset_y = MS_NINT(p->y - d*.5*symbol->sizey +  style->offsety);
 
-            pszFill = strcatalloc(pszFill,"");
+            pszFill = msStringConcatenate(pszFill,"");
             if (MS_VALID_COLOR(style->color) && symbol->filled)
             {
                 sprintf(szTmp, "fill=\"#%02x%02x%02x\"",
                         style->color.red, 
                         style->color.green,
                         style->color.blue);
-                pszFill = strcatalloc(pszFill, szTmp);
+                pszFill = msStringConcatenate(pszFill, szTmp);
             }
             else
             {
                 bFillSetToNone = 1;
-                pszFill = strcatalloc(pszFill,"fill=\"none\"");
+                pszFill = msStringConcatenate(pszFill,"fill=\"none\"");
             }
-            pszStroke = strcatalloc(pszStroke, "");
+            pszStroke = msStringConcatenate(pszStroke, "");
             if (MS_VALID_COLOR(style->outlinecolor))
             {
                 sprintf(szTmp, "stroke=\"#%02x%02x%02x\"",
                         style->outlinecolor.red, 
                         style->outlinecolor.green,
                         style->outlinecolor.blue);
-                pszStroke = strcatalloc(pszStroke, szTmp);
+                pszStroke = msStringConcatenate(pszStroke, szTmp);
             }
             else if (bFillSetToNone)
             {
@@ -1619,7 +1463,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
                 the outline color is not set, set the stroke to black. 
                 This is the way the gd outputs reacts to this case */
                 sprintf(szTmp, "stroke=\"#%02x%02x%02x\"",0,0,0);
-                pszStroke = strcatalloc(pszStroke, szTmp);
+                pszStroke = msStringConcatenate(pszStroke, szTmp);
             }
             if (width <= 0)
               width = 1;
