@@ -24,123 +24,17 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.100.2.1  2007/02/15 06:16:41  sdlime
- * Applied patch for bug 1978 (roating transparent pixmaps).
- *
- * Revision 1.100  2006/07/23 03:28:45  sdlime
- * Fixed error in msAddImageSymbol() where imagepath is not set. (bug 1832)
- *
- * Revision 1.99  2006/04/27 04:05:18  sdlime
- * Initial support for relative coordinates. (bug 1547)
- *
- * Revision 1.98  2006/03/30 03:58:04  sdlime
- * Fixed what I hope are the last of the routines that create PIXMAP symbols but don't set symbol sizex and sizey. (bug 1725)
- *
- * Revision 1.97  2006/03/21 06:27:08  sdlime
- * Made sure msRotateSymbol sets the new symbol's sizex and sizey members for PIXMAP symbols.
- *
- * Revision 1.96  2006/03/16 05:28:44  sdlime
- * Cleaned up a build warning in mapsymbol.c.
- *
- * Revision 1.95  2006/03/16 04:45:24  sdlime
- * Reverted to old means of scaling symbols based solely on height. Fixed possiblity of memory leak with symbol rotation. Made rotation and scaling behavior more consistent across all GD rendering functions (point, line, polygon and circle). (bugs 1684 and 1705)
- *
- * Revision 1.94  2006/03/02 16:43:44  jani
- * This fixes second part of bug 1684, which happens when image is loaded
- * by symbol file.
- *
- * * loadSymbol: initialize sizex, sizey when loading image as symbol.
- *
- * Revision 1.93  2006/03/02 01:51:13  jani
- * When we add a pixmap symbol, be sure to initialize sizex,sizey fields of
- * symbol. These fields are used later (e.g.mapgd.c:msDrawMarkerSymbolGD)
- * to calculate scaling factor.  If they are uninitialized (0.0),
- * the scaling factor will be 1. This worked before with the old code because
- * msDrawMarkerSymbolGD used directly img->sy when it calculated
- * d (the scaling factor).
- *
- * This fixes bug #1684
- *
- * * msAddImageSymbol: Initialize sizex, sizey
- *
- * Revision 1.92  2006/02/18 21:04:20  hobu
- * warning nanny on line 1223, unused variable
- *
- * Revision 1.91  2006/01/16 19:46:35  sean
- * close handle when saving symbolset, pointed out by Albert Rovira
- *
- * Revision 1.90  2006/01/09 18:04:19  frank
- * fix for gd calls when different heaps in use - win32 (bug 1513)
- *
- * Revision 1.89  2006/01/03 03:19:05  sdlime
- * Rotation for pixmap symbols is *very* close but transparency is not handled correctly for 8-bit images. I think this is a bug in GD but I can't be sure yet. Rotated images look crappy anyway so this is just a start.
- *
- * Revision 1.88  2005/12/15 05:50:30  sdlime
- * Symbol writer ignores type SIMPLE. It doesn't anymore...
- *
- * Revision 1.87  2005/11/17 06:13:23  sdlime
- * Fixed symbol set copying so that the image cache is not copied and the destination symbol set cache is initialized properly. (bug 1521)
- *
- * Revision 1.86  2005/11/05 05:34:41  sdlime
- * Removed misplaced closing of msyyin in loadSymbol(). (bug 178)
- *
- * Revision 1.85  2005/10/14 05:04:12  sdlime
- * Added msRotateSymbol(), changed freeSymbol to a public function called msFreeSymbol() in mapsymbol.c. Added to map.h as well.
- *
- * Revision 1.84  2005/09/21 04:23:27  sdlime
- * Updated msLoadImageSymbol() to clean-up any previously allocated values for imagepath and img. (bug 1472)
- *
- * Revision 1.83  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.82  2005/05/10 13:18:25  dan
- * Fixed several issues with writeSymbol() (bug 1344)
- *
- * Revision 1.81  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.80  2005/01/28 06:16:54  sdlime
- * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
- *
- * Revision 1.79  2004/11/19 17:38:41  sean
- * final fix to bug 1074, image transparency preserved through msSymbolSetImageGD
- * and symbolObj::setImage.
- *
- * Revision 1.78  2004/11/19 03:59:20  sean
- * Fix to msSymbolSetImageGD so that pixmap transparency is preserved.  Renamed
- * the msGDSetImage and GetImage to msSymbolSetImageGD and msSymbolGetImageGD each
- * with a symbolObj* as the first argument (bug 1074).
- *
- * Revision 1.77  2004/11/11 05:42:07  sean
- * use gd's gdioctx in msLoadImageSymbol (bug 1047)
- *
- * Revision 1.76  2004/11/10 19:50:23  sean
- * Use gd's fileIOCtx in loadSymbol with gdImageCreate*Ctx instead of gdImageCreate*
- *
- * Revision 1.75  2004/11/09 16:04:54  frank
- * avoid warning, avoid casting size to int before scaling
- *
- * Revision 1.74  2004/10/27 18:13:40  sean
- * msCopySymbol now properly copies symbol images so that cloned maps render
- * exactly as the original maps (bug 931).
- *
- * Revision 1.73  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ *****************************************************************************/
 
 #include <stdarg.h> /* variable number of function arguments support */
 #include <time.h> /* since the parser handles time/date we need this */
 
-#include "map.h"
+#include "mapserver.h"
 #include "mapfile.h"
 #include "mapcopy.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapsymbol.c 5913 2007-02-15 06:16:41Z sdlime $")
+MS_CVSID("$Id: mapsymbol.c 6789 2007-09-05 14:04:06Z sdlime $")
 
 extern int msyylex(void); /* lexer globals */
 extern void msyyrestart(FILE *);
@@ -154,7 +48,7 @@ extern unsigned char JPEGsig[3];
 
 /*
 ** Symbol to string static arrays needed for writing map files.
-** Must be kept in sync with enumerations and defines found in map.h.
+** Must be kept in sync with enumerations and defines found in mapserver.h.
 */
 extern char *msPositionsText[]; /* Defined in mapfile.c */
 static char *msCapsJoinsCorners[7]={"NONE", "BEVEL", "BUTT", "MITER", "ROUND", "SQUARE", "TRIANGLE"};
@@ -228,10 +122,11 @@ double msSymbolGetDefaultSize(symbolObj *s) {
 
 void initSymbol(symbolObj *s)
 {
+  MS_REFCNT_INIT(s);
   s->type = MS_SYMBOL_VECTOR;
   s->transparent = MS_FALSE;
   s->transparentcolor = 0;
-  s->stylelength = 0; /* solid line */
+  s->patternlength = 0; /* solid line */
   s->sizex = 0;
   s->sizey = 0;
   s->filled = MS_FALSE;
@@ -243,7 +138,7 @@ void initSymbol(symbolObj *s)
   s->inmapfile = MS_FALSE;
   s->antialias = MS_FALSE;
   s->font = NULL;
-  s->character = '\0';
+  s->character = NULL;
   s->position = MS_CC;
 
   s->linecap = MS_CJC_BUTT;
@@ -251,12 +146,19 @@ void initSymbol(symbolObj *s)
   s->linejoinmaxsize = 3;
 }
 
-void msFreeSymbol(symbolObj *s) {
-  if(!s) return;
+int msFreeSymbol(symbolObj *s) {
+  if(!s) return MS_FAILURE;
+  if( MS_REFCNT_DECR_IS_NOT_ZERO(s) ) {
+  	return MS_FAILURE;
+  }
+  
   if(s->name) free(s->name);
   if(s->img) gdImageDestroy(s->img);
   if(s->font) free(s->font);
   if(s->imagepath) free(s->imagepath);
+  if(s->character) free(s->character);
+  
+  return MS_SUCCESS;
 }
 
 int loadSymbol(symbolObj *s, char *symbolpath)
@@ -378,6 +280,35 @@ int loadSymbol(symbolObj *s, char *symbolpath)
     case(NAME):
       if(getString(&s->name) == MS_FAILURE) return(-1);
       break;
+    case(STYLE): /* depricated */
+      /* TODO: output warning */
+    case(PATTERN):
+      done = MS_FALSE;
+      for(;;) { /* read till the next END */
+	switch(msyylex()) {  
+	case(END):
+	  if(s->patternlength < 2) {
+	    msSetError(MS_SYMERR, "Not enough pattern elements. A minimum of 2 are required", "loadSymbol()");
+	    return(-1);
+	  }	  
+	  done = MS_TRUE;
+	  break;
+	case(MS_NUMBER): /* read the pattern values */
+	  if(s->patternlength == MS_MAXPATTERNLENGTH) {
+	    msSetError(MS_SYMERR, "Pattern too long.", "loadSymbol()");
+	    return(-1);
+	  }
+	  s->pattern[s->patternlength] = atoi(msyytext);
+	  s->patternlength++;
+	  break;
+	default:
+	  msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyytext, msyylineno);
+	  return(-1);
+	}
+	if(done == MS_TRUE)
+	  break;
+      }      
+      break;
     case(POINTS):
       done = MS_FALSE;
       for(;;) {
@@ -401,33 +332,6 @@ int loadSymbol(symbolObj *s, char *symbolpath)
 	  break;
       }
       break;    
-    case(STYLE):      
-      done = MS_FALSE;
-      for(;;) { /* read till the next END */
-	switch(msyylex()) {  
-	case(END):
-	  if(s->stylelength < 2) {
-	    msSetError(MS_SYMERR, "Not enough style elements. A minimum of 2 are required", "loadSymbol()");
-	    return(-1);
-	  }	  
-	  done = MS_TRUE;
-	  break;
-	case(MS_NUMBER): /* read the style values */
-	  if(s->stylelength == MS_MAXSTYLELENGTH) {
-	    msSetError(MS_SYMERR, "Style too long.", "loadSymbol()");
-	    return(-1);
-	  }
-	  s->style[s->stylelength] = atoi(msyytext);
-	  s->stylelength++;
-	  break;
-	default:
-	  msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyytext, msyylineno);
-	  return(-1);
-	}
-	if(done == MS_TRUE)
-	  break;
-      }      
-      break;
     case(TRANSPARENT):
       s->transparent = MS_TRUE;
       if(getInteger(&(s->transparentcolor)) == -1) return(-1);
@@ -466,6 +370,8 @@ void writeSymbol(symbolObj *s, FILE *stream)
   case(MS_SYMBOL_PIXMAP):
     fprintf(stream, "    TYPE PIXMAP\n");
     if(s->imagepath != NULL) fprintf(stream, "    IMAGE \"%s\"\n", s->imagepath);
+    if (s->gap != 0)
+      fprintf(stream, "    GAP %d\n", s->gap);
     fprintf(stream, "    TRANSPARENT %d\n", s->transparentcolor);
     break;
   case(MS_SYMBOL_TRUETYPE):
@@ -501,11 +407,11 @@ void writeSymbol(symbolObj *s, FILE *stream)
       fprintf(stream, "    END\n");
     }
 
-    /* STYLE */
-    if(s->stylelength != 0) {
-      fprintf(stream, "    STYLE\n     ");
-      for(i=0; i<s->stylelength; i++) {
-	fprintf(stream, " %d", s->style[i]);
+    /* PATTERN */
+    if(s->patternlength != 0) {
+      fprintf(stream, "    PATTERN\n     ");
+      for(i=0; i<s->patternlength; i++) {
+	fprintf(stream, " %d", s->pattern[i]);
       }
       fprintf(stream, "\n    END\n");
     }
@@ -534,11 +440,9 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 
   if(!filename || strlen(filename) == 0) return(-1);
 
-  if(symbolset->numsymbols == MS_MAXSYMBOLS) { /* no room */
-    msSetError(MS_SYMERR, "Maximum number of symbols reached.", "msAddImageSymbol()");
-    return(-1);
-  }
-
+  /* Allocate/init memory for new symbol if needed */
+  if (msGrowSymbolSet(symbolset) == NULL)
+      return -1;
   
   if(symbolset->map) {
     if((stream = fopen(msBuildPath(szPath, symbolset->map->mappath, filename), "rb")) == NULL) {
@@ -551,9 +455,8 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
       return(-1);
     }
   }
-  i = symbolset->numsymbols;  
 
-  initSymbol(&symbolset->symbol[i]);
+  i = symbolset->numsymbols;  
 
   fread(bytes,8,1,stream); /* read some bytes to try and identify the file */
   rewind(stream); /* reset the image for the readers */
@@ -561,7 +464,7 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 #ifdef USE_GD_GIF
     gdIOCtx *ctx;
     ctx = msNewGDFileCtx(stream);
-    symbolset->symbol[i].img = gdImageCreateFromGifCtx(ctx);
+    symbolset->symbol[i]->img = gdImageCreateFromGifCtx(ctx);
     ctx->gd_free(ctx);
 #else
     msSetError(MS_MISCERR, "Unable to load GIF symbol.", "msAddImageSymbol()");
@@ -572,7 +475,7 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 #ifdef USE_GD_PNG
     gdIOCtx *ctx;
     ctx = msNewGDFileCtx(stream);
-    symbolset->symbol[i].img = gdImageCreateFromPngCtx(ctx);
+    symbolset->symbol[i]->img = gdImageCreateFromPngCtx(ctx);
     ctx->gd_free(ctx);
 #else
     msSetError(MS_MISCERR, "Unable to load PNG symbol.", "msAddImageSymbol()");
@@ -583,44 +486,122 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 
   fclose(stream);
   
-  if(!symbolset->symbol[i].img) {
+  if(!symbolset->symbol[i]->img) {
     msSetError(MS_GDERR, NULL, "msAddImageSymbol()");
     return(-1);
   }
 
-  symbolset->symbol[i].name = strdup(filename);
-  symbolset->symbol[i].imagepath = strdup(filename);
-  symbolset->symbol[i].type = MS_SYMBOL_PIXMAP;
-  symbolset->symbol[i].sizex = symbolset->symbol[i].img->sx;
-  symbolset->symbol[i].sizey = symbolset->symbol[i].img->sy;
+  symbolset->symbol[i]->name = strdup(filename);
+  symbolset->symbol[i]->imagepath = strdup(filename);
+  symbolset->symbol[i]->type = MS_SYMBOL_PIXMAP;
+  symbolset->symbol[i]->sizex = symbolset->symbol[i]->img->sx;
+  symbolset->symbol[i]->sizey = symbolset->symbol[i]->img->sy;
   symbolset->numsymbols++;
 
   return(i);
 }
 
-void msFreeSymbolSet(symbolSetObj *symbolset)
+int msFreeSymbolSet(symbolSetObj *symbolset)
 {
   int i;
 
   freeImageCache(symbolset->imagecache);
-  for(i=1; i<symbolset->numsymbols; i++)
-    msFreeSymbol(&(symbolset->symbol[i]));
+  for(i=0; i<symbolset->numsymbols; i++) {
+	  if (symbolset->symbol[i]!=NULL) {
+		  if ( msFreeSymbol((symbolset->symbol[i])) == MS_SUCCESS ) {
+			  msFree(symbolset->symbol[i]);
+			  symbolset->symbol[i]=NULL;
+		  }
+	  }
+  }
+  msFree(symbolset->symbol);
 
   /* no need to deal with fontset, it's a pointer */
+  return MS_SUCCESS;
 }
 
 void msInitSymbolSet(symbolSetObj *symbolset)
 {
   symbolset->filename = NULL;
-  symbolset->numsymbols = 1; /* always 1 symbol */
+
   symbolset->imagecache = NULL;
   symbolset->imagecachesize = 0; /* 0 symbols in the cache */
 
   symbolset->fontset = NULL;
   symbolset->map = NULL;
-  memset( symbolset->symbol, 0, sizeof(symbolObj) );
+
+  symbolset->numsymbols = 0;
+  symbolset->maxsymbols = 0;
+  symbolset->symbol = NULL;
+
+  /* Alloc symbol[] array and ensure there is at least 1 symbol:
+   * symbol 0 which is the default symbol with all default params.
+   */
+  if (msGrowSymbolSet(symbolset) == NULL)
+      return; /* alloc failed */
+
+  /* Just increment numsymbols to reserve symbol 0.
+   * initSymbol() has already been called
+   */
+  symbolset->numsymbols = 1;
+
 }
 
+
+/*
+** Ensure there is at least one free entry in the symbol array of this
+** symbolSetObj. Grow the allocated symbol[] array if necessary and 
+** allocate a new symbol for symbol[numsymbols] if there is not already one
+** and call initSymbol() on it.
+**
+** This function is safe to use for the initial allocation of the symbol[]
+** array as well (i.e. when maxsymbols==0 and symbol==NULL)
+**
+** Returns a reference to the new symbolObj on success, NULL on error.
+*/
+symbolObj *msGrowSymbolSet( symbolSetObj *symbolset )
+{
+    /* Do we need to increase the size of symbol[] by MS_SYMBOL_ALLOCSIZE? */
+    if (symbolset->numsymbols == symbolset->maxsymbols) {
+        int i;
+        if (symbolset->maxsymbols == 0) {
+            /* Initial allocation of array */
+            symbolset->maxsymbols += MS_SYMBOL_ALLOCSIZE;
+            symbolset->numsymbols = 0;
+            symbolset->symbol = (symbolObj**)malloc(symbolset->maxsymbols*sizeof(symbolObj*));
+        } else {
+            /* realloc existing array */
+            symbolset->maxsymbols += MS_SYMBOL_ALLOCSIZE;
+            symbolset->symbol = (symbolObj**)realloc(symbolset->symbol,
+                                                     symbolset->maxsymbols*sizeof(symbolObj*));
+        }
+
+        if (symbolset->symbol == NULL) {
+            msSetError(MS_MEMERR, "Failed to allocate memory for symbol array.", "msGrowSymbolSet()");
+            return NULL;
+        }
+
+        for(i=symbolset->numsymbols; i<symbolset->maxsymbols; i++)
+            symbolset->symbol[i] = NULL;
+    }
+
+    if (symbolset->symbol[symbolset->numsymbols]==NULL) {
+        symbolset->symbol[symbolset->numsymbols]=(symbolObj*)malloc(sizeof(symbolObj));
+        if (symbolset->symbol[symbolset->numsymbols]==NULL) {
+          msSetError(MS_MEMERR, "Failed to allocate memory for a symbolObj", "msGrowSymbolSet()");
+          return NULL;
+        }
+    }
+
+    /* Always call initSymbol() even if we didn't allocate a new symbolObj
+     * Since it's possible to dynamically remove/reuse symbols */
+    initSymbol(symbolset->symbol[symbolset->numsymbols]);
+
+    return symbolset->symbol[symbolset->numsymbols];
+}
+
+
+
 /* ---------------------------------------------------------------------------
    msLoadSymbolSet and loadSymbolSet
 
@@ -667,7 +648,7 @@ int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
     return(-1);
   }
 
-  pszSymbolPath = getPath(szPath);
+  pszSymbolPath = msGetPath(szPath);
 
   msyylineno = 0; /* reset line counter */
   msyyrestart(msyyin); /* flush the scanner - there's a better way but this works for now */
@@ -682,13 +663,14 @@ int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
       status = 0;
       break;
     case(SYMBOL):
-      if(symbolset->numsymbols == MS_MAXSYMBOLS) { 
-	msSetError(MS_SYMERR, "Too many symbols defined.", "msLoadSymbolSet()");
-	status = -1;      
-      }
-      if((loadSymbol(&(symbolset->symbol[symbolset->numsymbols]), pszSymbolPath) == -1)) 
+      /* Allocate/init memory for new symbol if needed */
+      if (msGrowSymbolSet(symbolset) == NULL) {
 	status = -1;
-      symbolset->numsymbols++;
+      }
+      else if((loadSymbol((symbolset->symbol[symbolset->numsymbols]), pszSymbolPath) == -1)) 
+	  status = -1;
+      else
+          symbolset->numsymbols++;
       break;
     case(SYMBOLSET):
       break;
@@ -701,6 +683,7 @@ int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
   } /* end for */
 
   fclose(msyyin);
+  msyyin = NULL;
   free(pszSymbolPath);
   return(status);
 }
@@ -728,7 +711,7 @@ int msGetMarkerSize(symbolSetObj *symbolset, styleObj *style, int *width, int *h
 
   if(style->size == -1) {
       size = MS_NINT(
-          msSymbolGetDefaultSize( &( symbolset->symbol[style->symbol] ) )
+          msSymbolGetDefaultSize( ( symbolset->symbol[style->symbol] ) )
           * scalefactor );
   }
   else
@@ -736,14 +719,14 @@ int msGetMarkerSize(symbolSetObj *symbolset, styleObj *style, int *width, int *h
   size = MS_MAX(size, style->minsize);
   size = MS_MIN(size, style->maxsize);
 
-  switch(symbolset->symbol[style->symbol].type) {  
+  switch(symbolset->symbol[style->symbol]->type) {  
    
 #ifdef USE_GD_FT
   case(MS_SYMBOL_TRUETYPE):
-    font = msLookupHashTable(&(symbolset->fontset->fonts), symbolset->symbol[style->symbol].font);
+    font = msLookupHashTable(&symbolset->fontset->fonts, symbolset->symbol[style->symbol]->font);
     if(!font) return(MS_FAILURE);
 
-    if(msGetCharacterSize(symbolset->symbol[style->symbol].character, size, 
+    if(msGetCharacterSize(symbolset->symbol[style->symbol]->character, size, 
                           (char *) font, &rect) != MS_SUCCESS) 
       return(MS_FAILURE);
 
@@ -755,20 +738,20 @@ int msGetMarkerSize(symbolSetObj *symbolset, styleObj *style, int *width, int *h
 
   case(MS_SYMBOL_PIXMAP):
     if(size == 1) {        
-      *width = MS_MAX(*width, symbolset->symbol[style->symbol].img->sx);
-      *height = MS_MAX(*height, symbolset->symbol[style->symbol].img->sy);
+      *width = MS_MAX(*width, symbolset->symbol[style->symbol]->img->sx);
+      *height = MS_MAX(*height, symbolset->symbol[style->symbol]->img->sy);
     } else {
-      *width = MS_MAX(*width, MS_NINT((size/symbolset->symbol[style->symbol].img->sy) * symbolset->symbol[style->symbol].img->sx));
+      *width = MS_MAX(*width, MS_NINT((size/symbolset->symbol[style->symbol]->img->sy) * symbolset->symbol[style->symbol]->img->sx));
       *height = MS_MAX(*height, size);
     }
     break;
   default: /* vector and ellipses, scalable */
     if(style->size > 0) {
-      *width = MS_MAX(*width, MS_NINT((size/symbolset->symbol[style->symbol].sizey) * symbolset->symbol[style->symbol].sizex));
+      *width = MS_MAX(*width, MS_NINT((size/symbolset->symbol[style->symbol]->sizey) * symbolset->symbol[style->symbol]->sizex));
       *height = MS_MAX(*height, size);
     } else { /* use symbol defaults */
-      *width = (int) MS_MAX(*width, symbolset->symbol[style->symbol].sizex);
-      *height = (int) MS_MAX(*height, symbolset->symbol[style->symbol].sizey);
+      *width = (int) MS_MAX(*width, symbolset->symbol[style->symbol]->sizex);
+      *height = (int) MS_MAX(*height, symbolset->symbol[style->symbol]->sizey);
     }
     break;
   }  
@@ -791,15 +774,12 @@ int msAddNewSymbol(mapObj *map, char *name)
     if (i >= 0)
       return i;
 
-    if (map->symbolset.numsymbols == MS_MAXSYMBOLS) 
-    {
-        msSetError(MS_SYMERR, "Maximum number of symbols reached.", 
-                   "msAddNewSymbol()");
-        return(-1);
-    }
+    /* Allocate memory for new symbol if needed */
+    if (msGrowSymbolSet(&(map->symbolset)) == NULL)
+        return -1;
+
     i = map->symbolset.numsymbols;  
-    initSymbol(&map->symbolset.symbol[i]);
-    map->symbolset.symbol[i].name = strdup(name);
+    map->symbolset.symbol[i]->name = strdup(name);
 
     map->symbolset.numsymbols++;
 
@@ -812,13 +792,12 @@ int msAddNewSymbol(mapObj *map, char *name)
 
 
 int msAppendSymbol(symbolSetObj *symbolset, symbolObj *symbol) {
-    /* Possible to add another symbol? */
-    if (symbolset->numsymbols == MS_MAXSYMBOLS) {
-        msSetError(MS_CHILDERR, "Maximum number of symbols, %d, has been reached", "msAppendSymbol()", MS_MAXSYMBOLS);
+    /* Allocate memory for new symbol if needed */
+    if (msGrowSymbolSet(symbolset) == NULL)
         return -1;
-    }
     symbolset->numsymbols++;
-    msCopySymbol(&(symbolset->symbol[symbolset->numsymbols-1]), symbol, NULL);
+    symbolset->symbol[symbolset->numsymbols-1]=symbol;
+    MS_REFCNT_INCR(symbol);
     return symbolset->numsymbols-1;
 }
 
@@ -835,12 +814,13 @@ symbolObj *msRemoveSymbol(symbolSetObj *symbolset, int nSymbolIndex) {
         return NULL;
     }
     else {
-        symbol = (symbolObj *)malloc(sizeof(symbolObj));
-        msCopySymbol(symbol, &(symbolset->symbol[nSymbolIndex]), NULL);
+        symbol=symbolset->symbol[nSymbolIndex];
         for (i=nSymbolIndex+1; i<symbolset->numsymbols; i++) {
             symbolset->symbol[i-1] = symbolset->symbol[i];
         }
+		symbolset->symbol[i-1]=NULL;
         symbolset->numsymbols--;
+        MS_REFCNT_DECR(symbol);
         return symbol;
     }
 }
@@ -853,8 +833,8 @@ int msSaveSymbolSetStream(symbolSetObj *symbolset, FILE *stream) {
     }
     /* Don't ever write out the default symbol at index 0 */
     for (i=1; i<symbolset->numsymbols; i++) {
-        symbolset->symbol[i].inmapfile = MS_TRUE;
-        writeSymbol(&(symbolset->symbol[i]), stream);
+        symbolset->symbol[i]->inmapfile = MS_TRUE;
+        writeSymbol((symbolset->symbol[i]), stream);
     }
     return MS_SUCCESS;
 }
@@ -892,7 +872,7 @@ int msLoadImageSymbol(symbolObj *symbol, const char *filename) {
     }
 
     if ((stream = fopen(filename, "rb")) == NULL) {
-          msSetError(MS_IOERR, "Error opening image file %s.", "msLoadImageSymbol()");
+          msSetError(MS_IOERR, "Error opening image file %s.", "msLoadImageSymbol()", filename);
           return MS_FAILURE;
     }
 
@@ -966,16 +946,16 @@ int msCopySymbol(symbolObj *dst, symbolObj *src, mapObj *map) {
   MS_COPYSTELEM(sizex);
   MS_COPYSTELEM(sizey);
   
-  for (i=0; i < MS_MAXVECTORPOINTS; i++) {
+  for (i=0; i < src->numpoints; i++) {
     MS_COPYPOINT(&(dst->points[i]), &(src->points[i]));
   }
   
   MS_COPYSTELEM(numpoints);
   MS_COPYSTELEM(filled);
-  MS_COPYSTELEM(stylelength);
+  MS_COPYSTELEM(patternlength);
 
-  for (i=0; i<MS_MAXSTYLELENGTH; i++) {
-      dst->style[i] = src->style[i];
+  for (i=0; i < src->patternlength; i++) {
+    dst->pattern[i] = src->pattern[i];
   }
 
   MS_COPYSTRING(dst->imagepath, src->imagepath);
@@ -997,14 +977,17 @@ int msCopySymbol(symbolObj *dst, symbolObj *src, mapObj *map) {
     
     if (gdImageTrueColor(src->img)) {
       dst->img = gdImageCreateTrueColor(gdImageSX(src->img), gdImageSY(src->img));
-      gdImageColorTransparent(dst->img, gdImageGetTransparent(src->img));
-      gdImageAlphaBlending(dst->img, 0);
       gdImageCopy(dst->img, src->img, 0, 0, 0, 0, gdImageSX(src->img), gdImageSY(src->img));
+      gdImageAlphaBlending(dst->img, 0);
+      gdImageFilledRectangle(dst->img, 0, 0, gdImageSX(src->img), gdImageSY(src->img), gdImageColorAllocateAlpha(dst->img, 0, 0, 0, gdAlphaTransparent)); 
     } else {
+      int tc = gdImageGetTransparent(src->img);
+
       dst->img = gdImageCreate(gdImageSX(src->img), gdImageSY(src->img));
-      gdImageAlphaBlending(dst->img, 0);
-      gdImageColorTransparent(dst->img, gdImageGetTransparent(src->img));
-      gdImageCopy(dst->img, src->img, 0, 0, 0, 0, gdImageSX(src->img), gdImageSY(src->img));
+      if(tc != -1)
+        gdImageColorTransparent(dst->img, gdImageColorAllocate(dst->img, gdImageRed(src->img, tc), gdImageGreen(src->img, tc), gdImageBlue(src->img, tc)));
+
+      gdImageCopy(dst->img, src->img, 0, 0, 0, 0, gdImageSX(src->img), gdImageSY(src->img));      
     }
   }
 
@@ -1026,15 +1009,16 @@ int msCopySymbolSet(symbolSetObj *dst, symbolSetObj *src, mapObj *map)
   dst->map = map;
   dst->fontset = &(map->fontset);
   
-  MS_COPYSTELEM(numsymbols);
- 
   /* Copy child symbols */
-  for (i = 0; i < dst->numsymbols; i++) {
-    return_value = msCopySymbol(&(dst->symbol[i]), &(src->symbol[i]), map);
+  for (i = 0; i < src->numsymbols; i++) {
+    if (msGrowSymbolSet(dst) == NULL)
+        return MS_FAILURE;
+    return_value = msCopySymbol(dst->symbol[i], src->symbol[i], map);
     if (return_value != MS_SUCCESS) {
       msSetError(MS_MEMERR,"Failed to copy symbol.","msCopySymbolSet()");
       return(MS_FAILURE);
     }
+    dst->numsymbols++;
   }
 
   /* MS_COPYSTELEM(imagecachesize); */
@@ -1292,13 +1276,16 @@ symbolObj *msRotateSymbol(symbolObj *symbol, double angle)
 
       /* create the new image based on the computed width/height */
       gdFree(newSymbol->img);
-      if (gdImageTrueColor(symbol->img))
+      if (gdImageTrueColor(symbol->img)) {
 	newSymbol->img = gdImageCreateTrueColor(width, height);
-      else
+        gdImageAlphaBlending(newSymbol->img, 0);
+        gdImageFilledRectangle(newSymbol->img, 0, 0, width, height, gdImageColorAllocateAlpha(newSymbol->img, 0, 0, 0, gdAlphaTransparent)); 
+      } else {
+        int tc = gdImageGetTransparent(symbol->img);
 	newSymbol->img = gdImageCreate(width, height);	
-
-      gdImageColorTransparent(newSymbol->img, gdImageGetTransparent(symbol->img));
-      gdImageAlphaBlending(newSymbol->img, 0);
+        if(tc != -1)
+          gdImageColorTransparent(newSymbol->img, gdImageColorAllocate(newSymbol->img, gdImageRed(symbol->img, tc), gdImageGreen(symbol->img, tc), gdImageBlue(symbol->img, tc)));
+      }
 
       newSymbol->sizex = maxx;
       newSymbol->sizey = maxy;
@@ -1312,3 +1299,4 @@ symbolObj *msRotateSymbol(symbolObj *symbol, double angle)
 
   return newSymbol;
 }
+
