@@ -25,30 +25,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.18  2006/04/19 16:36:18  hobu
- * don't throw an MS_IOERR when we're in debug mode when we don't find an index (bug 1752)
- *
- * Revision 1.17  2005/08/25 14:20:16  sdlime
- * Applied patch for bug 1440.
- *
- * Revision 1.16  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.15  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.14  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "maptree.h"
 
-MS_CVSID("$Id: maptree.c 5397 2006-04-19 16:36:18Z hobu $")
+MS_CVSID("$Id: maptree.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 /* -------------------------------------------------------------------- */
 /*      If the following is 0.5, nodes will be split in half.  If it    */
@@ -251,7 +233,7 @@ treeObj *msCreateTree(shapefileObj *shapefile, int maxdepth)
   tree->root = treeNodeCreate(shapefile->bounds);
 
   for(i=0; i<shapefile->numshapes; i++) {
-    if ( !msSHPReadBounds(shapefile->hSHP, i, &bounds))
+    if(msSHPReadBounds(shapefile->hSHP, i, &bounds) == MS_SUCCESS)
       treeAddShapeId(tree, i, bounds);
   }
 
@@ -787,9 +769,8 @@ void msFilterTreeSearch(shapefileObj *shp, char *status, rectObj search_rect)
 
   for(i=0;i<shp->numshapes;i++) { /* for each shape */
     if(msGetBit(status, i)) {
-      if(!msSHPReadBounds(shp->hSHP, i, &shape_rect))
-	if(msRectOverlap(&shape_rect, &search_rect) != MS_TRUE)
-	  msSetBit(status, i, 0);
+      if(msSHPReadBounds(shp->hSHP, i, &shape_rect) == MS_SUCCESS)
+	if(msRectOverlap(&shape_rect, &search_rect) != MS_TRUE) msSetBit(status, i, 0);
     }
   }
 }
