@@ -24,114 +24,15 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.190  2006/08/15 17:24:56  dan
- * Fixed problem with PHP MapScript's saveWebImage() filename collisions
- * when mapscript was loaded in php.ini with PHP as an Apache DSO (bug 1322)
- *
- * Revision 1.189  2006/07/05 05:54:53  frank
- * implement per-thread io contexts
- *
- * Revision 1.188  2006/06/27 13:35:45  sdlime
- * Initial version of GEOS C-API code including SWIG bindings for most everything useful.
- *
- * Revision 1.187  2005/08/27 04:57:25  sdlime
- * Fixed a recursion problem in context evaluation function msEvalContext(). Had to disable the [raster] option though and will try to implement some other way. (bug 1283)
- *
- * Revision 1.186  2005/07/27 15:33:11  frank
- * Make sure we have room for big counters in temp file names.
- *
- * Revision 1.185  2005/07/27 15:01:32  frank
- * Fixes in msTmpFile() to ensure counter actually increments and that
- * forced basenames work again.  Bug 1312.
- *
- * Revision 1.184  2005/07/13 19:35:08  julien
- * Bug 1381: Support for case-insensitive Expression
- *
- * Revision 1.183  2005/06/24 13:32:32  assefa
- * in function msTmpFile : reinitialize always the sting tmpId. There was
- * a problem with this string not being initialized when the function was
- * call more than once : Bug 1312.
- *
- * Revision 1.182  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.181  2005/05/27 15:00:12  dan
- * New regex wrappers to solve issues with previous version (bug 1354)
- *
- * Revision 1.180  2005/04/21 15:09:28  julien
- * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
- *
- * Revision 1.179  2005/04/14 15:17:14  julien
- * Bug 1244: Remove Z and M from point by default to gain performance.
- *
- * Revision 1.178  2005/04/12 23:56:44  sean
- * change tmpId string to be non static in msTmpFile() (bug 1312).
- *
- * Revision 1.177  2005/04/07 17:23:16  assefa
- * Remove #ifdef USE_SVG. It was added during development.
- *
- * Revision 1.176  2005/02/18 05:07:26  frank
- * fixed return from msForceTmpFileBase
- *
- * Revision 1.175  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.174  2005/02/09 21:51:17  frank
- * added msForceTmpFileBase and -tmpbase mapserv switch
- *
- * Revision 1.173  2005/02/09 20:43:50  assefa
- * Add SVG utility function msSaveImagetoFpSVG.
- *
- * Revision 1.172  2005/02/08 17:34:46  sean
- * call msThreadInit in msSetup (bug 1223).
- *
- * Revision 1.171  2005/02/06 19:31:32  sean
- * msSetup() function calls gdFontCacheSetup().  msSetup should be called immediately on mapscript module import to avoid race condition with font cache (bug 1203).
- *
- * Revision 1.170  2005/02/03 00:06:57  assefa
- * Add SVG function prototypes and related calls.
- *
- * Revision 1.169  2005/01/28 06:16:54  sdlime
- * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
- *
- * Revision 1.168  2005/01/20 01:59:19  sean
- * protect against labelitemindex < 0 take 2, > 0 changed to >= 0
- *
- * Revision 1.167  2005/01/20 00:53:47  sean
- * protect against labelitemindex < 0
- *
- * Revision 1.166  2004/12/07 21:33:28  sean
- * protect against getting a shape value when values is NULL in msShapeGetAnnotation().  also implement setValue for mapscript::shapeObj so we can define labels for inline features (bug 1114)
- *
- * Revision 1.165.2.1  2004/12/07 21:10:20  sean
- * protect against getting a shape value when values is NULL in msShapeGetAnnotation().  also implement setValue for mapscript::shapeObj so we can define labels for inline features (bug 1114)
- *
- * Revision 1.165  2004/11/22 16:19:42  sdlime
- * Trivial formating change.
- *
- * Revision 1.164  2004/11/22 15:44:42  sean
- * quick fixes to work around new requirement for non-null layer names in msValidateContext.  Set ltag to '[NULL]' for a NULL name in msValidateContext, skip null-named layers in msEvalContext (bug 1081).
- *
- * Revision 1.163  2004/11/22 03:43:54  sdlime
- * Added tests to mimimize the threat of recursion problems when evaluating LAYER REQUIRES or LABELREQUIRES expressions. Note that via MapScript it is possible to circumvent that test by defining layers with problems after running prepareImage. Other things crop up in that case too (symbol scaling dies) so it should be considered bad programming practice.
- *
- * Revision 1.162  2004/10/28 02:12:48  frank
- * Don't try and use msBuildPath() if filename is NULL (stdout) in msSaveImage().
- * This prevents "noise" calls to msSetError() by msBuildPath().
- *
- * Revision 1.161  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ *****************************************************************************/
+
 
 #include <time.h>
 
-#include "map.h"
+#include "mapserver.h"
 #include "mapparser.h"
 #include "mapthread.h"
+#include "mapfile.h"
 
 #ifdef _WIN32
 #include <fcntl.h>
@@ -142,16 +43,157 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: maputil.c 5597 2006-08-15 17:24:56Z dan $")
+MS_CVSID("$Id: maputil.c 6741 2007-08-30 04:29:16Z hobu $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
+extern int msyylex_destroy(void);
 extern char *msyytext;
 
 extern int msyyresult; /* result of parsing, true/false */
 extern int msyystate;
 extern char *msyystring;
 
+/*
+** Helper functions to convert from strings to other types or objects.
+*/
+static int bindIntegerAttribute(int *attribute, char *value)
+{
+  if(!value || strlen(value) == 0) return MS_FAILURE;
+  *attribute = atoi(value);
+  return MS_SUCCESS;
+}
+
+static int bindDoubleAttribute(double *attribute, char *value)
+{
+  if(!value || strlen(value) == 0) return MS_FAILURE;
+  *attribute = atof(value);
+  return MS_SUCCESS;
+}
+
+static int bindColorAttribute(colorObj *attribute, char *value)
+{
+  if(!value || strlen(value) == 0) return MS_FAILURE;
+
+  if(value[0] == '#' && strlen(value) == 7) { /* got a hex color */
+    char hex[2];
+
+    hex[0] = value[1];
+    hex[1] = value[2];
+    attribute->red = msHexToInt(hex);
+    hex[0] = value[3];
+    hex[1] = value[4];
+    attribute->green = msHexToInt(hex);
+    hex[0] = value[5];
+    hex[1] = value[6];
+    attribute->blue = msHexToInt(hex);
+
+    return MS_SUCCESS;
+  } else { /* try a space delimited string */
+    char **tokens=NULL;
+    int numtokens=0;
+
+    tokens = msStringSplit(value, ' ', &numtokens);
+    if(tokens==NULL || numtokens != 3) {
+      msFreeCharArray(tokens, numtokens);
+      return MS_FAILURE; /* punt */
+    }
+
+    attribute->red = atoi(tokens[0]);
+    attribute->blue = atoi(tokens[1]);
+    attribute->green = atoi(tokens[2]);
+    msFreeCharArray(tokens, numtokens);
+
+    return MS_SUCCESS;
+  }
+
+  return MS_FAILURE; /* shouldn't get here */
+}
+
+/*
+** Function to bind various layer properties to shape attributes.
+*/
+int msBindLayerToShape(layerObj *layer, shapeObj *shape)
+{
+  int i, j;
+  labelObj *label; /* for brevity */
+  styleObj *style;
+
+  if(!layer || !shape) return MS_FAILURE;
+
+  for(i=0; i<layer->numclasses; i++) {
+
+    /* check the styleObj's */
+    for(j=0; j<layer->class[i]->numstyles; j++) {
+      style = layer->class[i]->styles[j];
+
+      if(style->numbindings > 0) {
+
+        if(style->bindings[MS_STYLE_BINDING_SYMBOL].index != -1) {
+          style->symbol = msGetSymbolIndex(&(layer->map->symbolset), shape->values[style->bindings[MS_STYLE_BINDING_SYMBOL].index], MS_TRUE);
+          if(style->symbol == -1) style->symbol = 0; /* a reasonable default (perhaps should throw an error?) */
+        }
+
+        if(style->bindings[MS_STYLE_BINDING_ANGLE].index != -1) {
+          style->angle = 360.0;
+          bindDoubleAttribute(&style->angle, shape->values[style->bindings[MS_STYLE_BINDING_ANGLE].index]);
+        }
+
+        if(style->bindings[MS_STYLE_BINDING_SIZE].index != -1) {
+          style->size = 1;
+          bindIntegerAttribute(&style->size, shape->values[style->bindings[MS_STYLE_BINDING_SIZE].index]);
+        }
+
+        if(style->bindings[MS_STYLE_BINDING_COLOR].index != -1) {
+          MS_INIT_COLOR(style->color, -1,-1,-1);
+          bindColorAttribute(&style->color, shape->values[style->bindings[MS_STYLE_BINDING_COLOR].index]);
+        }
+
+        if(style->bindings[MS_STYLE_BINDING_OUTLINECOLOR].index != -1) {
+          MS_INIT_COLOR(style->outlinecolor, -1,-1,-1);
+          bindColorAttribute(&style->outlinecolor, shape->values[style->bindings[MS_STYLE_BINDING_OUTLINECOLOR].index]);
+        }
+      }
+    } /* next styleObj */
+
+    /* check the labelObj */
+    label = &(layer->class[i]->label);
+
+    if(label->numbindings > 0) {
+      if(label->bindings[MS_LABEL_BINDING_ANGLE].index != -1) {
+        label->angle = 0.0;
+        bindDoubleAttribute(&label->angle, shape->values[label->bindings[MS_LABEL_BINDING_ANGLE].index]);
+      }
+
+      if(label->bindings[MS_LABEL_BINDING_SIZE].index != -1) {
+        label->size = 1;
+        bindIntegerAttribute(&label->size, shape->values[label->bindings[MS_LABEL_BINDING_SIZE].index]);
+      }
+
+      if(label->bindings[MS_LABEL_BINDING_COLOR].index != -1) {
+        MS_INIT_COLOR(label->color, -1,-1,-1);
+        bindColorAttribute(&label->color, shape->values[label->bindings[MS_LABEL_BINDING_COLOR].index]);
+      }
+
+      if(label->bindings[MS_LABEL_BINDING_OUTLINECOLOR].index != -1) {
+        MS_INIT_COLOR(label->outlinecolor, -1,-1,-1);
+        bindColorAttribute(&label->outlinecolor, shape->values[label->bindings[MS_LABEL_BINDING_OUTLINECOLOR].index]);
+      }
+
+      if(label->bindings[MS_LABEL_BINDING_FONT].index != -1) {
+        msFree(label->font);
+        label->font = strdup(shape->values[label->bindings[MS_LABEL_BINDING_FONT].index]);
+      }
+
+      if(label->bindings[MS_LABEL_BINDING_PRIORITY].index != -1) {
+        label->priority = MS_DEFAULT_LABEL_PRIORITY;
+        bindIntegerAttribute(&label->priority, shape->values[label->bindings[MS_LABEL_BINDING_PRIORITY].index]);
+      }
+    }
+  } /* next classObj */
+
+  return MS_SUCCESS;
+}
 
 /*
  * Used to get red, green, blue integers separately based upon the color index
@@ -183,9 +225,9 @@ static int searchContextForTag(mapObj *map, char **ltags, char *tag, char *conte
   for(i=0; i<map->numlayers; i++) {
     if(strstr(context, ltags[i]) != NULL) { /* need to check this layer */
       if(requires == MS_TRUE) {
-        if(searchContextForTag(map, ltags, tag, map->layers[i].requires, MS_TRUE) == MS_SUCCESS) return MS_SUCCESS;
+        if(searchContextForTag(map, ltags, tag, GET_LAYER(map, i)->requires, MS_TRUE) == MS_SUCCESS) return MS_SUCCESS;
       } else {
-        if(searchContextForTag(map, ltags, tag, map->layers[i].labelrequires, MS_FALSE) == MS_SUCCESS) return MS_SUCCESS;      
+        if(searchContextForTag(map, ltags, tag, GET_LAYER(map, i)->labelrequires, MS_FALSE) == MS_SUCCESS) return MS_SUCCESS;      
       }
     }
   }
@@ -206,28 +248,28 @@ int msValidateContexts(mapObj *map)
 
   ltags = (char **) malloc(map->numlayers*sizeof(char *));
   for(i=0; i<map->numlayers; i++) {
-    if(map->layers[i].name == NULL) {
+    if(GET_LAYER(map, i)->name == NULL) {
       ltags[i] = strdup("[NULL]");
     } else {
-      ltags[i] = (char *) malloc(sizeof(char)*strlen(map->layers[i].name) + 3);
-      sprintf(ltags[i], "[%s]", map->layers[i].name);
+      ltags[i] = (char *) malloc(sizeof(char)*strlen(GET_LAYER(map, i)->name) + 3);
+      sprintf(ltags[i], "[%s]", GET_LAYER(map, i)->name);
     }
   }
 
   /* check each layer's REQUIRES and LABELREQUIRES parameters */
   for(i=0; i<map->numlayers; i++) { 
-    /* printf("working on layer %s, looking for references to %s\n", map->layers[i].name, ltags[i]); */
-    if(searchContextForTag(map, ltags, ltags[i], map->layers[i].requires, MS_TRUE) == MS_SUCCESS) {
-      msSetError(MS_PARSEERR, "Recursion error found for REQUIRES parameter for layer %s.", "msValidateContexts", map->layers[i].name);
+    /* printf("working on layer %s, looking for references to %s\n", GET_LAYER(map, i)->name, ltags[i]); */
+    if(searchContextForTag(map, ltags, ltags[i], GET_LAYER(map, i)->requires, MS_TRUE) == MS_SUCCESS) {
+      msSetError(MS_PARSEERR, "Recursion error found for REQUIRES parameter for layer %s.", "msValidateContexts", GET_LAYER(map, i)->name);
       status = MS_FAILURE;
       break;
     }
-    if(searchContextForTag(map, ltags, ltags[i], map->layers[i].labelrequires, MS_FALSE) == MS_SUCCESS) {
-      msSetError(MS_PARSEERR, "Recursion error found for LABELREQUIRES parameter for layer %s.", "msValidateContexts", map->layers[i].name);
+    if(searchContextForTag(map, ltags, ltags[i], GET_LAYER(map, i)->labelrequires, MS_FALSE) == MS_SUCCESS) {
+      msSetError(MS_PARSEERR, "Recursion error found for LABELREQUIRES parameter for layer %s.", "msValidateContexts", GET_LAYER(map, i)->name);
       status = MS_FAILURE;
       break;
     }
-    /* printf("done layer %s\n", map->layers[i].name); */
+    /* printf("done layer %s\n", GET_LAYER(map, i)->name); */
   }
 
   /* clean up */
@@ -248,23 +290,23 @@ int msEvalContext(mapObj *map, layerObj *layer, char *context)
 
   for(i=0; i<map->numlayers; i++) { /* step through all the layers */
     if(layer->index == i) continue; /* skip the layer in question */    
-    if (map->layers[i].name == NULL) continue; /* Layer without name cannot be used in contexts */
+    if (GET_LAYER(map, i)->name == NULL) continue; /* Layer without name cannot be used in contexts */
 
-    tmpstr2 = (char *)malloc(sizeof(char)*strlen(map->layers[i].name) + 3);
-    sprintf(tmpstr2, "[%s]", map->layers[i].name);
+    tmpstr2 = (char *)malloc(sizeof(char)*strlen(GET_LAYER(map, i)->name) + 3);
+    sprintf(tmpstr2, "[%s]", GET_LAYER(map, i)->name);
 
     if(strstr(tmpstr1, tmpstr2)) {
-      if(msLayerIsVisible(map, &(map->layers[i])))
-	tmpstr1 = gsub(tmpstr1, tmpstr2, "1");
+      if(msLayerIsVisible(map, (GET_LAYER(map, i))))
+          tmpstr1 = msReplaceSubstring(tmpstr1, tmpstr2, "1");
       else
-	tmpstr1 = gsub(tmpstr1, tmpstr2, "0");
+          tmpstr1 = msReplaceSubstring(tmpstr1, tmpstr2, "0");
     }
 
     free(tmpstr2);
   }
 
   msAcquireLock( TLOCK_PARSER );
-  msyystate = 4; msyystring = tmpstr1;
+  msyystate = MS_TOKENIZE_EXPRESSION; msyystring = tmpstr1;
   status = msyyparse();
   result = msyyresult;
   msReleaseLock( TLOCK_PARSER );
@@ -306,36 +348,34 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
       msSetError(MS_MISCERR, "Invalid item index.", "msEvalExpression()");
       return(MS_FALSE);
     }
-    if(expression->flags & MS_EXP_INSENSITIVE)
-    {
+    if(expression->flags & MS_EXP_INSENSITIVE) {
       if(strcasecmp(expression->string, items[itemindex]) == 0) return(MS_TRUE); /* got a match */
-    }
-    else
-    {
+    } else {
       if(strcmp(expression->string, items[itemindex]) == 0) return(MS_TRUE); /* got a match */
     }
     break;
   case(MS_EXPRESSION):
     tmpstr = strdup(expression->string);
 
-    for(i=0; i<expression->numitems; i++)      
-      tmpstr = gsub(tmpstr, expression->items[i], items[expression->indexes[i]]);
-
+    for(i=0; i<expression->numitems; i++) {
+        items[expression->indexes[i]] = msReplaceSubstring( items[expression->indexes[i]], "\'", "\\\'");
+        items[expression->indexes[i]] = msReplaceSubstring( items[expression->indexes[i]], "\"", "\\\"");
+        tmpstr = msReplaceSubstring(tmpstr, expression->items[i], items[expression->indexes[i]]);
+    }
     msAcquireLock( TLOCK_PARSER );
-    msyystate = 4;
+    msyystate = MS_TOKENIZE_EXPRESSION;
     msyystring = tmpstr; /* set lexer state to EXPRESSION_STRING */
     status = msyyparse();
     expresult = msyyresult;
     msReleaseLock( TLOCK_PARSER );
-    free(tmpstr);
 
     if (status != 0)
     {
-        msSetError(MS_PARSEERR, "Failed to parse expression",
-                                "msEvalExpression");
+        msSetError(MS_PARSEERR, "Failed to parse expression: %s", "msEvalExpression", tmpstr);
+        free(tmpstr);
         return MS_FALSE;
     }
-
+    free(tmpstr);
     return expresult;
     
     break;
@@ -380,7 +420,7 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
  *   int i;
  * 
  *   for(i=0; i<layer->numclasses; i++) {
- *     if(layer->class[i].status != MS_DELETE && msEvalExpression(&(layer->class[i].expression), layer->classitemindex, shape->values, layer->numitems) == MS_TRUE)
+ *     if(layer->class[i]->status != MS_DELETE && msEvalExpression(&(layer->class[i]->expression), layer->classitemindex, shape->values, layer->numitems) == MS_TRUE)
  *       return(i);
  *   }
  * 
@@ -388,7 +428,7 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
  * }
  */
 
-int msShapeGetClass(layerObj *layer, shapeObj *shape, double scale)
+int msShapeGetClass(layerObj *layer, shapeObj *shape, double scaledenom)
 {
   int i;
 
@@ -397,10 +437,10 @@ int msShapeGetClass(layerObj *layer, shapeObj *shape, double scale)
   if(layer->connectiontype == MS_INLINE) {
     if(shape->classindex < 0 || shape->classindex >= layer->numclasses) return(-1);
 
-    if(scale > 0) {  /* verify scale here */
-      if((layer->class[shape->classindex].maxscale > 0) && (scale > layer->class[shape->classindex].maxscale))
+    if(scaledenom > 0) {  /* verify scaledenom here */
+      if((layer->class[shape->classindex]->maxscaledenom > 0) && (scaledenom > layer->class[shape->classindex]->maxscaledenom))
         return(-1); /* can skip this feature */
-      if((layer->class[shape->classindex].minscale > 0) && (scale <= layer->class[shape->classindex].minscale))
+      if((layer->class[shape->classindex]->minscaledenom > 0) && (scaledenom <= layer->class[shape->classindex]->minscaledenom))
         return(-1); /* can skip this feature */
     }
 
@@ -409,14 +449,14 @@ int msShapeGetClass(layerObj *layer, shapeObj *shape, double scale)
 
   for(i=0; i<layer->numclasses; i++) {
     
-    if(scale > 0) {  /* verify scale here  */
-      if((layer->class[i].maxscale > 0) && (scale > layer->class[i].maxscale))
+    if(scaledenom > 0) {  /* verify scaledenom here  */
+      if((layer->class[i]->maxscaledenom > 0) && (scaledenom > layer->class[i]->maxscaledenom))
         continue; /* can skip this one, next class */
-      if((layer->class[i].minscale > 0) && (scale <= layer->class[i].minscale))
+      if((layer->class[i]->minscaledenom > 0) && (scaledenom <= layer->class[i]->minscaledenom))
         continue; /* can skip this one, next class */
     }
 
-    if(layer->class[i].status != MS_DELETE && msEvalExpression(&(layer->class[i].expression), layer->classitemindex, shape->values, layer->numitems) == MS_TRUE)
+    if(layer->class[i]->status != MS_DELETE && msEvalExpression(&(layer->class[i]->expression), layer->classitemindex, shape->values, layer->numitems) == MS_TRUE)
       return(i);
   }
 
@@ -428,16 +468,16 @@ char *msShapeGetAnnotation(layerObj *layer, shapeObj *shape)
   int i;
   char *tmpstr=NULL;
 
-  if(layer->class[shape->classindex].text.string) { /* test for global label first */
-    tmpstr = strdup(layer->class[shape->classindex].text.string);
-    switch(layer->class[shape->classindex].text.type) {
+  if(layer->class[shape->classindex]->text.string) { /* test for global label first */
+    tmpstr = strdup(layer->class[shape->classindex]->text.string);
+    switch(layer->class[shape->classindex]->text.type) {
     case(MS_STRING):
       break;
     case(MS_EXPRESSION):
-      tmpstr = strdup(layer->class[shape->classindex].text.string);
+      tmpstr = strdup(layer->class[shape->classindex]->text.string);
 
-      for(i=0; i<layer->class[shape->classindex].text.numitems; i++)
-	tmpstr = gsub(tmpstr, layer->class[shape->classindex].text.items[i], shape->values[layer->class[shape->classindex].text.indexes[i]]);
+      for(i=0; i<layer->class[shape->classindex]->text.numitems; i++)
+        tmpstr = msReplaceSubstring(tmpstr, layer->class[shape->classindex]->text.items[i], shape->values[layer->class[shape->classindex]->text.indexes[i]]);
       break;
     }
   } else {
@@ -478,8 +518,8 @@ double msAdjustExtent(rectObj *rect, int width, int height)
   if(cellsize <= 0) /* avoid division by zero errors */
     return(0);
 
-  ox = MS_MAX((width - (rect->maxx - rect->minx)/cellsize)/2,0); /* these were width-1 and height-1 */
-  oy = MS_MAX((height - (rect->maxy - rect->miny)/cellsize)/2,0);
+  ox = MS_MAX(((width-1) - (rect->maxx - rect->minx)/cellsize)/2,0); /* these were width-1 and height-1 */
+  oy = MS_MAX(((height-1) - (rect->maxy - rect->miny)/cellsize)/2,0);
 
   rect->minx = rect->minx - ox*cellsize;
   rect->miny = rect->miny - oy*cellsize;
@@ -539,6 +579,7 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
 {
     int nReturnVal = -1;
     char szPath[MS_MAXPATHLEN];
+
     if (img)
     {
         if( MS_DRIVER_GD(img->format) )
@@ -551,7 +592,19 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
             else
                 nReturnVal = msSaveImageGD(img->img.gd, filename, img->format);
         }
-	else if( MS_DRIVER_IMAGEMAP(img->format) )
+#ifdef USE_AGG
+        else if( MS_DRIVER_AGG(img->format) )
+        {
+            if(map != NULL && filename != NULL )
+                nReturnVal = msSaveImageAGG(img->img.gd, 
+                                           msBuildPath(szPath, map->mappath, 
+                                                       filename), 
+                                           img->format );
+            else
+                nReturnVal = msSaveImageAGG(img->img.gd, filename, img->format);
+        }
+#endif
+        else if( MS_DRIVER_IMAGEMAP(img->format) )
             nReturnVal = msSaveImageIM(img, filename, img->format);
 #ifdef USE_GDAL
         else if( MS_DRIVER_GDAL(img->format) )
@@ -605,6 +658,35 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
     return nReturnVal;
 }
 
+/*
+** Generic function to save an image to a byte array.
+** - the return value is the pointer to the byte array 
+** - size_ptr contains the number of bytes returned
+** - format: the desired output format
+**
+** The caller is responsible to free the returned array
+** The function returns NULL if the output format is not supported. 
+*/
+
+unsigned char *msSaveImageBuffer(imageObj* image, int *size_ptr, outputFormatObj *format)
+{
+    *size_ptr = 0;
+	
+	if( MS_DRIVER_GD(image->format) )
+    {
+        return msSaveImageBufferGD(image->img.gd, size_ptr, format);
+    }
+#ifdef USE_AGG
+    else if( MS_DRIVER_AGG(image->format) )
+    {
+        return msSaveImageBufferAGG(image->img.gd, size_ptr, format);
+    }
+#endif   
+	
+	msSetError(MS_MISCERR, "Unsupported image type", "msSaveImage()");
+    return NULL;
+}
+
 /**
  * Generic function to free the imageObj
  */
@@ -615,6 +697,10 @@ void msFreeImage(imageObj *image)
         if( MS_RENDERER_GD(image->format) ) {
             if( image->img.gd != NULL )
                 msFreeImageGD(image->img.gd);
+#ifdef USE_AGG
+        } else if( MS_RENDERER_AGG(image->format) ) {
+            msFreeImageAGG(image);
+#endif
         } else if( MS_RENDERER_IMAGEMAP(image->format) )
             msFreeImageIM(image);
         else if( MS_RENDERER_RAWDATA(image->format) )
@@ -669,9 +755,9 @@ int *msGetLayersIndexByGroup(mapObj *map, char *groupname, int *pnCount)
 
     for(i=0;i<map->numlayers; i++)
     {
-        if(!map->layers[i].group) /* skip it */
+        if(!GET_LAYER(map, i)->group) /* skip it */
             continue;
-        if(strcmp(groupname, map->layers[i].group) == 0)
+        if(strcmp(groupname, GET_LAYER(map, i)->group) == 0)
         {
             aiIndex[iLayer] = i;
             iLayer++;
@@ -1043,7 +1129,7 @@ char **msGetAllGroupNames(mapObj *map, int *numTok)
         for (i=0; i<nCount; i++)
         {
             layerObj *lp;
-            lp = &(map->layers[map->layerorder[i]]);
+            lp = (GET_LAYER(map, map->layerorder[i]));
 
             bFound = 0;
             if (lp->group && lp->status != MS_DELETE)
@@ -1162,7 +1248,14 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
                                 imagepath, imageurl);
         if( image != NULL && map) msImageInitGD( image, &map->imagecolor );
     }
-
+#ifdef USE_AGG
+    else if( MS_RENDERER_AGG(format) )
+    {
+        image = msImageCreateAGG(width, height, format,
+                                imagepath, imageurl);
+        if( image != NULL && map) msImageInitAGG( image, &map->imagecolor );
+    }
+#endif
     else if( MS_RENDERER_RAWDATA(format) )
     {
         if( format->imagemode != MS_IMAGEMODE_INT16
@@ -1286,6 +1379,14 @@ void  msTransformShape(shapeObj *shape, rectObj extent, double cellsize,
 
         return;
     }
+#ifdef USE_AGG
+    if (image != NULL && MS_RENDERER_AGG(image->format) )
+    {
+        msTransformShapeAGG(shape, extent, cellsize);
+
+        return;
+    }
+#endif
 
     msTransformShapeToPixel(shape, extent, cellsize);
 }
@@ -1306,6 +1407,10 @@ int msSetup()
    msThreadInit();
 #endif
 
+  /* Use MS_ERRORFILE and MS_DEBUGLEVEL env vars if set */
+  if (msDebugInitFromEnv() != MS_SUCCESS)
+    return MS_FAILURE;
+
 #ifdef USE_GD_FT
   if (gdFontCacheSetup() != 0) {
     return MS_FAILURE;
@@ -1321,13 +1426,11 @@ int msSetup()
   
 /* This is intended to be a function to cleanup anything that "hangs around"
    when all maps are destroyed, like Registered GDAL drivers, and so forth. */
-extern void lexer_cleanup(void);
-
 void msCleanup()
 {
   msForceTmpFileBase( NULL );
   msConnPoolFinalCleanup();
-  lexer_cleanup();
+  msyylex_destroy();
 
 #ifdef USE_OGR
   msOGRCleanup();
@@ -1342,12 +1445,20 @@ void msCleanup()
 #if defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
   msHTTPCleanup();
 #endif
+
+#ifdef USE_GD_FT
+  gdFontCacheShutdown(); 
+#endif
+
 #ifdef USE_GEOS
   msGEOSCleanup();
 #endif
   msIO_Cleanup();
 
   msResetErrorList();
+
+  /* Close/cleanup log/debug output. Keep this at the very end. */
+  msDebugCleanup();
 }
 
 /************************************************************************/
@@ -1406,3 +1517,27 @@ int msAlphaBlend (int dst, int src)
 /* -------------------------------------------------------------------- */
     return ((alpha << 24) + (red << 16) + (green << 8) + blue);
 }
+
+/*
+ RFC 24: check if the parent pointer is NULL and raise an error otherwise
+*/
+int msCheckParentPointer(void* p, char *objname) {
+    char* fmt="The %s parent object is null";
+    char* msg=NULL;
+    if (p == NULL) {
+        if(objname != NULL) {
+            msg=malloc( sizeof(char) * ( ( strlen(fmt)+strlen(objname) ) ) );
+            if(msg == NULL) {
+                msg="A required parent object is null";
+            } else {
+                sprintf(msg, "The %s parent object is null", objname);
+            }
+        } else {
+            msg="A required parent object is null";
+        }
+        msSetError(MS_NULLPARENTERR, msg, "");
+        return MS_FAILURE;
+    }
+    return MS_SUCCESS;
+}
+
