@@ -24,84 +24,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.71.4.1  2007/01/19 03:46:45  sdlime
- * Fixed requestResponseCRSs output to create multiple tags instead of one with a space delimited list. (bug 1148)
- *
- * Revision 1.71  2005/09/23 04:37:32  frank
- * added preliminary WCS INTERPOLATION support
- *
- * Revision 1.70  2005/06/27 19:58:30  frank
- * Fixed some small WCS params related memory leaks.
- *
- * Revision 1.69  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.68  2005/05/12 18:14:52  frank
- * another missing WCS exception - if msSaveImage() fails
- *
- * Revision 1.67  2005/05/12 17:41:49  frank
- * If msDrawRasterLayerLow() fails, ensure that the error message is posted as a WCS exception.
- *
- * Revision 1.66  2005/02/18 18:14:11  dan
- * New patch to fix url strings broken by previous patch to bug 1238
- *
- * Revision 1.65  2005/02/18 16:09:18  hobu
- * patch up msvc errors related to the comment commit
- *
- * Revision 1.64  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.63  2005/02/12 04:41:32  sdlime
- * Changed %g to %.15g for WCS output (bug 1232).
- *
- * Revision 1.62  2004/11/29 14:36:22  sdlime
- * Last of the typos for bug 954, also use ows_service_onlineresource if using WMS metadata.
- *
- * Revision 1.61  2004/11/29 04:30:14  sdlime
- * Fixed typo in ResponsibleParty, positionName tag.
- *
- * Revision 1.60  2004/11/29 04:09:23  sdlime
- * Fixed bug 954, added ResponsibleParty output to the service block. Supports new wcs_responsibleparty_* metadata tags or tries to use WMS contact metadata.
- *
- * Revision 1.59  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.58  2004/11/08 00:33:00  frank
- * Fixed support for non-square pixels in WCS. (bug 1014)
- *
- * Revision 1.57  2004/11/04 21:47:07  frank
- * Removed unused variable.
- *
- * Revision 1.56  2004/10/29 22:48:03  assefa
- * Use of metadata ows_schema_location (Bug 1013).
- *
- * Revision 1.55  2004/10/28 01:27:32  frank
- * Generate an error if a GetCoverage lacks a COVERAGE specification.
- * Must nicer than just segfaulting.
- *
- * Revision 1.54  2004/10/26 15:20:19  julien
- * msOWSPrintURLType: metadata:Link had a %s missing in it's format. (Bug 944)
- *
- * Revision 1.53  2004/10/26 04:26:28  sdlime
- * Fixed bug 989, metadataLink for the CoverageOffering section of the WCS capabilities.
- *
- * Revision 1.52  2004/10/25 17:30:38  julien
- * Print function for OGC URLs components. msOWSPrintURLType() (Bug 944)
- *
- * Revision 1.51  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ *****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "maperror.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapwcs.c 6405 2007-07-27 17:33:25Z warmerdam $")
+MS_CVSID("$Id: mapwcs.c 6637 2007-08-17 21:30:26Z warmerdam $")
 
 #ifdef USE_WCS_SVR
 
@@ -166,7 +95,7 @@ static char *msWCSConvertRangeSetToString(const char *value) {
   char buf1[128], *buf2=NULL;
 
   if(strchr(value, '/')) { /* value is min/max/res */
-    tokens = split(value, '/', &numtokens);
+    tokens = msStringSplit(value, '/', &numtokens);
     if(tokens==NULL || numtokens != 3) {
       msFreeCharArray(tokens, numtokens);
       return NULL; /* not a set of equally spaced intervals */
@@ -182,7 +111,7 @@ static char *msWCSConvertRangeSetToString(const char *value) {
         snprintf(buf1, 128, "%g", val);     
       else
         snprintf(buf1, 128, ",%g", val);
-      buf2 = strcatalloc(buf2, buf1);
+      buf2 = msStringConcatenate(buf2, buf1);
     }
    
     return buf2;
@@ -193,26 +122,36 @@ static char *msWCSConvertRangeSetToString(const char *value) {
 /************************************************************************/
 /*                           msWCSException()                           */
 /************************************************************************/
-static int msWCSException(mapObj *map, const char *version) 
+static int msWCSException(mapObj *map, const char *version, const char *code, const char *locator) 
 {
   char *pszEncodedVal = NULL;
   /* msIO_printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10); */
   msIO_printf("Content-type: text/xml%c%c",10,10);
 
-  /* TODO: see WCS specific exception schema (Appendix 6), this is a copy of Dan's for WFS */
+  msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), NULL, "wcs_encoding", OWS_NOERR, "<?xml version='1.0' encoding=\"%s\" ?>\n", "ISO-8859-1");
 
-  msIO_printf("<ServiceExceptionReport\n");
+  msIO_printf("<ServiceExceptionReport version=\"1.2.0\"\n");
   msIO_printf("xmlns=\"http://www.opengis.net/ogc\" "); 
   msIO_printf("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "); 
   pszEncodedVal = msEncodeHTMLEntities(msOWSGetSchemasLocation(map));
   msIO_printf("xsi:schemaLocation=\"http://www.opengis.net/ogc %s/wms/1.1.1/OGC-exception.xsd\">\n", 
               pszEncodedVal);
   msFree(pszEncodedVal);
-  msIO_printf("  <ServiceException>\n");
+  msIO_printf("  <ServiceException");
+  if (code) {
+    msIO_printf(" code=\"%s\"", code);
+  }
+  if (locator) {
+    msIO_printf(" locator=\"%s\"", locator);
+  }
+  msIO_printf(">");
   msWriteErrorXML(stdout);
   msIO_printf("  </ServiceException>\n");
   msIO_printf("</ServiceExceptionReport>\n");
 
+  /* clear error since we have already reported it */
+  msResetErrorList();
+
   return MS_FAILURE;
 }
 
@@ -391,10 +330,10 @@ static int msWCSParseRequest(cgiRequestObj *request, wcsParamsObj *params, mapOb
          char **tokens;
          int n;
           
-         tokens = split(request->ParamValues[i], ',', &n);
+         tokens = msStringSplit(request->ParamValues[i], ',', &n);
          if(tokens==NULL || n != 4) {
            msSetError(MS_WMSERR, "Wrong number of arguments for BBOX.", "msWCSParseRequest()");
-           return msWCSException(map, params->version);
+           return msWCSException(map, params->version, "InvalidParameterValue", "bbox");
          }
          params->bbox.minx = atof(tokens[0]);
          params->bbox.miny = atof(tokens[1]);
@@ -585,7 +524,7 @@ static int msWCSGetCapabilities_Capability(mapObj *map, wcsParamsObj *params, cg
 
    /* we need this server's onlineresource for the request section */
   if((script_url=msOWSGetOnlineResource(map, "CO", "onlineresource", req)) == NULL || (script_url_encoded = msEncodeHTMLEntities(script_url)) == NULL) {
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, NULL, NULL);
   }
 
   /* start the Capabilty section, only need the full start tag if this is the only section requested */
@@ -692,7 +631,7 @@ static int msWCSGetCapabilities_ContentMetadata(mapObj *map, wcsParamsObj *param
            "   xsi:schemaLocation=\"http://www.opengis.net/wcs %s/wcs/%s/wcsCapabilities.xsd\">\n", params->version, msOWSGetSchemasLocation(map), params->version); 
 
   for(i=0; i<map->numlayers; i++)
-    msWCSGetCapabilities_CoverageOfferingBrief(&(map->layers[i]), params);
+    msWCSGetCapabilities_CoverageOfferingBrief((GET_LAYER(map, i)), params);
 
   /* done */
   msIO_printf("</ContentMetadata>\n");
@@ -791,7 +730,7 @@ static int msWCSDescribeCoverage_AxisDescription(layerObj *layer, char *name)
     char **tokens;
     int numtokens;
 
-     tokens = split(value, '/', &numtokens);
+     tokens = msStringSplit(value, '/', &numtokens);
      if(tokens && numtokens > 0) {
        msIO_printf("            <interval>\n");
        if(numtokens >= 1) msIO_printf("            <min>%s</min>\n", tokens[0]); /* TODO: handle closure */
@@ -825,7 +764,11 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   coverageMetadataObj cm;
   int i, status;
 
+  if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+	return MS_FAILURE;
+
   if(!msWCSIsLayerSupported(layer)) return MS_SUCCESS; /* not an error, this layer cannot be served via WCS */
+  
 
   status = msWCSGetCoverageMetadata(layer, &cm);
   if(status != MS_SUCCESS) return MS_FAILURE;
@@ -908,6 +851,7 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
 
     /* TimePosition (should support a value AUTO, then we could mine positions from the timeitem) */
     msOWSPrintEncodeMetadataList(stdout, &(layer->metadata), "COM", "timeposition", NULL, NULL, "        <gml:timePosition>%s</gml:timePosition>\n", NULL);    
+
     /* TODO:  add TimePeriod (only one per layer)  */
 
     msIO_printf("      </temporalDomain>\n");
@@ -928,12 +872,12 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   
   /* compound range sets */
   if((value = msOWSLookupMetadata(&(layer->metadata), "COM", "rangeset_axes")) != NULL) {
-    tokens = split(value, ',', &numtokens);
-    if(tokens && numtokens > 0) {
-      for(i=0; i<numtokens; i++)
-       msWCSDescribeCoverage_AxisDescription(layer, tokens[i]);
-      msFreeCharArray(tokens, numtokens);
-    }
+     tokens = msStringSplit(value, ',', &numtokens);
+     if(tokens && numtokens > 0) {
+       for(i=0; i<numtokens; i++)
+         msWCSDescribeCoverage_AxisDescription(layer, tokens[i]);
+       msFreeCharArray(tokens, numtokens);
+     }
   }
   
   msIO_printf("      </RangeSet>\n");
@@ -944,14 +888,14 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   
   /* requestResposeCRSs: check the layer metadata/projection, and then the map metadata/projection if necessary (should never get to the error message) */
   if((value = msOWSGetEPSGProj(&(layer->projection), &(layer->metadata), "COM", MS_FALSE)) != NULL) {
-    tokens = split(value, ' ', &numtokens);
+    tokens = msStringSplit(value, ' ', &numtokens);
     if(tokens && numtokens > 0) {
       for(i=0; i<numtokens; i++)
         msIO_printf("      <requestResponseCRSs>%s</requestResponseCRSs>\n", tokens[i]);
       msFreeCharArray(tokens, numtokens);
     }
   } else if((value = msOWSGetEPSGProj(&(layer->map->projection), &(layer->map->web.metadata), "COM", MS_FALSE)) != NULL) {
-    tokens = split(value, ' ', &numtokens);
+    tokens = msStringSplit(value, ' ', &numtokens);
     if(tokens && numtokens > 0) {
       for(i=0; i<numtokens; i++)
         msIO_printf("      <requestResponseCRSs>%s</requestResponseCRSs>\n", tokens[i]);
@@ -975,7 +919,16 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   msIO_printf("    <supportedFormats");
   msOWSPrintEncodeMetadata(stdout, &(layer->metadata), "COM", "nativeformat", OWS_NOERR, " nativeFormat=\"%s\"", NULL);
   msIO_printf(">\n");
-  msOWSPrintEncodeMetadata(stdout, &(layer->metadata), "COM", "formats", OWS_NOERR, "      <formats>%s</formats>\n", NULL);
+
+  if( (value = msOWSGetEncodeMetadata( &(layer->metadata), "COM", "formats",
+                                       "GTiff" )) != NULL ) {
+    tokens = msStringSplit(value, ' ', &numtokens);
+    if(tokens && numtokens > 0) {
+      for(i=0; i<numtokens; i++)
+        msIO_printf("      <formats>%s</formats>\n", tokens[i]);
+      msFreeCharArray(tokens, numtokens);
+    }
+  }
   msIO_printf("    </supportedFormats>\n");
   
   msIO_printf("    <supportedInterpolations default=\"nearest neighbour\">\n");
@@ -998,13 +951,25 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
 static int msWCSDescribeCoverage(mapObj *map, wcsParamsObj *params)
 {
   int i,j;
-  
+ 
+  if(params->coverages) { /* use the list */
+    for( j = 0; params->coverages[j]; j++ ) {
+      i = msGetLayerIndex(map, params->coverages[j]);
+      if(i == -1) {
+        msSetError( MS_WCSERR,
+                "COVERAGE %s cannot be opened / does not exist",
+                "msWCSDescribeCoverage()", params->coverages[j]);
+        return msWCSException(map, params->coverages[j], "CoverageNotDefined", "coverage");
+      }
+    }
+  }
+ 
   /* printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10); */
   msIO_printf("Content-type: text/xml%c%c",10,10);
 
   /* print common capability elements  */
   msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), NULL, "wcs_encoding", OWS_NOERR, "<?xml version='1.0' encoding=\"%s\" ?>\n", "ISO-8859-1");
-  
+
   /* start the DescribeCoverage section */
   msIO_printf("<CoverageDescription\n"
          "   version=\"%s\" \n"
@@ -1018,12 +983,11 @@ static int msWCSDescribeCoverage(mapObj *map, wcsParamsObj *params)
   if(params->coverages) { /* use the list */
     for( j = 0; params->coverages[j]; j++ ) {
       i = msGetLayerIndex(map, params->coverages[j]);
-      if(i == -1) continue; /* skip this layer, could/should generate an error */
-      msWCSDescribeCoverage_CoverageOffering(&(map->layers[i]), params);
+      msWCSDescribeCoverage_CoverageOffering((GET_LAYER(map, i)), params);
     }
   } else { /* return all layers */
     for(i=0; i<map->numlayers; i++)
-      msWCSDescribeCoverage_CoverageOffering(&(map->layers[i]), params);
+      msWCSDescribeCoverage_CoverageOffering((GET_LAYER(map, i)), params);
   }
   
   /* done */
@@ -1051,28 +1015,28 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
   /* make sure all required parameters are available (at least the easy ones) */
   if(!params->crs) {
     msSetError( MS_WCSERR, "Required parameter CRS was not supplied.", "msWCSGetCoverage()");
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, "MissingParameterValue", "crs");
   }
 
   if( params->coverages == NULL || params->coverages[0] == NULL ) {
     msSetError( MS_WCSERR, 
                 "Required parameter COVERAGE was not supplied.", 
                 "msWCSGetCoverage()");
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, "MissingParameterValue", "coverage");
   }
   
   /* find the layer we are working with.  */
   lp = NULL;
   for(i=0; i<map->numlayers; i++) {
-    if( EQUAL(map->layers[i].name, params->coverages[0]) ) {
-      lp = map->layers + i;
+    if( EQUAL(GET_LAYER(map, i)->name, params->coverages[0]) ) {
+      lp = GET_LAYER(map, i);
       break;
     }
   }
 
   if(lp == NULL) {
     msSetError( MS_WCSERR, "COVERAGE=%s not found, not in supported layer list.", "msWCSGetCoverage()", params->coverages[0] );
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, "InvalidParameterValue", "coverage");
   }
 
   /* make sure the layer is on */
@@ -1099,14 +1063,14 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
       sprintf(buffer, "init=epsg:%.20s", crs_to_use+5);
 
       if (msLoadProjectionString(&(map->projection), buffer) != 0)
-        return msWCSException( map, params->version );
+        return msWCSException( map, params->version, NULL, NULL);
         
       iUnits = GetMapserverUnitUsingProj(&(map->projection));
       if (iUnits != -1)
         map->units = iUnits;
     } else {  /* should we support WMS style AUTO: projections? (not for now) */
       msSetError(MS_WMSERR, "Unsupported SRS namespace (only EPSG currently supported).", "msWCSGetCoverage()");
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, "InvalidParameterValue", "srs");
     }
   }
 
@@ -1120,42 +1084,42 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
     /* check format of TIME parameter */
     if(strchr(params->time, ',')) {
       msSetError( MS_WCSERR, "Temporal lists are not supported, only individual values.", "msWCSGetCoverage()" );
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, "InvalidParameterValue", "time");
     }
     if(strchr(params->time, '/')) {
       msSetError( MS_WCSERR, "Temporal ranges are not supported, only individual values.", "msWCSGetCoverage()" );
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, "InvalidParameterValue", "time");
     }
       
     /* TODO: will need to expand this check if a time period is supported */
     value = msOWSLookupMetadata(&(lp->metadata), "COM", "timeposition");
     if(!value) {
       msSetError( MS_WCSERR, "The coverage does not support temporal subsetting.", "msWCSGetCoverage()" );
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, "InvalidParameterValue", "time");
     }
     if(!strstr(value, params->time)) { /* this is likely too simple a test */
       msSetError( MS_WCSERR, "The coverage does not have a time position of %s.", "msWCSGetCoverage()", params->time );
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, "InvalidParameterValue", "time");
     }
       
     /* make sure layer is tiled appropriately */
     if(!lp->tileindex) {
       msSetError( MS_WCSERR, "Underlying layer is not tiled, unable to do temporal subsetting.", "msWCSGetCoverage()" );
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, NULL, NULL);
     }
     tli = msGetLayerIndex(map, lp->tileindex);
     if(tli == -1) {
       msSetError( MS_WCSERR, "Underlying layer does not use appropriate tiling mechanism.", "msWCSGetCoverage()" );
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, NULL, NULL);
     }
 
-    tlp = &(map->layers[tli]);
+    tlp = (GET_LAYER(map, tli));
 
     /* make sure there is enough information to filter */
     value = msOWSLookupMetadata(&(lp->metadata), "COM", "timeitem");
     if(!tlp->filteritem && !value) {
       msSetError( MS_WCSERR, "Not enough information available to filter.", "msWCSGetCoverage()" );
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, NULL, NULL);
     }
       
     /* override filteritem if specified in metadata */
@@ -1177,20 +1141,20 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
     char tag[100];
     const char *rangeitem;
 
-    tokens = split(value, ',', &numtokens);
+    tokens = msStringSplit(value, ',', &numtokens);
     for(i=0; i<numtokens; i++) {
       if((value = msWCSGetRequestParameter(request, tokens[i])) == NULL) continue; /* next rangeset parameter */
        
       if(msWCSValidateRangeSetParam(&(lp->metadata), tokens[i], "COM", value) != MS_SUCCESS) {
-        msSetError( MS_WCSERR, "Error specifying \"%s\" paramter value(s).", "msWCSGetCoverage()", tokens[i]);
-        return msWCSException(map, params->version);
+        msSetError( MS_WCSERR, "Error specifying \"%s\" parameter value(s).", "msWCSGetCoverage()", tokens[i]);
+        return msWCSException(map, params->version, NULL, NULL);
       }
        
       /* xxxxx_rangeitem tells us how to subset */
       snprintf(tag, 100, "%s_rangeitem", tokens[i]);
       if((rangeitem = msOWSLookupMetadata(&(lp->metadata), "COM", tag)) == NULL) {
         msSetError( MS_WCSERR, "Missing required metadata element \"%s\", unable to process %s=%s.", "msWCSGetCoverage()", tag, tokens[i], value);
-        return msWCSException(map, params->version);
+        return msWCSException(map, params->version, NULL, NULL);
       }
          
       if(strcasecmp(rangeitem, "_bands") == 0) { /* special case, subset bands */
@@ -1198,14 +1162,14 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
            
         if(!bandlist) {
           msSetError( MS_WCSERR, "Error specifying \"%s\" paramter value(s).", "msWCSGetCoverage()", tokens[i]);
-          return msWCSException(map, params->version);
+          return msWCSException(map, params->version, NULL, NULL);
         }          
       } else if(strcasecmp(rangeitem, "_pixels") == 0) { /* special case, subset pixels */
         msSetError( MS_WCSERR, "Arbitrary range sets based on pixel values are not yet supported.", "msWCSGetCoverage()" );
-        return msWCSException(map, params->version);
+        return msWCSException(map, params->version, NULL, NULL);
       } else {
         msSetError( MS_WCSERR, "Arbitrary range sets based on tile (i.e. image) attributes are not yet supported.", "msWCSGetCoverage()" );
-        return msWCSException(map, params->version);
+        return msWCSException(map, params->version, NULL, NULL);
       }
     }
        
@@ -1237,7 +1201,7 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
   /* are we still underspecified?  */
   if( params->width == 0 || params->height == 0 || params->resx == 0.0 || params->resy == 0.0 ) {
     msSetError( MS_WCSERR, "A non-zero RESX/RESY or WIDTH/HEIGHT is required but neither was provided.", "msWCSGetCoverage()" );
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, "MissingParameterValue", "width/height/resx/resy");
   }
 
   map->cellsize = params->resx;
@@ -1264,13 +1228,19 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
                       "INTERPOLATION=%s specifies an unsupported interpolation method.",
                       "msWCSGetCoverage()",
                       params->interpolation );
-          return msWCSException(map, params->version);
+          return msWCSException(map, params->version, "InvalidParameterValue", "interpolation");
       }
   }
    
   /* apply region and size to map object.  */
   map->width = params->width;
   map->height = params->height;
+
+  /* adjust OWS BBOX to MapServer's pixel model */
+  params->bbox.minx += params->resx*0.5;
+  params->bbox.miny += params->resy*0.5;
+  params->bbox.maxx -= params->resx*0.5;
+  params->bbox.maxy -= params->resy*0.5;
   map->extent = params->bbox;
  
   map->cellsize = params->resx; /* pick one, MapServer only supports square cells (what about msAdjustExtent here!) */
@@ -1286,11 +1256,11 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
   /* check and make sure there is a format, and that it's valid (TODO: make sure in the layer metadata) */
   if(!params->format) {
     msSetError( MS_WCSERR,  "Missing required FORMAT parameter.", "msWCSGetCoverage()" );
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, "MissingParameterValue", "format");
   }
   if(msGetOutputFormatIndex(map,params->format) == -1) {
     msSetError( MS_WCSERR,  "Unrecognized value for the FORMAT parameter.", "msWCSGetCoverage()" );
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, "InvalidParameterValue", "format");
   }
 
   /* create a temporary outputformat (we likely will need to tweak parts) */
@@ -1305,27 +1275,32 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
   }
  
   msLayerSetProcessingKey(lp, "BANDS", bandlist);
-  sprintf(numbands, "%d", countChars(bandlist, ',')+1);
+  sprintf(numbands, "%d", msCountChars(bandlist, ',')+1);
   msSetOutputFormatOption(map->outputformat, "BAND_COUNT", numbands);
                
   /* create the image object  */
   if(!map->outputformat) {
     msSetError(MS_WCSERR, "The map outputformat is missing!", "msWCSGetCoverage()");
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, NULL, NULL);
   } else if( MS_RENDERER_GD(map->outputformat) ) {
     image = msImageCreateGD(map->width, map->height, map->outputformat, map->web.imagepath, map->web.imageurl);        
     if( image != NULL ) msImageInitGD( image, &map->imagecolor );
+#ifdef USE_AGG
+  } else if( MS_RENDERER_AGG(map->outputformat) ) {
+    image = msImageCreateAGG(map->width, map->height, map->outputformat, map->web.imagepath, map->web.imageurl);        
+    if( image != NULL ) msImageInitAGG( image, &map->imagecolor );
+#endif
   } else if( MS_RENDERER_RAWDATA(map->outputformat) ) {
     image = msImageCreate(map->width, map->height, map->outputformat, map->web.imagepath, map->web.imageurl, map);
   } else {
     msSetError(MS_WCSERR, "Map outputformat not supported for WCS!", "msWCSGetCoverage()");
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, NULL, NULL);
   }
 
   /* Actually produce the "grid". */
   status = msDrawRasterLayerLow( map, lp, image );
   if( status != MS_SUCCESS ) {
-    return msWCSException(map, params->version);
+    return msWCSException(map, params->version, NULL, NULL);
   }
     
   /* Emit back to client. */
@@ -1337,7 +1312,7 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
       /* unfortunately, the image content type will have already been sent
          but that is hard for us to avoid.  The main error that could happen
          here is a misconfigured tmp directory or running out of space. */
-      return msWCSException(map, params->version);
+      return msWCSException(map, params->version, NULL, NULL);
   }
 
   /* Cleanup */
@@ -1392,7 +1367,7 @@ int msWCSDispatch(mapObj *map, cgiRequestObj *request)
   /* version is optional, but we do set a default value of 1.0.0, make sure request isn't for something different */
   if(strcmp(params->version, "1.0.0") != 0) {
     msSetError(MS_WCSERR, "WCS Server does not support VERSION %s.", "msWCSDispatch()", params->version);
-    msWCSException(map, params->version);
+    msWCSException(map, params->version, "InvalidParameterValue", "version");
   
     msWCSFreeParams(params); /* clean up */
     free(params);
@@ -1425,6 +1400,9 @@ int msWCSDispatch(mapObj *map, cgiRequestObj *request)
 #ifdef USE_WCS_SVR
 static int msWCSGetCoverageMetadata( layerObj *layer, coverageMetadataObj *cm )
 {
+  if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+	return MS_FAILURE;
+
   /* get information that is "data" independent */
   if((cm->srs = msOWSGetEPSGProj(&(layer->projection), &(layer->metadata), "COM", MS_TRUE)) == NULL) {
     if((cm->srs = msOWSGetEPSGProj(&(layer->map->projection), &(layer->map->web.metadata), "COM", MS_TRUE)) == NULL) {
@@ -1455,7 +1433,7 @@ static int msWCSGetCoverageMetadata( layerObj *layer, coverageMetadataObj *cm )
       char **tokens;
       int n;
             
-      tokens = split(value, ' ', &n);
+      tokens = msStringSplit(value, ' ', &n);
       if( tokens == NULL || n != 2 ) {
         msSetError( MS_WCSERR, "Wrong number of arguments for wcs|ows_resolution metadata.", "msWCSGetCoverageMetadata()");
         msFreeCharArray( tokens, n );
@@ -1473,7 +1451,7 @@ static int msWCSGetCoverageMetadata( layerObj *layer, coverageMetadataObj *cm )
       char **tokens;
       int n;
             
-      tokens = split(value, ' ', &n);
+      tokens = msStringSplit(value, ' ', &n);
       if( tokens == NULL || n != 2 ) {
         msSetError( MS_WCSERR, "Wrong number of arguments for wcs|ows_size metadata.", "msWCSGetCoverageDomain()");
         msFreeCharArray( tokens, n );
