@@ -24,22 +24,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.11  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.10  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "maptree.h"
 #include <string.h>
 
-MS_CVSID("$Id: shptree.c 4695 2005-06-14 16:03:36Z dan $")
+MS_CVSID("$Id: shptree.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 char* AddFileSuffix ( const char * Filename, const char * Suffix ) {
   char	*pszFullname, *pszBasename;
